// Function to generate the mean files from an MDRF.

// The method is straight-forward and open to tinkering.
// Initial means are found using all values.  A window is set using the standard deviation
// from the mean and new means are calculated.  The process is repeated several times with
// a shrinking window.

// jyh, 11/05

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

main(int argc, char **argv)
{

  FILE *mdrf_file, *mean_file, *num_evt_file;
  char *path_in, *path_out, filename[256];
  int mdrf, num_pmts, num_events, curr_data;
  int i,j,k,l,n;
  short int *pix_header, *header, *data;
  float *total_mean, mean[9], std[9];
  double sum, sum_sq;
  int count;  
  int num_stds;
  float std_set[4];

  setlinebuf(stdout);

  // Check the number of input arguments
  // The input arguments should include the input raw mdrf file, the name of the output mean file, and the mdrf dimension (typically 79).
  if (argc != 4){
    printf("usage:prog_name raw_mdrf mean_file_out mdrf_dimension \n");
    exit(0);
  }


  path_in = argv[1];
  path_out = argv[2];
  mdrf = atoi(argv[3]);

  // SET THE NUMBER OF PMTS TO CONSTANT
  num_pmts = 9;
  
  // OPEN RAW MDRF  FILE
  sprintf(filename,"%s",path_in);
  mdrf_file = fopen(filename, "r");
  if (mdrf_file <= 0){
    printf("couldn't open %s\n", path_in);
    exit(0);
  }
  else printf(" opened the raw mdrf file: %s\n",path_in);

  // OPEN OUTPUT MEAN FILE
  sprintf(filename,"%s",path_out);
  mean_file = fopen(filename, "w");
  if (mean_file <= 0){
    printf("couldn't open %s\n", path_out);
    exit(0);
  }
  else printf(" opened the output mean file: %s\n",path_out);

  num_evt_file = fopen("../Marq_Data/Num_Evt_File","w");

  // READ IN THE 18 BIT HEADER
  header = (short int*)malloc(2*num_pmts*sizeof(short int));
  fread(header,sizeof(short int),2*num_pmts,mdrf_file);
  for (n = 0; n < 2*num_pmts; n++) header[n] = ntohs(header[n]);

  // ALLOCATE MEMORY FOR FINAL MEAN FILE
  total_mean = (float *)malloc(num_pmts*mdrf*mdrf*sizeof(float));
  for (i = 0; i < num_pmts*mdrf*mdrf; i++) total_mean[i] = 0.0;

  // SET STD WINDOW VALUES
  std_set[0] = 3.5;  // 4.0
  std_set[1] = 3.0;  // 3.5
  std_set[2] = 2.0;    // 2.5
  std_set[3] = 1.5;  // 2.0
  num_stds = 4;

  // READ IN THE MDRF PIXEL BY PIXEL
  for (i = 0; i < mdrf*mdrf; i++){
    pix_header = (short int*)malloc(num_pmts*sizeof(short int));    
    fread(pix_header,sizeof(short int),num_pmts,mdrf_file);
    for (n = 0; n < num_pmts; n++) pix_header[n] = ntohs(pix_header[n]);
    num_events = pix_header[3]*1000+pix_header[4];
    fwrite(&num_events,sizeof(int),1,num_evt_file);
    data = (short int*)malloc(num_events*num_pmts*sizeof(short int));
    fread(data,sizeof(short int),num_events*num_pmts,mdrf_file);
    for (n = 0; n < num_events*num_pmts; n++) data[n] = ntohs(data[n]);

    // CALCULATE AN INITIAL MEAN AND STD DEVIATION
    for (k = 0; k < num_pmts; k++){
      sum = 0.0;
      sum_sq = 0.0;
      for (l = 0; l < num_events; l++){
	curr_data = l*num_pmts+k;
	sum = sum + (double)data[curr_data];
	sum_sq = sum_sq + (double)(data[curr_data]*data[curr_data]);
      }
      mean[k] = (float)sum/(float)num_events;
      std[k] = sqrt((sum_sq - (double)(num_events*mean[k]*mean[k]))/(double)(num_events-1));
    }

    // CALCULATE MEANS USING A SHRINKING WINDOW
    for (j = 0; j < num_stds; j++){
      for (k = 0; k < num_pmts; k++){
	count = 0;
	sum = 0.0;
	sum_sq = 0.0;
	for (l = 0; l < num_events; l++){
	  curr_data = l*num_pmts+k;
	  if ((float)data[curr_data] < (mean[k] + std_set[j]*std[k]) & (float)data[curr_data] > (mean[k]-std_set[j]*std[k])){
	    sum = sum + (double)data[curr_data];
	    sum_sq = sum_sq + (double)(data[curr_data]*data[curr_data]);
	    count++;
	  }
	}
	mean[k] = (double)sum/(double)count;
	std[k] = sqrt((sum_sq - (double)(count*mean[k]*mean[k]))/(double)(count-1));
      }
    }

    // WRITE OUT THE FINAL CALCULATED MEAN
    for (j = 0; j < num_pmts; j++) total_mean[j*mdrf*mdrf + i] = mean[j];

    free(data);free(pix_header);
    
    if(i%500 == 0) printf("i = %d\n",i);

  }
  fclose(num_evt_file);
  for (n = 0; n < num_pmts*mdrf*mdrf; n++) *(long *)&total_mean[n] = htonl(*(long *)&total_mean[n]);
  fwrite(total_mean,sizeof(float),num_pmts*mdrf*mdrf,mean_file);
  free(total_mean);free(header);
}
