// This code will read through the MDRF file and write out files containing a projection image 
// of the MDRF, the positions of all events in the MDRF, and the likelihood value for each event in the MDRF.
// Subroutines:
//   Gen_LW_pos_est_int.c, ML_LW_zero_pad_int.c

// jyh, 11/05

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

main(int argc, char **argv)
{

	int  det_size;
		/*  
			det_size --> number of MDRF steps across face (typically 78 or 79)
		*/
	
	FILE *meanfile, *logfactfile, *datafile, *imagefile, *likefile, *posfile, *gainfile;
	char *im_path, *mean_path, *path;
	long int *mean_vals, *log_mean_vals, *mean_vals_all, *log_factorial_vals, *likedata;
	float *mean_vals_temp, *log_factorial_vals_temp;
	short int *data_vals, *header, *frontheader, *gain_vals;
	short int *posdata;
	int *im_vals, *im_vals_total;
	int num_pmts, det_zp, size_tester, det_diff;
	int i,j,k,count,num_log_factorial_vals,n;
	long int scale;
	short int num_events;	
	char filename[256];

	setlinebuf(stdout);
	
	// Check the number of input arguments
	// The input arguments should include a path to the likelihood windowing files, a mean file, a log-factorial file, a data
	// file, a gain file, files for the projection images, positions, and likelihood values, and the MDRF dimension.
	if (argc != 10){
		printf("usage:prog_name path meanfile log_fact_file datafile_in gain_file imagefile_out likefile_out posfile_out det_size \n");
		exit(0);
	}

	path = argv[1];
	mean_path = argv[2];
	im_path = argv[4];
	det_size = atoi(argv[9]);

	// OPEN ORIGINAL MEAN FILE
	sprintf(filename,"%s%s",path,argv[2]);
	meanfile = fopen(filename, "r");
	if (meanfile <= 0){
		printf("couldn't open %s\n", argv[2]);
		exit(0);
	}
	else printf(" opened the mean file: %s\n",argv[2]);
	
	// OPEN LOG FACTORIAL FILE
	sprintf(filename,"%s%s",path,argv[3]);
	logfactfile = fopen(filename, "r");
	if (logfactfile <= 0){
		printf("couldn't open %s\n", argv[3]);
		exit(0);
	}
	else printf(" opened the log factorial file: %s\n",argv[3]);
	
	// OPEN DATA FILE
	sprintf(filename,"%s%s",path,argv[4]);
       	datafile = fopen(filename,"r");
	if (datafile <= 0){
		printf("couldn't open %s\n", argv[4]);
		exit(0);
	}
	else printf(" opened the data file: %s\n",argv[4]);
	
	// OPEN GAIN FILE
	sprintf(filename,"%s%s",path,argv[5]);
       	gainfile = fopen(filename,"r");
	if (gainfile <= 0){
		printf("couldn't open %s\n", argv[5]);
		exit(0);
	}
	else printf(" opened the gain file: %s\n",argv[5]);


	// OPEN OUTPUT IMAGE FILE
	sprintf(filename,"%s%s",path,argv[6]);
	imagefile = fopen(filename, "w");
	if (imagefile <= 0){
		printf("couldn't open %s\n", argv[6]);
		exit(0);
	}
	else printf(" opened the image output file: %s\n",argv[6]);

	// OPEN OUTPUT LIKELIHOOD FILE
	sprintf(filename,"%s%s",path,argv[7]);
	likefile = fopen(filename,"w");
	if (likefile <= 0){
		printf("couldn't open %s\n", argv[7]);
		exit(0);
	}
	else printf(" opened the likelihood output file: %s\n",argv[7]);

	// OPEN OUTPUT POSITION FILE
	sprintf(filename,"%s%s",path,argv[8]);
	posfile = fopen(filename,"w");
	if (posfile <= 0){
		printf("couldn't open %s\n", argv[8]);
		exit(0);
	}
	else printf(" opened the position output file: %s\n",argv[8]);

	// DO SOME HOUSEKEEPING.  DECLARE SCALE, NUMBER OF PMTS, AND FIND THE NEW DET SIZE,
	// and DIFFERENCE IN DETECTOR SIZES
	scale = 65536;
	num_pmts = 9;
	size_tester = 1;
	det_zp = 1;
	count = 1;
	while (size_tester != 0){
		det_zp = 2*det_zp;
		if (det_size/det_zp == 0){
			size_tester = 0;
		}
		count++;
	}
	num_log_factorial_vals = 2048;
	
	if ((det_zp - det_size)%2 == 0)
		det_diff = (det_zp - det_size)/2;
	else
		det_diff = (det_zp - det_size + 1)/2;
		
	// READ IN THE ORIGINAL MEAN FILE
	mean_vals_temp = (float *)malloc(det_size*det_size*num_pmts*sizeof(float));
	fread(mean_vals_temp,sizeof(float),det_size*det_size*num_pmts,meanfile);
	for (n = 0; n < det_size*det_size*num_pmts; n++) *(long *)&mean_vals_temp[n] = ntohl(*(long *)&mean_vals_temp[n]);
	fclose(meanfile);
	
	// READ IN THE GAIN FILE
	gain_vals = (short int *)malloc(num_pmts*sizeof(short int));
	fread(gain_vals,sizeof(short int),num_pmts,gainfile);
	for (n = 0; n < num_pmts; n++) gain_vals[n] = ntohs(gain_vals[n]);
	fclose(gainfile);

	// READ IN THE LOG FACTORIAL FILE
	log_factorial_vals = (long int *)malloc(num_log_factorial_vals*sizeof(long int));
	log_factorial_vals_temp = (float *)malloc(num_log_factorial_vals*sizeof(float));
	fread(log_factorial_vals_temp,sizeof(float),num_log_factorial_vals,logfactfile);
	for (n = 0; n < num_log_factorial_vals; n++) *(long *)&log_factorial_vals[n] = ntohl(*(long *)&log_factorial_vals[n]);
	fclose(logfactfile);

	for (i = 0; i < num_log_factorial_vals; i++) log_factorial_vals[i] = (long int)((float)scale*log_factorial_vals_temp[i]);

	// ALLOCATE MEMORY FOR THE ZERO-PADDED MEAN FILE AND LOG MEAN FILE
	mean_vals = (long int *)malloc(det_zp*det_zp*sizeof(long int));
	mean_vals_all = (long int *)malloc(det_zp*det_zp*num_pmts*sizeof(long int));
	log_mean_vals = (long int *)malloc(det_zp*det_zp*num_pmts*sizeof(long int));
	
	// ZEROPAD THE ORIGINAL MEAN FILE UP TO THE NEXT POWER OF 2
	ML_LW_zero_pad_int(mean_vals_temp, det_size, det_zp, det_diff, num_pmts, scale, gain_vals, mean_vals, log_mean_vals,mean_vals_all);
	
	// ALLOCATE MEMORY FOR THE HEADER AND FRONT HEADER AND READ IN THE LATTER
	header = (short int *)malloc(num_pmts*sizeof(short int));
	frontheader = (short int *)malloc(2*num_pmts*sizeof(short int));
	fread(frontheader,sizeof(short int),2*num_pmts,datafile);
	for (n = 0; n < 2*num_pmts; n++) frontheader[n] = ntohs(frontheader[n]);
	
	// ALLOCATE MEMORY FOR THE OUTPUT FILE
	im_vals_total = (int *)malloc(det_size*det_size*sizeof(int));
	im_vals = (int *)malloc(det_size*det_size*sizeof(int));

	// READ THROUGH THE MDRF FILE, DOING POSITION ESTIMATION AS YOU GO AND KEEPING THE POSITIONS AND LIKELIHOODS
	for (i = 0; i < det_size*det_size; i++){

	  for (j = 0; j < det_size*det_size; j++) im_vals[j] = 0;

	  // READ IN DATA FILE
	  fread(header, sizeof(short int),num_pmts,datafile);
	  for (n = 0; n < num_pmts; n++) header[n] = ntohs(header[n]);
	  num_events = header[3]*1000 + header[4];
	  data_vals = (short int *)malloc(num_events*num_pmts*sizeof(short int));
	  fread(data_vals,sizeof(short int),num_events*num_pmts,datafile);
	  for (n = 0; n < num_pmts*num_events; n++) data_vals[n] = ntohs(data_vals[n]);
      
	  posdata = (short int *)malloc(num_events*sizeof(short int));
	  likedata = (long int *)malloc(num_events*sizeof(long int));
	
	  // DO POSITION ESTIMATION ON THE EVENTS
	  Gen_LW_pos_est_int(data_vals, mean_vals, log_mean_vals, log_factorial_vals, gain_vals, num_events, det_zp, det_diff, det_size, num_pmts, im_vals, likedata, posdata);
	
	  fwrite(likedata,sizeof(long int),num_events,likefile);
	  fwrite(posdata, sizeof(short int),num_events,posfile);

	  for (j = 0; j < det_size*det_size; j++) im_vals_total[j] += im_vals[j];

	  if (i%500 == 0) printf("%d\n",i);

	  free(likedata);
	  free(posdata);

	}

	fclose(likefile);
	fclose(datafile);
	// WRITE THE IMAGE DATA
	for (n = 0; n < det_size*det_size; n++) im_vals_total[n] = htonl(im_vals_total[n]); 
	fwrite(im_vals_total, sizeof(int), det_size*det_size, imagefile);
	fclose(imagefile);
	
	// FREE MEMORY
	free(mean_vals_temp); free(data_vals); free(im_vals); free(header); free(mean_vals); free(log_mean_vals);free(mean_vals_all);free(im_vals_total);free(frontheader); free(log_factorial_vals_temp); free(log_factorial_vals);free(gain_vals);
}
