// This function will split out the individual pinhole projections from a multiple pinhole PSF.
// 
// 
// 

// jyh,6/06

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

void Mask_Img(short *img, int x_m, int y_m, int mdrf, short *img_ph);

main(int argc, char **argv)
{

	 FILE *psf_file, *split_psf_file, *co_file, *cent_file;
	 char *psf_path, *split_psf_path, *co_path, *cent_path, filename[256];
	 float *ph_centers;
	 int camera, ph_num, range, sum_chk;
	 short *img, *img_exp, *flt, *img_ph;
	 int mdrf, mdrf_exp;
	 short *pos, *val, *pos_ph, *val_ph;
	 float x_c, y_c;
	 int x_cr, y_cr, max_img, max_idx, x_m, y_m, nnz;
	 long int sum_img;
	 long int num_coords;
	 short int *coords, header[4], header_ph[4];
	 int i,n,m;

	 setlinebuf(stdout);
	 
	 // Check the number of input arguments
	 // The inputs should include a coordinates file, two distances - object space to plate, plate to camera,
	 // (x,y) pinhole coords, mdrf dimension, mdrf step size, psf step size, and correction factor (often zeros). 
    if (argc != 11){
		  printf("usage:prog_name psf_file split_psf_file coord_file center_file camera pinhole mdrf num_coords range sum_chk\n");
		  exit(0);
    }

	 // READ IN INPUT PARAMETERS
	 psf_path = argv[1];
	 split_psf_path = argv[2];
	 co_path = argv[3];
	 cent_path = argv[4];
	 camera = atoi(argv[5]);
	 ph_num = atoi(argv[6]);
	 mdrf = atoi(argv[7]);
	 num_coords = atol(argv[8]);
	 range = atoi(argv[9]);
	 sum_chk = atoi(argv[10]);
	 
	 // OPEN PSF FILE
	 sprintf(filename,"%s",psf_path);
	 psf_file = fopen(filename, "r");
	 if (psf_file <= 0){
		  printf("couldn't open %s\n", psf_path);
		  exit(0);
	 }
	 else printf(" opened the original PSF file: %s\n",psf_path);
	 
	 // OPEN SPLIT PSF FILE
	 sprintf(filename,"%s",split_psf_path);
	 split_psf_file = fopen(filename, "w");
	 if (split_psf_file <= 0){
		  printf("couldn't open %s\n", filename);
		  exit(0);
	 }
	 else printf(" opened the split PSF file: %s\n",filename);

	 // OPEN CENTERS FILE
	 sprintf(filename,"%s",cent_path);
	 cent_file = fopen(filename, "r");
	 if (cent_file <= 0){
		  printf("couldn't open %s\n", cent_path);
		  exit(0);
	 }
	 else printf(" opened the pinhole centers file: %s\n",cent_path);

	 // OPEN COORDINATE FILE
	 sprintf(filename,"%s",co_path);
	 co_file = fopen(filename, "r");
	 if (co_file <= 0){
		  printf("couldn't open %s\n", co_path);
		  exit(0);
	 }
	 else printf(" opened the coordinates file: %s\n",co_path);
	
	 // READ IN THE COORDINATES
	 coords = (short *)malloc(num_coords*3*sizeof(short));
	 fread(coords,sizeof(short),num_coords*3,co_file);
	 for (n = 0; n < num_coords*3; n++) coords[n] = ntohs(coords[n]);
	 fclose(co_file);

	 // READ IN THE PINHOLE CENTERS	 
	 ph_centers = (float *)malloc(num_coords*2*sizeof(float));
	 fread(ph_centers, sizeof(float),num_coords*2,cent_file);
	 for (n = 0; n < num_coords*2; n++) *(long *)&ph_centers[n] = ntohl(*(long *)&ph_centers[n]);
	 fclose(cent_file);

	 // ALLOCATE MEMORY FOR A PROJECTION IMAGE AND AN EXPANDED PROJECTION IMAGE
	 img = (short *)malloc(mdrf*mdrf*sizeof(short));
	 mdrf_exp = mdrf + 2*range;
	 img_exp = (short *)malloc(mdrf_exp*mdrf_exp*sizeof(short));
	 flt = (short *)malloc(mdrf_exp*mdrf_exp*sizeof(short));
	 pos_ph = (short *)malloc(mdrf*mdrf*sizeof(short));
	 val_ph = (short *)malloc(mdrf*mdrf*sizeof(short));
	 img_ph = (short *)malloc(mdrf*mdrf*sizeof(short));

	 for (i = 0; i < num_coords; i++){		  
		  // CLEAR OUT PROJECTION IMAGE
		  for (n = 0; n < mdrf*mdrf; n++) img[n] = 0;
		  for (n = 0; n < mdrf_exp*mdrf_exp; n++){
				img_exp[n] = 0;
				flt[n] = 0;
		  }

		  // READ IN NEW PROJECTION DATA FROM PSF
		  fread(header, sizeof(short),4,psf_file);
		  for (n = 0; n < 4; n++) header[n] = ntohs(header[n]);
		  pos = (short *)malloc(header[3]*sizeof(short));
		  val = (short *)malloc(header[3]*sizeof(short));
		  fread(pos,sizeof(short),header[3],psf_file);
		  fread(val,sizeof(short),header[3],psf_file);
		  for (n = 0; n < header[3]; n++) {
				pos[n] = ntohs(pos[n]);
				val[n] = ntohs(val[n]);
		  }
		  for (n = 0; n < header[3]; n++){
				img[pos[n]] = val[n];
				img_exp[range*(mdrf+2*range) + (pos[n]/mdrf)*(mdrf+2*range) + range + (pos[n]%mdrf)] = val[n];
		  }

		  // GRAB CURRENT PINHOLE PROJECTION PREDICTION
		  x_c = ph_centers[i];
		  y_c = ph_centers[num_coords + i];

		  // FIND THE MAXIMUM VALUE IN A REGION SURROUNDING THE PREDICTED CENTER
		  if (x_c <= mdrf & x_c >= 1 & y_c <= mdrf & y_c >= 1){
				x_cr = (int)x_c;
				y_cr = (int)y_c;
				for (m = 0; m < 2*range; m++){
					 for (n = 0; n < 2*range; n++){
						  flt[(y_cr-1)*mdrf_exp + x_cr + m*mdrf_exp + n] = 1;
					 }
				}
				for (n = 0; n < mdrf_exp*mdrf_exp; n++) img_exp[n] = img_exp[n]*flt[n];

				max_img = 0;
				max_idx = 0;
				sum_img = 0;
				for (n = 0; n < mdrf_exp*mdrf_exp; n++){
					 if (img_exp[n] > max_img){
						  max_img = img_exp[n];
						  max_idx = n;
					 }
					 sum_img += img_exp[n];
				}
				x_m = max_idx%mdrf_exp - range;
				y_m = max_idx/mdrf_exp - range;

				// CALL CODE THAT WILL MASK OUT THE PROJECTION SURROUNDING THIS MAXIMUM
				for (n = 0; n < 3; n++) header_ph[n] = header[n];
				if (sum_img > sum_chk){
					 Mask_Img(img, x_m, y_m, mdrf, img_ph);
					 nnz = 0;
					 for (n = 0; n < mdrf*mdrf; n++){
						  if (img_ph[n] > 0){
								pos_ph[nnz] = n;
								val_ph[nnz] = img_ph[n];
								nnz++;
						  }
					 }
					 header_ph[3] = nnz;
					 fwrite(header_ph,sizeof(short),4,split_psf_file);
					 fwrite(pos_ph,sizeof(short),nnz,split_psf_file);
					 fwrite(val_ph,sizeof(short),nnz,split_psf_file);
				}
				else{
					 header_ph[3] = 0;
					 fwrite(header_ph,sizeof(short),4,split_psf_file);
				}
		  }
		  else{
				header[3] = 0;
				fwrite(header,sizeof(short),4,split_psf_file);
		  }
		  free(pos);free(val);
	 }
	 fclose(split_psf_file);fclose(psf_file);

	 // FREE MEMORY
	 free(img_ph);free(img);free(img_exp);free(flt);
	 free(coords);free(ph_centers);free(pos_ph);free(val_ph);
}
