// Functions for generating a projection image from event data with likelihood windowing.
// The code takes advantage of a binary search technique to complete the ML position estimate
// more quickly.  Note that all input files should be in big endian format.
// This code basically opens and closes files as well as calling the other two
// functions in the library --> ML_LW_zero_pad_int & ML_LW_pos_est_int.

// jyh,11/05

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

void ML_LW_zero_pad_int(float *mean_vals, int det_size, int det_zp, int det_diff, int num_pmts, long int scale, float gain_vals, float *mean_vals_sum, float *log_mean_vals, float *mean_vals_all);

void ML_LW_pos_est_int(short int *data_vals, float *mean_vals, float *log_mean_vals, float *log_factorial_vals, float *like_thresh_vals, float gain_vals, long int num_events, int det_zp, int det_diff, int det_size, int num_pmts, int *im_vals);   

main(int argc, char **argv)
{

  float percent_events;
  long int num_events;
  int det_size;
		/*  percent_events --> percentage of events for which to find position
			det_size --> number of MDRF steps across face (typically 78 or 79)
		*/
	
	FILE *meanfile, *logfile, *datafile, *imagefile, *likefile, *gainfile;
	char *path, filename[256];
	float *mean_vals, *mean_vals_sum, *log_mean_vals, *mean_vals_all, *log_factorial_vals, *like_thresh_vals;
	short int *data_vals, *header;
	float gain_vals;
	int *im_vals;
	int num_pmts, det_zp, size_tester, det_diff;
	int i,j,k,count,num_log_factorial_vals,n;
	long int scale;
	
	setlinebuf(stdout);
	
	// Check the number of input arguments
	// The inputs should include a mean file, a log factorial file, a likelihood threshold file, a gain file,
	// a data file, a destination file for the projection image, the percentage of events for which you'd like 
	// to find the position, and the size of the detector.
    if (argc != 10){
		printf("usage:prog_name path meanfile logfile likefile gainfile datafile_in imagefile_out percentage_of_events det_size \n");
		exit(0);
    }

    path = argv[1];
	percent_events = atof(argv[8]);
	det_size = atoi(argv[9]);

	// OPEN ORIGINAL MEAN FILE
	sprintf(filename,"%s%s",path,argv[2]);
	meanfile = fopen(filename, "r");
	if (meanfile <= 0){
		printf("couldn't open %s\n", argv[2]);
		exit(0);
	}
	else printf(" opened the mean file: %s\n",argv[2]);
	
	// OPEN LOG FACTORIAL FILE
	sprintf(filename,"%s%s",path,argv[3]);
	logfile = fopen(filename,"r");
	if(logfile<= 0){
	  printf("couldn't open %s\n", argv[3]);
	  exit(0);
	}
	else printf(" opened the log factorial file %s\n.",argv[3]);

	// OPEN LOG FACTORIAL FILE
	sprintf(filename,"%s%s",path,argv[4]);
	likefile = fopen(filename,"r");
	if(likefile<= 0){
	  printf("couldn't open %s\n", argv[4]);
	  exit(0);
	}
	else printf(" opened the likelihood threshold file %s\n.",argv[4]);

	//READ IN GAIN VALUE
	gain_vals = atof(argv[5]);

	// OPEN DATA FILE
	datafile = fopen(argv[6], "r");
	if (datafile <= 0){
		printf("couldn't open %s\n", argv[6]);
		exit(0);
	}
	else printf(" opened the data file: %s\n",argv[6]);
	
	// OPEN OUTPUT IMAGE FILE
	imagefile = fopen(argv[7], "w");
	if (imagefile <= 0){
		printf("couldn't open %s\n", argv[7]);
		exit(0);
	}
	else printf(" opened the image output file: %s\n",argv[7]);

	// DO SOME HOUSEKEEPING.  DECLARE SCALE, NUMBER OF PMTS, AND FIND THE NEW DET SIZE,
	// and DIFFERENCE IN DETECTOR SIZES
	scale = 1;
	num_pmts = 9;
	size_tester = 1;
	det_zp = 1;
	count = 1;
	while (size_tester != 0){
		det_zp = 2*det_zp;
		if (det_size/det_zp == 0){
			size_tester = 0;
		}
		count++;
	}
	num_log_factorial_vals = 2048;
	
	if ((det_zp - det_size)%2 == 0)
		det_diff = (det_zp - det_size)/2;
	else
		det_diff = (det_zp - det_size + 1)/2;

	// READ IN THE ORIGINAL MEAN FILE
	mean_vals = (float *)malloc(det_size*det_size*num_pmts*sizeof(float));
	fread(mean_vals,sizeof(float),det_size*det_size*num_pmts,meanfile);
	for (n = 0; n < det_size*det_size*num_pmts; n++) *(long *)&mean_vals[n] = ntohl(*(long *)&mean_vals[n]);
	fclose(meanfile);
	
	// READ IN THE LOG FACTORIAL FILE
	log_factorial_vals = (float *)malloc(num_log_factorial_vals*sizeof(float));
	fread(log_factorial_vals,sizeof(float),num_log_factorial_vals,logfile);
	for (n = 0; n < num_log_factorial_vals; n++) *(long *)&log_factorial_vals[n] = ntohl(*(long *)&log_factorial_vals[n]);
	fclose(logfile);

	// READ IN THE LIKELIHOOD THRESHOLD FILE
	like_thresh_vals = (float *)malloc(det_size*det_size*sizeof(float));
	fread(like_thresh_vals,sizeof(float),det_size*det_size,likefile);
	for (n = 0; n < det_size*det_size; n++) *(long *)&like_thresh_vals[n] = ntohl(*(long *)&like_thresh_vals[n]);
	fclose(likefile);


	// ALLOCATE MEMORY FOR THE ZERO-PADDED MEAN FILE AND LOG MEAN FILE
	mean_vals_all = (float *)malloc(det_zp*det_zp*num_pmts*sizeof(float));
	mean_vals_sum = (float *)malloc(det_zp*det_zp*sizeof(float));
	log_mean_vals = (float *)malloc(det_zp*det_zp*num_pmts*sizeof(float));

	// ZEROPAD THE ORIGINAL MEAN FILE UP TO THE NEXT POWER OF 2
	ML_LW_zero_pad_int(mean_vals, det_size, det_zp, det_diff, num_pmts, scale, gain_vals, mean_vals_sum, log_mean_vals,mean_vals_all);

	// ALLOCATE MEMORY FOR THE DATA AND OUTPUT FILES
	header = (short int *)malloc(num_pmts*sizeof(short int));
	im_vals = (int *)malloc(det_size*det_size*sizeof(int));
	
	// READ IN DATA FILES
	fread(header, sizeof(short int),num_pmts,datafile);
	for (n = 0; n < num_pmts; n++) header[n] = ntohs(header[n]);
	if (percent_events == 0.0) num_events = header[3]*1000 + header[4];
	else num_events = percent_events*(header[3]*1000 + header[4]);
	printf("num_events = %ld\n",num_events);
	data_vals = (short int *)malloc(num_events*num_pmts*sizeof(short int));
	fread(data_vals,sizeof(short int),num_events*num_pmts,datafile);
	for (n = 0; n < num_events*num_pmts; n++) data_vals[n] = ntohs(data_vals[n]);
	fclose(datafile);

	// DO POSITION ESTIMATION ON THE EVENTS
	ML_LW_pos_est_int(data_vals, mean_vals_sum, log_mean_vals, log_factorial_vals, like_thresh_vals, gain_vals, num_events, det_zp, det_diff, det_size, num_pmts, im_vals);
	
	// WRITE THE IMAGE DATA
	for (n = 0; n < det_size*det_size; n++) im_vals[n] = htonl(im_vals[n]);
	fwrite(im_vals, sizeof(int), det_size*det_size, imagefile);
	fclose(imagefile);
	
	// FREE MEMORY
	free(mean_vals_sum); free(data_vals); free(im_vals); free(header); free(mean_vals); free(log_mean_vals);free(mean_vals_all);free(log_factorial_vals);
}
