// Function for performing ML position estimation using a binary search technique.
// The function takes in nine PMT values from a given event and returns a position from
// 0 --> mdrf^2 where mdrf is the MDRF dimension (e.g. 79)
// Subroutine of several other pieces of code.

// jyh,11/05

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

void ML_LW_pos_est_int(short int *data_vals, float *mean_vals, float *log_mean_vals, float *log_factorial_vals,
	float *like_thresh_vals, float gain_vals, long int num_events, int det_zp, int det_diff, int det_size, int num_pmts, int *im_vals)
{
	int i, iter, j, k, l;
	int stepper, num_iters;
	int new_cen, center_init, center;
	int prod, prod_3, step_3;
	int step, step_init;
	float like, like_temp, event_data[num_pmts], final_like;
	int curr_coord, pmt_coord, final_pos;
	int flag;
	float log_fact_sum;

	// INITIALIZE THE IMAGE VECTOR
	for (i = 0; i < det_size*det_size; i++) im_vals[i] = 0;

	// DETERMINE THE INITIAL STEP SIZE AND NUMBER OF ITERATIONS NECESSARY
	step_init = det_zp/4;
	num_iters = 0;
	stepper = det_zp;
	while (stepper != 4){
		stepper = stepper/2;
		num_iters++;
	}
		
	// LOOP OVER EVENTS
	for (i = 0; i < num_events; i++){
		 for (k = 0; k < num_pmts; k++) event_data[k] = (float)data_vals[i*num_pmts + k]/gain_vals;
		 flag = 0;
                // The first set of center coordinates are simply at the center of the camera face.
		 center_init = det_zp/2 - 1;
		 center = center_init*det_zp + center_init;
		 step = step_init;
		 // This loop will take us from a 128x128 grid to a 4x4 grid.
		 if (flag == 0){
			  for (iter = 0; iter < num_iters; iter++){
					// REDUCE THE STEP SIZE FOR EACH ITERATION
					step = step/2;
					// DETERMINE THE NEW COORDINATE GRID
					prod = step*det_zp;
					step_3 = 3*step;
					prod_3 = step_3*det_zp;
					int coords[16] = {center - prod_3 - step_3, center - prod_3 - step, center - prod_3 + step, center - prod_3 + step_3,
				center - prod - step_3, center - prod - step, center - prod + step, center - prod + step_3,
											center + prod - step_3, center + prod - step, center + prod + step, center + prod + step_3,
											center + prod_3 - step_3, center + prod_3 - step, center + prod_3 + step, center + prod_3 + step_3};
					like = 0.0;
					new_cen = 0;
					// CALCULATE THE LIKELIHOOD FOR EACH COORDINATE PAIR IN THE GRID
					for (j = 0; j < 16; j++){
						 like_temp = 0.0;
						 for (k = 0; k < num_pmts; k++){
							  pmt_coord = k*det_zp*det_zp + coords[j];
							  like_temp += event_data[k]*log_mean_vals[pmt_coord];
						 }
						 like_temp = like_temp - mean_vals[coords[j]];
						 // SELECT THE LARGEST COORDINATE PAIR
						 if (like_temp > like){
							  like = like_temp;
							  new_cen = j;
						 }
					}
					//if (like <= 0.0) flag = 1;
					// SELECT THE NEW CENTER
					center = coords[new_cen];
			  }
		 }
		 
		 // WE ARE NOW DOWN TO 16 COORDINATES.  DEFINE THEM
		 
		 if (flag == 0){
			  int coords[16] = {center, center - 1, center - 2, center + 1,
									  center - det_zp, center - 1 - det_zp, center - 2 - det_zp, center + 1 - det_zp,
									  center - 2*det_zp, center - 1 - 2*det_zp, center - 2 - 2*det_zp, center + 1 - 2*det_zp,
									  center + det_zp, center - 1 + det_zp, center - 2 + det_zp, center + 1 + det_zp};
			  like = 0.0;
			  new_cen = 0;
			  // CALCULATE LIKELIHOODS FOR THE FINAL 16 POINTS
			  for (j = 0; j < 16; j++){
					like_temp = 0.0;
					for (k = 0; k < num_pmts; k++){
						 pmt_coord = k*det_zp*det_zp + coords[j];
						 like_temp += event_data[k]*log_mean_vals[pmt_coord];
					}
					like_temp = like_temp - mean_vals[coords[j]];
					// SELECT THE LARGEST ONE
					if (like_temp > like){
						 like = like_temp;
						 new_cen = j;
					}
			  }
			  
			  // CHECK THE LIKELIHOOD VALUE AGAINST A THRESHOLD
			  log_fact_sum = 0.0;
			  for (k = 0; k < num_pmts; k++){
					//printf("%d\n",(int)((float)data_vals[i*num_pmts+k]/gain_vals[k]));
					if (event_data[k] > 0.0) log_fact_sum += log_factorial_vals[(int)event_data[k]];
					else log_fact_sum += 0.0;
			  }
			  final_like = like - log_fact_sum;
			  
			  // RELATE THIS TO A LOCATION ON THE CAMERA FACE
			  final_pos = (coords[new_cen]/det_zp - det_diff)*det_size + (coords[new_cen]%det_zp - det_diff);
			  //printf("like = %f,thresh = %f\n",final_like,like_thresh_vals[final_pos]);
			  if (final_pos >= 0 & final_pos < det_size*det_size){
					if (final_like > like_thresh_vals[final_pos]) im_vals[final_pos] = im_vals[final_pos] + 1;
			  }
			  //printf("num_evt = %d\n",i);
			  
			  
		 }
	}
}
