// Function for zero-padding the original mean file and creating
// a zero-padded log mean file.  In order for the binary search technique to perform
// correctly, the meanfile must be zero-padded.  Also, the position estimation requires
// only the sum of the mean values.  Therefore, the two output files for this subroutine
// are the log mean values for each PMT and the mean values summed over PMTs.

// jyh,11/05

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

void ML_LW_zero_pad_int(float *mean_vals, int det_size, int det_zp, int det_diff, int num_pmts, long int scale, float gain_vals, float *mean_vals_sum, float *log_mean_vals, float *mean_vals_all)
{
	long int size_tester, det_temp, count;
	long int i,j,k;
	long int new_coord, old_coord;
	float scale_temp = (float)scale;

	// INITIALIZE THE NEW MEAN FILE AND LOG MEAN FILE
	for (i = 0; i < det_zp*det_zp*num_pmts; i++){
		 mean_vals_all[i] = 0.0;
		 log_mean_vals[i] = 0.0;
	}

	for (i = 0; i < det_zp*det_zp; i++){
	  mean_vals_sum[i] = 0.0;
	}
	
	// REBIN THE OLD MEAN FILE AND ITS LOG INTO THEIR PROPERLY ZERO-PADDED VERSIONS
	for (i = 0; i < num_pmts; i++){
		for (j = 0; j < det_size; j++){
			for (k = 0; k < det_size; k++){
				new_coord = i*det_zp*det_zp + (j+det_diff)*det_zp + det_diff + k;
				old_coord = i*det_size*det_size + j*det_size + k;
				mean_vals_all[new_coord] = scale_temp * mean_vals[old_coord]/gain_vals;		       
				log_mean_vals[new_coord] = scale_temp * log(mean_vals[old_coord]/gain_vals);
			}
		}
	}

	for (i = 0; i < det_zp*det_zp; i++){
	  for (j = 0; j < num_pmts; j++){
	    mean_vals_sum[i] += mean_vals_all[det_zp*det_zp*j + i];
	  }
	}

}
	
