// Function for crunching PSF data, using likelihood windowing

// jyh,11/05

// The function will read in the raw PSF data and return a crunched version with the following format
//     x,y,z coordinates --> short int (3)
//     number of non-zero values at that coordinate --> short int (1)
//     the positions of those non-zeros values --> short int (# non-zero values)
//     the values at those positions --> short int (# non-zero values)
// This structure is repeated for all of the coordinate points

// The position estimation is performed using the two functions ML_LW_zero_pad_int.c and ML_LW_pos_est_int.c
// A mean file is required.

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

void ML_LW_zero_pad_int(float *mean_vals, int det_size, int det_zp, int det_diff, int num_pmts, long int scale, float gain_vals, float *mean_vals_sum, float *log_mean_vals, float *mean_vals_all);                                                                                                                                                                                                                                                                                                     
void ML_LW_pos_est_int(short int *data_vals, float *mean_vals, float *log_mean_vals, float *log_factorial_vals, float *like_thresh_vals, float gain_vals, long int num_events, int det_zp, int det_diff, int det_size, int num_pmts, int *im_vals);            

main(int argc, char **argv)
{

  long int num_events, num_events_todo;
  int det_size, num_points;
  /*  num_events --> number of events for which to find position
      det_size --> number of MDRF steps across face (typically 78)
  */
  
  FILE *meanfile, *psf_in, *psf_out, *likefile, *logfile;
  char *meanpath, *logpath,*likepath,*psfinpath,*psfoutpath,filename[256];
  float *mean_vals, *mean_vals_sum, *log_mean_vals, *mean_vals_all, *like_thresh_vals, *log_factorial_vals;
  short int *data_vals, *data_vals_todo, *header, *pos;
  int *im_vals;
  int num_pmts, det_zp, size_tester, det_diff;
  float pct_evts;
  int i,j,k,count,n;
  short int count_vals, count_vals_orig;
  // NOTE THESE NUMBERS.  THEY COULD POTENTIALLY CAUSE TROUBLE BUT I DON'T THINK THEY WILL
  short int jj;
  short int pos_vals[30000];
  int fudge, num_log_factorial_vals;
  short int coord_vals[3];
  long int scale;
  float gain;
  
  setlinebuf(stdout);
	
  // This checks the number of inputs.  There should be a path, a mean file, a likelihood threshold file, a log factorial
  // file, a raw psf file, a destination for the
  // crunched PSF file, the number of points in the PSF, and the MDRF dimension.
  if (argc != 10){
    printf("usage:prog_name meanfile logfile likefile gainfile psf_in psf_out num_points det_size pct_events \n");
    exit(0);
  }
  
  meanpath = argv[1];
  logpath = argv[2];
  likepath = argv[3];
  gain = atof(argv[4]);
  psfinpath = argv[5];
  psfoutpath = argv[6];
  num_points = atoi(argv[7]);
  det_size = atoi(argv[8]);
  pct_evts = atof(argv[9]);
  pct_evts = pct_evts/100.0;
  if (pct_evts == 0.00) pct_evts = 1.0;
  //printf("%f\n",pct_evts);

  fudge = 1;

  // OPEN ORIGINAL MEAN FILE
  meanfile = fopen(meanpath, "r");
  if (meanfile <= 0){
    printf("couldn't open %s\n", meanpath);
    exit(0);
  }
  else printf(" opened the mean file: %s\n",meanpath);
  
  // OPEN LOG FACTORIAL FILE
  logfile = fopen(logpath,"r");
  if(logfile<= 0){
    printf("couldn't open %s\n", logpath);
    exit(0);
  }
  else printf(" opened the log factorial file %s\n.",logpath);
  
  // OPEN LIKELIHOOD THRESHOLD FACTORIAL FILE
  likefile = fopen(likepath,"r");
  if(likefile<= 0){
    printf("couldn't open %s\n", likepath);
    exit(0);
  }
  else printf(" opened the likelihood threshold file %s\n.",likepath);
  
  // OPEN DATA FILE
  psf_in = fopen(psfinpath, "r");
  if (psf_in <= 0){
    printf("couldn't open %s\n", psfinpath);
    exit(0);
  }
  else printf(" opened the data file: %s\n",psfinpath);
  
  // OPEN OUTPUT IMAGE FILE
  psf_out = fopen(psfoutpath, "w");
  if (psf_out <= 0){
    printf("couldn't open %s\n", psfoutpath);
    exit(0);
  }
  else printf(" opened the image output file: %s\n",psfoutpath);
  
  // DO SOME HOUSEKEEPING.  DECLARE NUMBER OF PMTS, SCALE, AND FIND THE NEW DET SIZE, and DIFFERENCE
  // IN DETECTOR SIZES
  scale = 1;
  num_pmts = 9;
  size_tester = 1;
  det_zp = 1;
  count = 1;
  while (size_tester != 0){
    det_zp = 2*det_zp;
    if (det_size/det_zp == 0){
      size_tester = 0;
    }
    count++;
  }
  num_log_factorial_vals = 2048;
  
  if ((det_zp - det_size)%2 == 0)
    det_diff = (det_zp - det_size)/2;
  else
    det_diff = (det_zp - det_size + 1)/2;
  
  
  // READ IN THE ORIGINAL MEAN FILE
  mean_vals = (float *)malloc(det_size*det_size*num_pmts*sizeof(float));
  fread(mean_vals,sizeof(float),det_size*det_size*num_pmts,meanfile);
  for (n = 0; n < det_size*det_size*num_pmts; n++) *(long *)&mean_vals[n] = ntohl(*(long *)&mean_vals[n]);
  fclose(meanfile);
  
  // READ IN THE LOG FACTORIAL FILE
  log_factorial_vals = (float *)malloc(num_log_factorial_vals*sizeof(float));
  fread(log_factorial_vals,sizeof(float),num_log_factorial_vals,logfile);
  for (n = 0; n < num_log_factorial_vals; n++) *(long *)&log_factorial_vals[n] = ntohl(*(long *)&log_factorial_vals[n]);
  fclose(logfile);
  
  // READ IN THE LIKELIHOOD THRESHOLD FILE
  like_thresh_vals = (float *)malloc(det_size*det_size*sizeof(float));
  fread(like_thresh_vals,sizeof(float),det_size*det_size,likefile);
  for (n = 0; n < det_size*det_size; n++) *(long *)&like_thresh_vals[n] = ntohl(*(long *)&like_thresh_vals[n]);
  fclose(likefile);
  
  // ALLOCATE MEMORY FOR THE ZERO-PADDED MEAN FILE AND LOG MEAN FILE
  mean_vals_sum = (float *)malloc(det_zp*det_zp*sizeof(float));
  mean_vals_all = (float *)malloc(det_zp*det_zp*num_pmts*sizeof(float));
  log_mean_vals = (float *)malloc(det_zp*det_zp*num_pmts*sizeof(float));

  //ALLOCATE MEMORY FOR THE POSITIONS OF NON-ZERO VALUES
  pos = (short int *)malloc(det_size*det_size*sizeof(short int));
  
  // ZEROPAD THE ORIGINAL MEAN FILE UP TO THE NEXT POWER OF 2
  // MAYBE USE non-int VERSION IN THE LONG RUN - TEST
  ML_LW_zero_pad_int(mean_vals, det_size, det_zp, det_diff, num_pmts, scale, gain, mean_vals_sum, log_mean_vals,mean_vals_all);
  
  for (i = 0; i < num_points; i++){
    
    // READ IN THE HEADER - CONTAINS POSITION, NUMBER OF EVENTS
    header = (short int *)malloc(num_pmts*sizeof(short int));
    fread(header, sizeof(short int),num_pmts, psf_in);
    for (n = 0; n < num_pmts; n++) header[n] = ntohs(header[n]);
    
    coord_vals[0] = header[0]/fudge;
    coord_vals[1] = header[2]/fudge;
    coord_vals[2] = header[1]/fudge + 1;
    num_events = 1000*header[3]+header[4];
    num_events_todo = round((float)num_events*pct_evts);

    for (n = 0; n < 3; n++) coord_vals[n] = htons(coord_vals[n]);
    fwrite(coord_vals,sizeof(short int),3,psf_out);
    
    // READ IN THE DATA FOR THIS POINT IN SPACE
    data_vals = (short int *)malloc(num_events*num_pmts*sizeof(short int));
    fread(data_vals,sizeof(short int),num_events*num_pmts,psf_in);
    for (n = 0; n < num_events*num_pmts; n++) data_vals[n] = ntohs(data_vals[n]);
    data_vals_todo = (short int *)malloc(num_events_todo*num_pmts*sizeof(short int));
    for (n = 0; n < num_events_todo*num_pmts; n++) data_vals_todo[n] = data_vals[n]; 

    // ALLOCATE MEMORY FOR THE IMAGE OF EACH POINT
    im_vals = (int *)malloc(det_size*det_size*sizeof(int));

    // DO THE POSITION ESTIMATION FOR THE EVENTS AT THIS POINT
    ML_LW_pos_est_int(data_vals_todo, mean_vals_sum, log_mean_vals, log_factorial_vals, like_thresh_vals, gain, num_events_todo, det_zp, det_diff, det_size, num_pmts, im_vals);		
    
    // WRITE OUT ANY NON-ZERO POSITIONS AND THEIR VALUES ON THE DETECTOR FOR THIS POINT
    count_vals = 0;
    
    for (jj = 0; jj < det_size*det_size; jj++){
      if (im_vals[jj] != 0){
	pos[count_vals] = jj;
	pos_vals[count_vals] = (short int)im_vals[jj];
	count_vals++;
      }
    }
    
    for (n = 0; n < count_vals; n++){
      pos[n] = htons(pos[n]);
      pos_vals[n] = htons(pos_vals[n]);
    }
    
    
    count_vals_orig = count_vals;
    count_vals = htons(count_vals);
	 fwrite(&count_vals,sizeof(short int),1,psf_out);
    fwrite(&pos,sizeof(short int),count_vals_orig,psf_out);
    fwrite(&pos_vals,sizeof(short int),count_vals_orig,psf_out);
    
    // FREE MEMORY
    free(header); free(im_vals); free(data_vals);free(data_vals_todo);
    if (i%1000 == 0) printf("%d\n",i);
  }
  // CLOSE FILES AND FREE OTHER MEMORY
  fclose(psf_in); fclose(psf_out);
  free(mean_vals); free(log_mean_vals); free(mean_vals_sum); free(mean_vals_all);
  free(log_factorial_vals);
  free(like_thresh_vals);free(pos);
}
