// Function for performing OSEM with center of rotation correction - 3 cameras.

// jyh,11/05

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

main(int argc, char **argv)
{
	
  int num_coords, det_size, num_angs, num_sets, num_iters, cam_0, cam_1, cam_2;
  /*  
      det_size --> number of MDRF steps across face (78)
      num_angs --> number of angles
      num_iters --> number of iterations
      num_coords --> coordinate pairs (740 for new PSF)
      num_sets --> number of subsets for the OSEM
      cam_i --> camera selections
  */
  FILE *coordinates; 
  FILE *all_images;
  FILE *num_vals_00_file, *num_vals_01_file, *num_vals_02_file;
  FILE *num_vals_sum_00_file, *num_vals_sum_01_file, *num_vals_sum_02_file;
  FILE *counts_00_file, *counts_01_file, *counts_02_file;
  FILE *pos_00_file, *pos_01_file, *pos_02_file;
  FILE *sn_file;
  FILE *f_final;
  FILE *ftest_file;
  
  unsigned long *coords;
  unsigned short *num_vals_00, *num_vals_01, *num_vals_02;
  unsigned short *pos_00, *pos_01, *pos_02;
  unsigned short  *counts_00, *counts_01, *counts_02;
  unsigned long *num_vals_sum_00, *num_vals_sum_01, *num_vals_sum_02;
  float *sn, *sn_base, *gtrue, *gtemp;
  float *ftest, *fresult, *ftemp;
  float *H_f, *Hadj_g;
  
  unsigned long pos_00_size, pos_01_size, pos_02_size;
  int curr_ang, curr_num_val;
  unsigned long curr_coord;
  int j, p, k, ns, i, z, nc, nnz;
  int det_sq;
  char filename[256];
  char *path, *image_name, *coordsbank_name;
	
  setlinebuf(stdout);
  
  if (argc != 13 && argc != 14){
    printf("usage:prog_name path image_file output_file coordinates_file num_coords det_size num_angs num_sets num_iters cam_0 cam_1 cam_2[f_starter]\n");
    exit(0);
  }
  
  // Read in the strings
  path = argv[1];
  image_name = argv[2];
  coordsbank_name = argv[4];
  
  // Read in all of the numerical values
  num_coords = atoi(argv[5]);
  det_size = atoi(argv[6]);
  num_angs = atoi(argv[7]);
  num_sets = atoi(argv[8]);
  num_iters = atoi(argv[9]);
  cam_0 = atoi(argv[10]);
  cam_1 = atoi(argv[11]);
  cam_2 = atoi(argv[12]);
  
  ftest = (float *)malloc(num_coords*sizeof(float));                                                                                      
                                                                                                                                          
  if (argc == 14){                                                                                                                        
      sprintf(filename,"%s",argv[13]);                                                                                                    
      ftest_file = fopen(filename,"r");                                                                                                   
      fread(ftest,sizeof(float),num_coords,ftest_file);                                                                                   
      for (i = 0; i < num_coords; i++) *(long *)&ftest[i] = ntohl(*(long *)&ftest[i]);                                                    
      fclose(ftest_file);                                                                                                                 
  }                                                                                                                                       
  else{                                                                                                                                   
      for (i = 0; i < num_coords; i++) ftest[i] = 1.0;                                                                                    
  }                                                                                                                                       

  int num_cam = 3;
  det_sq = det_size*det_size;
  
  
  //open output psf gaussian coefficient file
  sprintf(filename,"%s",argv[3]);
  f_final = fopen(filename, "w");
  if (f_final <= 0){
    printf("couldn't open %s\n", argv[3]);
    exit(0);
  }
  
  
  
  //////////////////////////////////////////////////////////////////////////////////////////	
  /* Now, for a while, we will be reading in all of our files.  These include:
     coords --> bank of coordinates that holds all coord locations for all rotations
     num_vals --> number of non-zero values for a given coordinate
     pos --> the positions on the detector face of each non-zero value for that coordinate
     counts --> the number of counts that struck pos for each non-zero value at that coordinate
     num_vals_sum --> the sum of the num_val elements. Note: starts at 0
     sn -->  the summed sensitivity of all four cameras
     gtrue --> all of the image files (covers cameras and angles)
  */
  coords = (unsigned long *)malloc(num_coords*num_angs*sizeof(unsigned long));
  num_vals_00 = (unsigned short *)malloc(num_coords*sizeof(unsigned short));
  num_vals_01 = (unsigned short *)malloc(num_coords*sizeof(unsigned short));
  num_vals_02 = (unsigned short *)malloc(num_coords*sizeof(unsigned short));
  num_vals_sum_00 = (unsigned long *)malloc(num_coords*sizeof(unsigned long));
  num_vals_sum_01 = (unsigned long *)malloc(num_coords*sizeof(unsigned long));
  num_vals_sum_02 = (unsigned long *)malloc(num_coords*sizeof(unsigned long));
  sn_base = (float *)malloc(num_coords*sizeof(float));
  gtrue = (float *)malloc(num_cam*det_sq*num_angs*sizeof(float));
  
  sprintf(filename,"%s/%s",path,coordsbank_name);
  coordinates = fopen(filename,"r");
  fread(coords,sizeof(unsigned long),num_coords*num_angs,coordinates);
  for (i = 0; i < num_coords*num_angs; i++) coords[i] = ntohl(coords[i]);
  fclose(coordinates);
  
  sprintf(filename,"%s/num_vals_0%d",path,cam_0);
  num_vals_00_file = fopen(filename,"r");
  fread(num_vals_00,sizeof(unsigned short),num_coords,num_vals_00_file);
  for (i = 0; i < num_coords; i++) num_vals_00[i] = ntohs(num_vals_00[i]);
  fclose(num_vals_00_file);
  
  sprintf(filename,"%s/num_vals_0%d",path,cam_1);
  num_vals_01_file = fopen(filename,"r");
  fread(num_vals_01,sizeof(unsigned short),num_coords,num_vals_01_file);
  for (i = 0; i < num_coords; i++) num_vals_01[i] = ntohs(num_vals_01[i]);
  fclose(num_vals_01_file);
  
  sprintf(filename,"%s/num_vals_0%d",path,cam_2);
  num_vals_02_file = fopen(filename,"r");
  fread(num_vals_02,sizeof(unsigned short),num_coords,num_vals_02_file);
  for (i = 0; i < num_coords; i++) num_vals_02[i] = ntohs(num_vals_02[i]);
  fclose(num_vals_02_file);
  
  sprintf(filename,"%s/num_vals_sum_0%d",path,cam_0);
  num_vals_sum_00_file = fopen(filename,"r");
  fread(num_vals_sum_00,sizeof(unsigned long),num_coords,num_vals_sum_00_file);
  for (i = 0; i < num_coords; i++) num_vals_sum_00[i] = ntohl(num_vals_sum_00[i]);
  fclose(num_vals_sum_00_file);
  
  sprintf(filename,"%s/num_vals_sum_0%d",path,cam_1);
  num_vals_sum_01_file = fopen(filename,"r");
  fread(num_vals_sum_01,sizeof(unsigned long),num_coords,num_vals_sum_01_file);
  for (i = 0; i < num_coords; i++) num_vals_sum_01[i] = ntohl(num_vals_sum_01[i]);
  fclose(num_vals_sum_01_file);
  
  sprintf(filename,"%s/num_vals_sum_0%d",path,cam_2);
  num_vals_sum_02_file = fopen(filename,"r");
  fread(num_vals_sum_02,sizeof(unsigned long),num_coords,num_vals_sum_02_file);
  for (i = 0; i < num_coords; i++) num_vals_sum_02[i] = ntohl(num_vals_sum_02[i]);
  fclose(num_vals_sum_02_file);
  
  sprintf(filename,"%s/sn_cam_%d%d%d",path,cam_0,cam_1,cam_2);
  sn_file = fopen(filename,"r");
  fread(sn_base,sizeof(float), num_coords, sn_file);
  for (i = 0; i < num_coords; i++) *(long *)&sn_base[i] = ntohl(*(long *)&sn_base[i]);
  fclose(sn_file);
  
  sprintf(filename,"%s",image_name);
  all_images = fopen(filename,"r");
  fread(gtrue,sizeof(float), det_sq*num_cam*num_angs, all_images);
  for (i = 0; i < det_sq*num_cam*num_angs; i++) *(long *)&gtrue[i] = ntohl(*(long *)&gtrue[i]);
  fclose(all_images);
  
  // Need to read in the num_vals_sum file before you know how big the pos and counts files are.
  pos_00_size = num_vals_sum_00[num_coords - 1] + (unsigned long)num_vals_00[num_coords - 1];
  pos_01_size = num_vals_sum_01[num_coords - 1] + (unsigned long)num_vals_01[num_coords - 1];
  pos_02_size = num_vals_sum_02[num_coords - 1] + (unsigned long)num_vals_02[num_coords - 1];
  
  pos_00 = (unsigned short *)malloc(pos_00_size*sizeof(unsigned short));
  pos_01 = (unsigned short *)malloc(pos_01_size*sizeof(unsigned short));
  pos_02 = (unsigned short *)malloc(pos_02_size*sizeof(unsigned short));
  
  counts_00 = (unsigned short *)malloc(pos_00_size*sizeof(unsigned short));
  counts_01 = (unsigned short *)malloc(pos_01_size*sizeof(unsigned short));
  counts_02 = (unsigned short *)malloc(pos_02_size*sizeof(unsigned short));
  
  sprintf(filename,"%s/pos_0%d",path,cam_0);
  pos_00_file = fopen(filename,"r");
  fread(pos_00,sizeof(unsigned short),pos_00_size,pos_00_file);
  for (i = 0; i < pos_00_size; i++) pos_00[i] = ntohs(pos_00[i]);
  fclose(pos_00_file);
  
  sprintf(filename,"%s/pos_0%d",path,cam_1);
  pos_01_file = fopen(filename,"r");
  fread(pos_01,sizeof(unsigned short),pos_01_size,pos_01_file);
  for (i = 0; i < pos_01_size; i++) pos_01[i] = ntohs(pos_01[i]);
  fclose(pos_01_file);
  
  sprintf(filename,"%s/pos_0%d",path,cam_2);
  pos_02_file = fopen(filename,"r");
  fread(pos_02,sizeof(unsigned short),pos_02_size,pos_02_file);
  for (i = 0; i < pos_02_size; i++) pos_02[i] = ntohs(pos_02[i]);
  fclose(pos_02_file);
  
  sprintf(filename,"%s/counts_0%d",path,cam_0);
  counts_00_file = fopen(filename,"r");
  fread(counts_00,sizeof(unsigned short),pos_00_size,counts_00_file);
  for (i = 0; i < pos_00_size; i++) counts_00[i] = ntohs(counts_00[i]);
  fclose(counts_00_file);
  
  sprintf(filename,"%s/counts_0%d",path,cam_1);
  counts_01_file = fopen(filename,"r");
  fread(counts_01,sizeof(unsigned short),pos_01_size,counts_01_file);
  for (i = 0; i < pos_01_size; i++) counts_01[i] = ntohs(counts_01[i]);
  fclose(counts_01_file);
  
  sprintf(filename,"%s/counts_0%d",path,cam_2);
  counts_02_file = fopen(filename,"r");
  fread(counts_02,sizeof(unsigned short),pos_02_size,counts_02_file);
  for (i = 0; i < pos_02_size; i++) counts_02[i] = ntohs(counts_02[i]);
  fclose(counts_02_file);
  
  //////////////////////////////////////////////////////////////////////////////////////////               
                                                                                                           
  // NOW WE WILL GENERATE THE SENSITIVITY MAPS FOR THIS PARTICULAR RUN                                     
  sn = (float *)malloc(num_coords*num_sets*sizeof(float));                                                 
  for (i = 0; i < num_sets*num_coords; i++) sn[i] = 0.0;                                                   
  for (i = 0; i < num_sets; i++){                                                                          
      for (j = 0; j < num_angs/num_sets; j++){                                                             
          curr_ang = j*num_sets + i;                                                                       
          for (k = 0; k < num_coords; k++){                                                                
              sn[i*num_coords+k] += sn_base[coords[curr_ang*num_coords+k]];                                
          }                                                                                                
      }                                                                                                    
  }

  //////////////////////////////////////////////////////////////////////////////////////////
  
  // Allocate memory for vectors we'll be using in the code
  fresult = (float *)malloc(num_coords*num_iters*sizeof(float));
  ftemp = (float *)malloc(num_coords*sizeof(float));
  H_f = (float *)malloc(det_sq*num_cam*sizeof(float));
  gtemp = (float *)malloc(det_sq*num_cam*sizeof(float));
  Hadj_g = (float *)malloc(num_coords*sizeof(float));
  
  for (p = 0; p < num_coords*num_iters; p++) fresult[p] = 0.0;
  
  // Begin the for loops --> over number of iterations, number of subsets, number of angles per set
  for (k = 0; k < num_iters; k++){
    for (ns = 0; ns < num_sets; ns++){
      for (p = 0; p < num_coords; p++) ftemp[p] = 0.0;
      
      for (i = 0; i < num_angs/num_sets; i++){
	
	curr_ang = i*num_sets+ns+1;
	
	for (z = 0; z < num_cam*det_sq; z++) H_f[z] = 0.0;
	
	// Compute g_hat = H*f
	for (nc = 0; nc < num_coords; nc++){
	  curr_coord = coords[num_coords*(curr_ang-1) + nc];
	  for (nnz = 0; nnz < num_vals_00[curr_coord]; nnz++){
	    curr_num_val = num_vals_sum_00[curr_coord] + nnz;
	    H_f[pos_00[curr_num_val]] += (float)counts_00[curr_num_val]*ftest[nc];
	  }
	  for (nnz = 0; nnz < num_vals_01[curr_coord]; nnz++){
	    curr_num_val = num_vals_sum_01[curr_coord] + nnz;
	    H_f[det_sq + pos_01[curr_num_val]] += (float)counts_01[curr_num_val]*ftest[nc];
	  }
	  for (nnz = 0; nnz < num_vals_02[curr_coord]; nnz++){
	    curr_num_val = num_vals_sum_02[curr_coord] + nnz;
	    H_f[2*det_sq + pos_02[curr_num_val]] += (float)counts_02[curr_num_val]*ftest[nc];
	  }
	}
	
	// Compute gtemp = g/H*f
	for (z = 0; z < num_cam*det_sq; z++){
	  if (H_f[z] != 0.0){
	    gtemp[z] = gtrue[(curr_ang - 1)*num_cam*det_sq + z]/H_f[z];
	  }
	  else gtemp[z] = 0.0;
	}
	
	for (z = 0; z < num_coords; z++) Hadj_g[z] = 0.0;
	
	// Compute H'*gtemp
	for (nc = 0; nc < num_coords; nc++){
	  curr_coord = coords[num_coords*(curr_ang-1) + nc];
	  for (nnz = 0; nnz < num_vals_00[curr_coord]; nnz++){
	    curr_num_val = num_vals_sum_00[curr_coord] + nnz;
	    Hadj_g[nc] += (float)counts_00[curr_num_val]*gtemp[pos_00[curr_num_val]];
	  }
	  for (nnz = 0; nnz < num_vals_01[curr_coord]; nnz++){
	    curr_num_val = num_vals_sum_01[curr_coord] + nnz;
	    Hadj_g[nc] += (float)counts_01[curr_num_val]*gtemp[det_sq + pos_01[curr_num_val]];
	  }
	  for (nnz = 0; nnz < num_vals_02[curr_coord]; nnz++){
	    curr_num_val = num_vals_sum_02[curr_coord] + nnz;
	    Hadj_g[nc] += (float)counts_02[curr_num_val]*gtemp[2*det_sq + pos_02[curr_num_val]];
	  }
	}
	
	// Compute f*H'*gtemp;
	for (z = 0; z < num_coords; z++){
	  ftemp[z] += Hadj_g[z];
	}
      }
      for (z = 0; z < num_coords; z++){
			 if (sn[ns*num_coords + z] != 0.0) ftest[z] = ftest[z]/sn[ns*num_coords+z]*ftemp[z];
			 else ftest[z] = 0.0;
		}
    }
    for (z = 0; z < num_coords; z++){
      fresult[k*num_coords + z] = ftest[z];
    }
    printf("Iteration %d completed\n",k+1);
  }
  
  for (i = 0; i < num_coords*num_iters; i++) *(long *)&fresult[i] = htonl(*(long *)&fresult[i]);
  fwrite(fresult,sizeof(float),num_coords*num_iters,f_final);
  fclose(f_final);
	
  free(fresult); free(ftemp); free(ftest);
  free(gtrue); free(gtemp); 
  free(sn); free(sn_base); free(coords);
  free(num_vals_00); free(num_vals_01); free(num_vals_02);
  free(num_vals_sum_00); free(num_vals_sum_01); free(num_vals_sum_02); 
  free(pos_00); free(pos_01); free(pos_02); 
  free(counts_00); free(counts_01); free(counts_02);
  free(H_f); free(Hadj_g);
}
	
