// This function is used to predict the centroids of PSF projections for a selected pinhole in a multiple pinhole PSF.
// It calls one sub-function named Calc_Det_Locs.c.
// 
// 
// 

// jyh,6/06

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

main(int argc, char **argv)
{

	 FILE *co_file, *cent_file;
	 char *co_path, *cent_path, filename[256];
	 float o_p, p_d, x_ph, y_ph, mdrf, del_px, del_psf, x_corr, y_corr, z_corr;
	 int camera;
	 long int num_coords;
	 short int *coords, N, z;
	 float obj_coords[3], *ph_centers, co_center[3];
	 float x_c, y_c, z_c, d_c, d_y, d_x;
	 int i,n;

	 setlinebuf(stdout);
	 
	 // Check the number of input arguments
	 // The inputs should include a coordinates file, two distances - object space to plate, plate to camera,
	 // (x,y) pinhole coords, mdrf dimension, mdrf step size, psf step size, and correction factor (often zeros). 
    if (argc != 15){
		  printf("usage:prog_name coord_file centers_file num_coords obj_to_ph ph_to_det x_ph y_ph mdrf mdrf_step psf_step camera x_corr y_corr z_corr\n");
		  exit(0);
    }

	 // READ IN INPUT PARAMETERS
	 co_path = argv[1];
	 cent_path = argv[2];
	 num_coords = atol(argv[3]);
	 o_p = atof(argv[4]); // object to pinhole distance
	 p_d = atof(argv[5]); // pinhole to detector distance
	 x_ph = atof(argv[6]); // x pinhole coordinate
	 y_ph = atof(argv[7]); // y pinhole coordinate
	 mdrf = atof(argv[8]);
	 del_px = atof(argv[9]);
	 del_psf = atof(argv[10]);
	 camera = atoi(argv[11]);
	 x_corr = atof(argv[12]);
	 y_corr = atof(argv[13]);
	 z_corr = atof(argv[14]);
	 
	 // OPEN COORDINATE FILE
	 sprintf(filename,"%s",co_path);
	 co_file = fopen(filename, "r");
	 if (co_file <= 0){
		  printf("couldn't open %s\n", co_path);
		  exit(0);
	 }
	 else printf(" opened the coordinates file: %s\n",co_path);

	 // OPEN CENTERS FILE
	 sprintf(filename,"%s",cent_path);
	 cent_file = fopen(filename, "w");
	 if (cent_file <= 0){
		  printf("couldn't open %s\n", cent_path);
		  exit(0);
	 }
	 else printf(" opened the coordinates file: %s\n",cent_path);
	
	 // READ IN THE COORDINATES
	 coords = (short *)malloc(num_coords*3*sizeof(short));
	 fread(coords,sizeof(short),num_coords*3,co_file);
	 for (n = 0; n < num_coords*3; n++) coords[n] = ntohs(coords[n]);
	 fclose(co_file);

	 // CALCULATE SOME NECESSARY COORDINATE INFORMATION
	 N = 0;
	 z = 0;
	 for (i = 0; i < num_coords; i++){
		  if (coords[i] > N) N = coords[i];
		  if (coords[num_coords*2 + i] > z) z = coords[num_coords*2+i];
	 }
	 co_center[0] = ((float)N-1.0)/2.0;
	 co_center[1] = ((float)N-1.0)/2.0;
	 co_center[2] = ((float)z-1.0)/2.0;

	 // ALLOCATE SPACE FOR PH_CENTERS
	 ph_centers = (float *)malloc(num_coords*2*sizeof(float));

	 for (i = 0; i < num_coords; i++){
		  x_c = ((float)coords[i]-co_center[0])*del_psf + x_corr;
		  y_c = ((float)coords[num_coords + i]-co_center[1])*del_psf + y_corr;
		  z_c = -1.0*((float)coords[2*num_coords + i]-co_center[2])*del_psf + z_corr;

		  // THIS SWITCH STATEMENT IS M3R-SPECIFIC.  CASE 1 WILL LEAVE COORDINATES UNCHANGED.
		  switch(camera){
		  case 0:
				obj_coords[0] = y_c;
				obj_coords[1] = -1.0*x_c;
				break;
		  case 1:
				obj_coords[0] = x_c;
				obj_coords[1] = y_c;
				break;
		  case 2:
				obj_coords[0] = -1.0*y_c;
				obj_coords[1] = x_c;
				break;
		  case 3:
				obj_coords[0] = -1.0*x_c;
				obj_coords[1] = -1.0*y_c;
				break;
		  default: 
				printf("Invalid Camera Selection\n");
		  }
		  obj_coords[2] = z_c;

		  d_c = (mdrf+1)/2;
		  d_y = -y_ph + p_d*(obj_coords[2] - y_ph)/(obj_coords[1] + o_p);
		  d_x = x_ph + p_d*(obj_coords[0] + x_ph)/(obj_coords[1] + o_p);

		  ph_centers[i] = d_c + d_y/del_px;
		  ph_centers[num_coords + i] = d_c + d_x/del_px;


	 }

	 fwrite(ph_centers,sizeof(float),num_coords*2,cent_file);
	 fclose(cent_file);

	 // FREE MEMORY
	 free(coords);free(ph_centers);
}
