// This function recombines the split out and interpolated PSF files from a multiple-pinhole PSF.
// 
// 
// 

// jyh,6/06

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

main(int argc, char **argv)
{

	 FILE **ph_file, *psf_file;
	 char *path, filename[256];
	 short coords[3];
	 long nnz, *pos, num_coords;
	 float *img, *val;
	 int cam, num_ph, mdrf;
	 int i, j, k;

	 setlinebuf(stdout);
	 
	 // Check the number of input arguments
	 // The inputs should include a coordinates file, two distances - object space to plate, plate to camera,
	 // (x,y) pinhole coords, mdrf dimension, mdrf step size, psf step size, and correction factor (often zeros). 
    if (argc != 6){
		  printf("usage:prog_name path cam num_ph num_coords mdrf\n");
		  exit(0);
    }

	 // READ IN INPUT PARAMETERS
	 path = argv[1];
	 cam = atoi(argv[2]);
	 num_ph = atoi(argv[3]);
	 num_coords = atol(argv[4]);
	 mdrf = atoi(argv[5]);
	 
	 ph_file = (FILE **)malloc(sizeof(FILE *)*num_ph);

	 for(i = 0; i < num_ph; i++){
		  sprintf(filename,"%s/interp_cam_%d_ph_%d/psf_0%d_interp.dat",path,cam,i+1,cam);
		  ph_file[i] = fopen(filename,"r");
	 }

	 sprintf(filename,"%s_Interp/psf_0%d_interp.dat",path,cam);
	 psf_file = fopen(filename,"w");

	 img = (float *)malloc(sizeof(float)*mdrf*mdrf);
	 pos = (long *)malloc(sizeof(short)*mdrf*mdrf);
	 val = (float *)malloc(sizeof(float)*mdrf*mdrf);

	 for(i = 0; i < num_coords; i++){
		  for (j = 0; j < mdrf*mdrf; j++) img[j] = 0.0;
		  for (j = 0; j < num_ph; j++){
				fread(coords,sizeof(short),3,ph_file[j]);
				fread(&nnz,sizeof(long),1,ph_file[j]);
				fread(pos,sizeof(long),nnz,ph_file[j]);
				fread(val,sizeof(float),nnz,ph_file[j]);
				for(k = 0; k < nnz; k++) img[pos[k]] += val[k];
		  }
		  nnz = 0;
		  for(j = 0; j < mdrf*mdrf; j++){
				if (img[j] > 0.0){
					 pos[nnz] = j;
					 val[nnz] = img[j];
					 nnz++;
				}
		  }
		  fwrite(coords,sizeof(short),3,psf_file);
		  fwrite(&nnz,sizeof(long),1,psf_file);
		  fwrite(pos,sizeof(long),nnz,psf_file);
		  fwrite(val,sizeof(float),nnz,psf_file);
	 }

	 fclose(psf_file);
	 for (i = 0; i < num_ph; i++) fclose(ph_file[i]);
	 free(img);free(pos);free(val);

}
