/* This file reads a psf gaussian coefficient file and re-arrange 
   the coefficients based on the voxel position. */

//gcc -o2 -g -static -o gcoef_vox_sort gcoef_vox_sort.c -lm

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

struct gcoef{
  float sum;
  float mx;
  float my;
  float phi;
  float lx;
  float ly;
};

main(int argc, char **argv)
{
  struct gcoef *gcoef;
  long i_cnt;
  FILE *f_psfin, *f_psfout;
  unsigned short ii, jj, kk;
  long num_angs;
  long num_vox;
  long vox_pos;
  long fp_temp;  
  long *fpos;
  long OD_X, OD_Y, OD_Z;
  long i, j, k, n;
  
  setlinebuf(stdout);
  
  if (argc != 7){
    printf("gcoef_vox_sort in_psfgcoef out_psfgcoef XDIM YDIM ZDIM num_angs\n\n");
    exit(0);
  }
  
  OD_X = atoi(argv[3]);
  OD_Y = atoi(argv[4]);
  OD_Z = atoi(argv[5]);
  num_angs = atoi(argv[6]);
  
  //open the input psf gcoef file
  f_psfin = fopen(argv[1], "r");
  printf("\n opened the input psf gcoef file, %s\n",argv[1]);

  gcoef = (struct gcoef *)malloc(num_angs*sizeof(struct gcoef));

  num_vox = 0;
  fseek(f_psfin, 0, SEEK_SET);
  //loop over psf voxels to get the total number of voxels
  while (i_cnt = fread(&ii, sizeof(unsigned short), 1, f_psfin), i_cnt > 0){
    fread(&jj, sizeof(unsigned short), 1, f_psfin);
    fread(&kk, sizeof(unsigned short), 1, f_psfin);
    fread(gcoef, sizeof(struct gcoef), num_angs, f_psfin);
    ii = ntohs(ii); jj = ntohs(jj); kk = ntohs(kk);
    for (n = 0; n < num_angs; n++){
      *(long *)&gcoef[n].sum = ntohl(*(long *)&gcoef[n].sum);
      *(long *)&gcoef[n].mx = ntohl(*(long *)&gcoef[n].mx);
      *(long *)&gcoef[n].my = ntohl(*(long *)&gcoef[n].my);
      *(long *)&gcoef[n].phi = ntohl(*(long *)&gcoef[n].phi);
      *(long *)&gcoef[n].lx = ntohl(*(long *)&gcoef[n].lx);
      *(long *)&gcoef[n].ly = ntohl(*(long *)&gcoef[n].ly);
    }
    num_vox++;
  }
  printf(" read over the whole psf gcoef file.\n number of voxel = %d\n", num_vox);
  
  fpos = (long *)malloc(OD_X*OD_Y*OD_Z*sizeof(long));
  for (j = 0; j < OD_X*OD_Y*OD_Z; j++) fpos[j] = -1;
  
  fseek(f_psfin, 0, SEEK_SET);
  //find the file position for each voxel
  for (j = 0; j < num_vox; j++){
    fp_temp = ftell(f_psfin);
    fread(&ii, sizeof(unsigned short), 1, f_psfin);
    fread(&jj, sizeof(unsigned short), 1, f_psfin);
    fread(&kk, sizeof(unsigned short), 1, f_psfin);
    fread(gcoef, sizeof(struct gcoef), num_angs, f_psfin);
    ii = ntohs(ii); jj = ntohs(jj); kk = ntohs(kk);
    for (n = 0; n < num_angs; n++){
      *(long *)&gcoef[n].sum = ntohl(*(long *)&gcoef[n].sum);
      *(long *)&gcoef[n].mx = ntohl(*(long *)&gcoef[n].mx);
      *(long *)&gcoef[n].my = ntohl(*(long *)&gcoef[n].my);
      *(long *)&gcoef[n].phi = ntohl(*(long *)&gcoef[n].phi);
      *(long *)&gcoef[n].lx = ntohl(*(long *)&gcoef[n].lx);
      *(long *)&gcoef[n].ly = ntohl(*(long *)&gcoef[n].ly);
    }

    vox_pos = ii*OD_Y*OD_Z + jj*OD_Z + kk;
    //printf("\n(x,y,z) = (%d,%d,%d) and vox_pos is %d.\n",ii,jj,kk,vox_pos);
    if (fpos[vox_pos] == -1)
      fpos[vox_pos] = fp_temp;
    else printf("\n voxel (%2d,%2d,%2d) has duplicate", ii, jj, kk);
  }
  printf("\n");

  //open the output psf gcoef file
  f_psfout = fopen(argv[2], "w");
  printf("\n opened the output psf gcoef file, %s\n",argv[2]);
  
  num_vox = 0;
  //output the gcoef file based on the voxel index  
  for (j = 0; j < OD_X*OD_Y*OD_Z; j++){
    fp_temp = fpos[j];
    if (fp_temp != -1){
      fseek(f_psfin, fp_temp, SEEK_SET);
      fread(&ii, sizeof(unsigned short), 1, f_psfin);
      fread(&jj, sizeof(unsigned short), 1, f_psfin);
      fread(&kk, sizeof(unsigned short), 1, f_psfin);
      fread(gcoef, sizeof(struct gcoef), num_angs, f_psfin);
      ii = ntohs(ii); jj = ntohs(jj); kk = ntohs(kk);
      for (n = 0; n < num_angs; n++){
        *(long *)&gcoef[n].sum = ntohl(*(long *)&gcoef[n].sum);
        *(long *)&gcoef[n].mx = ntohl(*(long *)&gcoef[n].mx);
        *(long *)&gcoef[n].my = ntohl(*(long *)&gcoef[n].my);
        *(long *)&gcoef[n].phi = ntohl(*(long *)&gcoef[n].phi);
        *(long *)&gcoef[n].lx = ntohl(*(long *)&gcoef[n].lx);
        *(long *)&gcoef[n].ly = ntohl(*(long *)&gcoef[n].ly);
      }

      ii = htons(ii); jj = htons(jj); kk = htons(kk);
      for (n = 0; n < num_angs; n++){
        *(long *)&gcoef[n].sum = htonl(*(long *)&gcoef[n].sum);
        *(long *)&gcoef[n].mx = htonl(*(long *)&gcoef[n].mx);
        *(long *)&gcoef[n].my = htonl(*(long *)&gcoef[n].my);
        *(long *)&gcoef[n].phi = htonl(*(long *)&gcoef[n].phi);
        *(long *)&gcoef[n].lx = htonl(*(long *)&gcoef[n].lx);
        *(long *)&gcoef[n].ly = htonl(*(long *)&gcoef[n].ly);
      }
      fwrite(&ii, sizeof(unsigned short), 1, f_psfout);
      fwrite(&jj, sizeof(unsigned short), 1, f_psfout);
      fwrite(&kk, sizeof(unsigned short), 1, f_psfout);
      fwrite(gcoef, sizeof(struct gcoef), num_angs, f_psfout);
      num_vox++;
    }
  }
  printf(" finished writing the new psf gcoef file.\n number of voxel = %d\n", num_vox);

  free(gcoef); free(fpos);
  fclose(f_psfin);
  fclose(f_psfout);
}
