// Function for generating a bank of coordinates that will be used to rotate the H matrix 
// during reconstruction.  Nearest neighbor interpolation is used.  A more elegant method
// may not be a bad idea.

// jyh,11/05

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

main(int argc, char **argv)
{

  int num_coords, num_angs;
  float ioff,joff,inew,jnew,ipos,jpos,irot,jrot;
  short int *coords,*coords_temp, *vox_occur;
  long int *coords_bank;
  short int N,z,P,idx;
  float pi,dst,dst_temp, total_deg;
  char *path,*coordsfile,filename[256],filename2[256];
  FILE *file_out,*file_in;
  int i,j,k,n;

  setlinebuf(stdout);

  // Check the number of input arguments
  // The input arguments will include a path to the projection files, a base name for the
  // projection images, the mdrf dimension, number of angles, number of cameras, and the
  // cameras of choice.

  if (argc != 6){
    if (argc != 8){
    printf("usage:prog_name path coordsfile num_coords num_angs total_deg [ioff] [joff] \n");
    printf("e.g. ./gen_coords_bank ../../Data/PSF_1014/ coords_01 41888 30 360 1.0 -1.0\n");
    exit(0);
    }
  }

  path = argv[1];
  coordsfile = argv[2];
  num_coords = atoi(argv[3]);
  num_angs = atoi(argv[4]);
  total_deg = atof(argv[5]);
  printf("%0.1f\n",total_deg);
  
  if (argc == 6){
    ioff = 0.0;
    joff = 0.0;
  }
  else{
    ioff = atof(argv[6]);
    joff = atof(argv[7]);
  }

  // GENERATE ANGLES FOR ROTATION
  float theta[num_angs];
  pi = 3.1415926535897;
  for (i = 0; i < num_angs; i++) theta[i] = total_deg/360.0*2.0*pi/((float)num_angs)*(float)i;

  // ALLOCATE MEMORY FOR COORDINATES AND COORDINATES BANK
  coords = (short int*)malloc(num_coords*3*sizeof(short int));
  coords_bank = (long int*)malloc(num_coords*num_angs*sizeof(long int));

  // OPEN AND READ IN COORDINATES FILE
  sprintf(filename,"%s%s",path,coordsfile);
  file_in = fopen(filename,"r");
  fread(coords,sizeof(short int),num_coords*3,file_in);
  for (n = 0; n < num_coords*3; n++) coords[n] = ntohs(coords[n]);
  fclose(file_in);

  // FIND MAXIMUM X/Y VALUE AND MAXIMUM Z VALUE
  N = 0;
  z = 0;
  for (i = 0; i < num_coords; i++){
    if (coords[i] > N) N = coords[i];
    if (coords[i+2*num_coords] > z) z = coords[i+2*num_coords];
  }
  P = num_coords/z;

  // ALLOCATE MEMORY FOR A TEMPORARY COORDINATES FILE (JUST COORDS FOR ONE SLICE)
  coords_temp = (short int*)malloc(3*P*sizeof(short int));

  // ALLOCATE MEMORY FOR A VOXEL OCCURENCE INDEX
  vox_occur = (short int*)malloc(num_coords*sizeof(short int));

  // FILL UP COORDS_TEMP
  for (i = 0; i < P; i++){
    coords_temp[i] = coords[i*z];
    coords_temp[i+P] = coords[num_coords+i*z];
    coords_temp[i+2*P] = coords[2*num_coords+i*z];
  }
      
  // FOR EACH COORDINATE IN THE ROTATED H MATRIX, WE FIND THE COORDINATE IN THE UNROTATED H MATRIX THAT WOULD BE THE BEST FIT.
  for (i = 0; i < num_angs; i++){
    for (j = 0; j < P; j++){
      irot = (float)coords_temp[j];
      jrot = (float)coords_temp[j+P];
      inew = (irot - ioff - (float)N/2.0 - 0.5)*cos(-1.0*theta[i]) + (jrot - joff - (float)N/2.0 - 0.5)*sin(-1.0*theta[i]);
      jnew = -1.0*(irot - ioff - (float)N/2.0 - 0.5)*sin(-1.0*theta[i]) + (jrot - joff - (float)N/2.0 - 0.5)*cos(-1.0*theta[i]);
      ipos = inew + (float)N/2.0 + .5 + ioff;
      jpos = jnew + (float)N/2.0 + .5 + joff;
      dst = 1000000.0;
      for (k = 0; k < P; k++){
	dst_temp = sqrt((ipos-(float)coords_temp[k])*(ipos-(float)coords_temp[k]) + 
			(jpos-(float)coords_temp[k+P])*(jpos-(float)coords_temp[k+P]));
	if (dst_temp < dst){
	  idx = k;
	  dst = dst_temp;
	}
      }
      for (k = 0; k < z; k++){
			 coords_bank[i*num_coords + z*j + k] = z*idx + k;
			 vox_occur[z*idx+k] += 1;
		}
    }
  }

  // AGAIN, THE NAMING STRUCTURE IS SPECIFIED.
  if (ioff == 0.0 & joff == 0.0){
	 if ((int)total_deg == 360) sprintf(filename,"%scoordsbank_%d",path,num_angs);
	 else sprintf(filename,"%scoordsbank_%d_%d",path,num_angs,(int)total_deg);
	 printf("%s\n",filename);
    //sprintf(filename,"%scoordsbank_%d",path,num_angs);
    file_out = fopen(filename,"w");
    for (n = 0; n < num_coords*num_angs; n++) coords_bank[n] = htonl(coords_bank[n]);
    fwrite(coords_bank,sizeof(long int),num_coords*num_angs,file_out);
    fclose(file_out);

	 // THIS FILE IS USED BY THE FOURIER CROSSTALK CODE
    sprintf(filename,"%snum_vox_occur_%d",path,num_angs);
    file_out = fopen(filename,"w");
    for (n = 0; n < num_coords; n++) vox_occur[n] = htons(vox_occur[n]);
    fwrite(vox_occur,sizeof(short int),num_coords,file_out);
    fclose(file_out);
  }
  else{
    sprintf(filename,"coordsbank_%d_%1.1f_%1.1f",num_angs,ioff,joff);
    i = 0;
    while (filename[i] != 0){
      if (filename[i] == '-') filename[i] = 'm';
      if (filename[i] == '.') filename[i] = 'p';
      i++;
    }
	 if ((int)total_deg == 360) sprintf(filename2,"%s%s",path,filename);
	 else sprintf(filename2,"%s%s_%d",path,filename,(int)total_deg);
	 printf("%s\n",filename2);
    file_out = fopen(filename2,"w");
    for (n = 0; n < num_coords*num_angs; n++) coords_bank[n] = htonl(coords_bank[n]);
    fwrite(coords_bank,sizeof(long int),num_coords*num_angs,file_out);
    fclose(file_out);
  }
  free(vox_occur);free(coords);free(coords_bank);free(coords_temp);
}
