/* This file read the input gaussian coefficients of the psf and 
   calculate the psf projection images in 3 sigma.*/
//gcc -g -static -o gen_gauspsf gen_gauspsf.c sub_gblob.c -lm -D_FILE_OFFSET_BITS=64 -D_LARGEFILE_SOURCE

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/* The gaussian coefficients are defined as following:
sum: sum (sensitivity) of the input projection image
mx, my: centroid in x, y directions
phi: rotation angle for principal axes [-PI/2, PI/2]
lx, ly: stand for lambda_x and lambda_y,
        eigenvalues of the covariance matrix (lx > ly)
*/
struct gcoef{
  float sum;
  float mx;
  float my;
  float phi;
  float lx;
  float ly;
};

main(int argc, char **argv)
{
struct gcoef *gcoef;
int i_cnt;
int OD_X, OD_Y, OD_Z;
FILE *f_psfin, *f_psfout;
FILE *f_fpos, *f_xyz;
int i, j, k, p, n;
unsigned long vox_pos;
long dim_proj, num_angs;
unsigned short ii, jj, kk, ii2, jj2, kk2;
unsigned long cnt, cnt2;
unsigned long *pos;
float *val;
float *image, *g;
unsigned long MDRFpts;
long fpos;
float ctmp[6],max_val;
long flength;
long pmin, pmax;

long loop, pts;
float percent;
time_t t0, t1, t2;
long dt;

   setlinebuf(stdout);

   if (argc != 10){
      printf("usage: gen_gauspsf in_psf_gcoef out_gauspsf XDIM YDIM ZDIM DIM_G NUM_CAM pmin pmax\n");
      exit(0);
   }

   OD_X = atoi(argv[3]);
   OD_Y = atoi(argv[4]);
   OD_Z = atoi(argv[5]);
   dim_proj = atoi(argv[6]);
   num_angs = atoi(argv[7]);
   printf("\n OD_X = %2d, OD_Y = %2d, OD_Z = %2d\n",OD_X,OD_Y,OD_Z);
   printf(" dim_proj = %2d, num_angs = %2d\n",dim_proj,num_angs);

   pmin = atol(argv[8]);
   pmax = atol(argv[9]);
   printf(" pmin = %6d, pmax = %6d\n",pmin,pmax);

   MDRFpts = dim_proj*dim_proj;

   pos = (unsigned long *)malloc(MDRFpts*num_angs*sizeof(unsigned long));
   val = (float *)malloc(MDRFpts*num_angs*sizeof(float));
   image = (float *)malloc(MDRFpts*num_angs*sizeof(float));
   g = (float *)malloc(MDRFpts*sizeof(float));
   gcoef = (struct gcoef *)malloc(num_angs*sizeof(struct gcoef));

   //open psf_gcoef file
   f_psfin = fopen(argv[1], "r");
   if (f_psfin <= 0){
      printf("couldn't open %s\n", argv[1]);
      exit(0);
   }
   else printf(" opened the input psf_gcoef file: %s\n",argv[1]);

   fseeko(f_psfin, 0, SEEK_END);
   flength = ftello(f_psfin);
   printf(" filename = %s, ",argv[1]);
   printf("file length = %d.\n",flength);
   fseeko(f_psfin, 0, SEEK_SET);

   //open output gaussian psf file
   f_psfout = fopen(argv[2], "w");
   if (f_psfout <= 0){
      printf("couldn't open %s\n", argv[2]);
      exit(0);
   }
   else printf(" opened the output gaussian psf file: %s\n\n",argv[2]);

   //loop over psf voxels (and do nothing else) till voxel #pmin
   for (k = 0; k < pmin; k++){
     if (flength == ftello(f_psfin)) break;

     fread(&ii, sizeof(unsigned short), 1, f_psfin);
     fread(&jj, sizeof(unsigned short), 1, f_psfin);
     fread(&kk, sizeof(unsigned short), 1, f_psfin);
     fread(gcoef, sizeof(struct gcoef), num_angs, f_psfin);
     ii = ntohs(ii); jj = ntohs(jj); kk = ntohs(kk);
     for (n = 0; n < num_angs; n++){
       *(long *)&gcoef[n].sum = ntohl(*(long *)&gcoef[n].sum);
       *(long *)&gcoef[n].mx = ntohl(*(long *)&gcoef[n].mx);
       *(long *)&gcoef[n].my = ntohl(*(long *)&gcoef[n].my);
       *(long *)&gcoef[n].phi = ntohl(*(long *)&gcoef[n].phi);
       *(long *)&gcoef[n].lx = ntohl(*(long *)&gcoef[n].lx);
       *(long *)&gcoef[n].ly = ntohl(*(long *)&gcoef[n].ly);
     }

   }

   time(&t0);
   t1 = t0;
   pts = (long) ceil(pow(10.0,ceil(log10((double) (pmax - pmin)/100.0))));
   loop = pts - 1;

   //loop over psf voxels and generate the gaussian psf
   for (k = pmin; k < pmax; k++){
     if (flength == ftello(f_psfin)) break;

     fread(&ii, sizeof(unsigned short), 1, f_psfin);
     fread(&jj, sizeof(unsigned short), 1, f_psfin);
     fread(&kk, sizeof(unsigned short), 1, f_psfin);
     fread(gcoef, sizeof(struct gcoef), num_angs, f_psfin);
     ii = ntohs(ii); jj = ntohs(jj); kk = ntohs(kk);
     for (n = 0; n < num_angs; n++){
       *(long *)&gcoef[n].sum = ntohl(*(long *)&gcoef[n].sum);
       *(long *)&gcoef[n].mx = ntohl(*(long *)&gcoef[n].mx);
       *(long *)&gcoef[n].my = ntohl(*(long *)&gcoef[n].my);
       *(long *)&gcoef[n].phi = ntohl(*(long *)&gcoef[n].phi);
       *(long *)&gcoef[n].lx = ntohl(*(long *)&gcoef[n].lx);
       *(long *)&gcoef[n].ly = ntohl(*(long *)&gcoef[n].ly);
     }

     //calculate projection images using gaussian coefficients
     for (j = 0; j < num_angs; j++){
       p = j*MDRFpts;

       ctmp[0] = gcoef[j].sum;
       ctmp[1] = gcoef[j].mx; 
       ctmp[2] = gcoef[j].my; 
       ctmp[3] = gcoef[j].phi; 
       ctmp[4] = gcoef[j].lx; 
       ctmp[5] = gcoef[j].ly;
/*
       printf(" %4.0f %2.0f %2.0f %5.4f %3.1f %3.1f\n",
	      ctmp[0],ctmp[1],ctmp[2],ctmp[3],ctmp[4],ctmp[5]);
*/
       sub_gblob(g, ctmp, dim_proj);

       for (i = 0; i < MDRFpts; i++){
	 image[p+i] = g[i];
       }
     }
     float     max_val = 0.0;
     for (i = 0; i < MDRFpts; i++){
       if (image[i] > max_val) max_val = image[i];
     }
     //printf("%f\n",max_val);

     //assign image to pos and val arrays
     cnt = 0;
     for (i = 0; i < MDRFpts*num_angs; i++){
       if (image[i] > 0.5){
	 pos[cnt] = i;
	 val[cnt] = image[i];
	 cnt += 1;
       }
     }

     ii2 = ii; jj2 = jj; kk2 = kk;
     ii = htons(ii);jj = htons(jj);kk = htons(kk);
     fwrite(&ii, sizeof(unsigned short), 1, f_psfout);
     fwrite(&jj, sizeof(unsigned short), 1, f_psfout);
     fwrite(&kk, sizeof(unsigned short), 1, f_psfout);
     cnt2 = cnt;
     cnt = htonl(cnt);
     fwrite(&cnt, sizeof(unsigned long), 1, f_psfout);
     for (n = 0; n < cnt2; n++){
       pos[n] = htonl(pos[n]);
       *(long *)&val[n] = htonl(*(long *)&val[n]);
     }
     fwrite(pos, sizeof(unsigned long), cnt2, f_psfout);
     fwrite(val, sizeof(float), cnt2, f_psfout);

     if ((k - pmin) == loop){
       percent = 100.0*((float) (k-pmin+1))/((float) (pmax-pmin));
       time(&t2);
       dt = (long) difftime(t2,t1);
       printf(" (x,y,z,count) = (%2d,%2d,%2d,%4d)\n",ii2,jj2,kk2,cnt2);
       printf(" %4.1f percents finished,",percent);
       printf(" execution time = %d seconds for %d pts.\n",dt,pts);
       t1 = t2;
       loop += pts;
     }
   }

   time(&t2);
   dt = (long) difftime(t2,t0);
   printf("\n read over the whole psf coefficient file\n");
   printf(" Total execution time = %d seconds for %d pts.\n\n",dt,(pmax-pmin)); 

   fclose(f_psfin); fclose(f_psfout);
   free(pos); free(val); free(image); free(g); free(gcoef);
}
