// This code is designed to prepare a sensitivity map for a given set of cameras.
// A sensitivity map is created for each individual camera upon running PSF_sort.c.  This code simply
// reads in the sensitivity maps for a chosen combination of cameras, combines them, and writes out
// a new sensitivity map for that camera combination.

// jyh,11/05

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

main(int argc, char **argv)
{

  int num_cams, num_coords;
  char *path,filename[256],sn_out_base[256];
  FILE *sn_out_file, *sn_file;
  float *sn_out, *sn_temp;
  int i,j,n;

  setlinebuf(stdout);

  // Check the number of input arguments
  // The input arguments will include a path to the sensitivity files, the number of coordinates,
  // the number of cameras, and the cameras of choice.
  
  num_cams = atoi(argv[3]);
 
  if (argc != (4 + num_cams)){
    printf("usage:prog_name path num_coords num_cams cams\n");
    printf("e.g. ./gen_sn_files ../../Data/PSF_1014/ 41888 3 0 1 3\n");
    exit(0);
  }
    
  path = argv[1];
  num_coords = atoi(argv[2]);

  int cams[num_cams];
  for (i = 0; i < num_cams; i++) cams[i] = atoi(argv[4+i]);

  sn_out = (float *)malloc(num_coords*sizeof(float));

  for (i = 0; i < num_cams; i++){
    sn_temp = (float *)malloc(num_coords*sizeof(float));
    sprintf(filename,"%ssn_cam_%d",path,cams[i]);
    sn_file = fopen(filename,"r");
    fread(sn_temp,sizeof(float),num_coords,sn_file);
    for (n = 0; n < num_coords; n++) *(long *)&sn_temp[n] = ntohl(*(long *)&sn_temp[n]);
    fclose(sn_file);
    for (j = 0; j < num_coords; j++){
      *(long *)&sn_temp[j] = ntohl(*(long *)&sn_temp[j]);
      sn_out[j] = sn_out[j] + sn_temp[j];
    }
    free(sn_temp);
  }		   
      
  sprintf(sn_out_base,"%ssn_cam_",path);
  for (i = 0; i < num_cams; i++) sprintf(sn_out_base,"%s%d",sn_out_base,cams[i]);
  sn_out_file = fopen(sn_out_base,"w");
  for (n = 0; n < num_coords; n++) *(long *)&sn_out[n] = htonl(*(long *)&sn_out[n]);
  fwrite(sn_out,sizeof(float),num_coords,sn_out_file);
  fclose(sn_out_file);

  free(sn_out);
}
