/* This file sorts all voxel index in file xyz.dat for 
given PSF scanning dimensions.  All possible vox_pos 
will be 0~(OD_X*OD_Y*OD_Z-1). 

Output files:
xyznew.dat: includes all possible voxel indices and their
	    corresponding indices in xyz.dat. If an index is
	    not found in xyz.dat, return -1 for that index.
xyzidx.dat: same as xyznew.dat without voxel indices.
*/
//gcc -g -static -o gen_xyzidx gen_xyzidx.c -lm

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

main(int argc, char **argv)
{
 int i, j, flag, n;
 int ii, i_cnt;
 int vox_pos;
 int *vox_xyz;
 int OD_X, OD_Y, OD_Z;
 FILE *f_xyz;
 FILE *f_xyzidx;
 long num_vox; //number of PSF voxels

   if (argc != 6){
      printf("usage: gen_xyzidx in_xyz out_xyzidx XDIM YDIM ZDIM\n");
      exit(0);
   }

   OD_X = atoi(argv[3]);
   OD_Y = atoi(argv[4]);
   OD_Z = atoi(argv[5]);

   f_xyz = fopen(argv[1], "r");

   num_vox = 0;
   while (i_cnt = fread(&ii, sizeof(unsigned long), 1, f_xyz), i_cnt > 0){
     num_vox += 1;
     ii = ntohl(ii);
   }
   printf(" total number of psf voxels = %6d\n",num_vox);

   vox_xyz = (int *)malloc(num_vox*sizeof(int));

   fseek(f_xyz, 0, SEEK_SET);
   fread(vox_xyz, sizeof(long), num_vox, f_xyz);
   for (n = 0; n < num_vox; n++) vox_xyz[n] = ntohl(vox_xyz[n]);
   fclose(f_xyz);

   f_xyzidx = fopen(argv[2], "w");

   for (j = 0; j < OD_X*OD_Y*OD_Z; j++){
      vox_pos = j;
      flag = -1;
      for (i = 0; i < num_vox; i++){
	if (vox_pos == vox_xyz[i]){
	  flag = i;
	  break;
	}
      }
      flag = htonl(flag);
      fwrite(&flag, sizeof(int), 1, f_xyzidx);
   }
   fclose(f_xyzidx);
}
