/* This file scans through voxels in psfgcoef0.dat and 
rewrite the xyz coordinates (eg. X = 0,2,4,6,...)
Output file:
psfgcoef1.dat: new PSF gaussian coefficient file.
*/
//gcc -g -static -o interp_gcoef1 interp_gcoef1.c -lm

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/* The gaussian coefficients are defined as following:
sum: sum (sensitivity) of the input projection image
mx, my: centroid in x, y directions
phi: rotation angle for principal axes [-PI/2, PI/2]
lx, ly: stand for lambda_x and lambda_y,
        eigenvalues of the covariance matrix (lx > ly)
*/
struct gcoef{
  float sum;
  float mx;
  float my;
  float phi;
  float lx;
  float ly;
};

main(int argc, char **argv)
{
 struct gcoef *gcoef;
 int i_cnt;
 int OD_X, OD_Y, OD_Z;
 FILE *f_psfin, *f_psfout;
 int i, j, k, p, n;
 unsigned long vox_pos;
 long dim_proj, num_angs;
 unsigned short ii, jj, kk;
 unsigned long cnt;
 unsigned long MDRFpts;
 long fpos;
 float ctmp[6];

   setlinebuf(stdout);

   if (argc != 8){
      printf("usage: interp_gcoef1 in_psf_gcoef out_psf_gcoef1 XDIM YDIM ZDIM DIM_G NUM_CAM\n");
      exit(0);
   }

   OD_X = atoi(argv[3]);
   OD_Y = atoi(argv[4]);
   OD_Z = atoi(argv[5]);
   dim_proj = atoi(argv[6]);
   num_angs = atoi(argv[7]);
   printf("\n OD_X = %2d, OD_Y = %2d, OD_Z = %2d\n",OD_X,OD_Y,OD_Z);
   printf(" dim_proj = %2d, num_angs = %2d\n",dim_proj,num_angs);

   MDRFpts = dim_proj*dim_proj;

   gcoef = (struct gcoef *)malloc(num_angs*sizeof(struct gcoef));

   //open psf_gcoef file
   f_psfin = fopen(argv[1], "r");
   if (f_psfin <= 0){
      printf("couldn't open %s\n", argv[1]);
      exit(0);
   }
   else printf(" opened the input original psf_gcoef file: %s\n",argv[1]);

   //open the output gaussian coefficient file
   f_psfout = fopen(argv[2], "w");
   if (f_psfout <= 0){
      printf("couldn't open %s\n", argv[2]);
      exit(0);
   }
   else printf(" opened the output psf_gcoef1 file: %s\n",argv[2]);

   //loop over psf voxels
   while (i_cnt = fread(&ii, sizeof(unsigned short), 1, f_psfin), i_cnt > 0){
     fread(&jj, sizeof(unsigned short), 1, f_psfin);
     fread(&kk, sizeof(unsigned short), 1, f_psfin);
     fread(gcoef, sizeof(struct gcoef), num_angs, f_psfin);
     ii = ntohs(ii); jj = ntohs(jj); kk = ntohs(kk);
     for (n = 0; n < num_angs; n++){
       *(long *)&gcoef[n].sum = ntohl(*(long *)&gcoef[n].sum);
       *(long *)&gcoef[n].mx = ntohl(*(long *)&gcoef[n].mx);
       *(long *)&gcoef[n].my = ntohl(*(long *)&gcoef[n].my);
       *(long *)&gcoef[n].phi = ntohl(*(long *)&gcoef[n].phi);
       *(long *)&gcoef[n].lx = ntohl(*(long *)&gcoef[n].lx);
       *(long *)&gcoef[n].ly = ntohl(*(long *)&gcoef[n].ly);
     }

     ii = ii*2;
     jj = jj*2;
     kk = kk*2;

     ii = htons(ii); jj = htons(jj); kk = htons(kk);
     for (n = 0; n < num_angs; n++){
       *(long *)&gcoef[n].sum = htonl(*(long *)&gcoef[n].sum);
       *(long *)&gcoef[n].mx = htonl(*(long *)&gcoef[n].mx);
       *(long *)&gcoef[n].my = htonl(*(long *)&gcoef[n].my);
       *(long *)&gcoef[n].phi = htonl(*(long *)&gcoef[n].phi);
       *(long *)&gcoef[n].lx = htonl(*(long *)&gcoef[n].lx);
       *(long *)&gcoef[n].ly = htonl(*(long *)&gcoef[n].ly);
     }
     fwrite(&ii, sizeof(unsigned short), 1, f_psfout);
     fwrite(&jj, sizeof(unsigned short), 1, f_psfout);
     fwrite(&kk, sizeof(unsigned short), 1, f_psfout);
     fwrite(gcoef, sizeof(struct gcoef), num_angs, f_psfout);
   }
   printf(" read over the whole psf gaussian coefficient file\n");

   fclose(f_psfin); fclose(f_psfout);
   free(gcoef);
}
