/* This file scans through voxels in psfgcoef1.dat and 
interpolates for mid-point response of one line.
Output file:
psfgcoef2.dat: new PSF gaussian coefficient file contains interpolated data.
*/
//gcc -g -static -o interp_gcoef2 interp_gcoef2.c sub_avgcoef2.c -lm

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/* The gaussian coefficients are defined as following:
sum: sum (sensitivity) of the input projection image
mx, my: centroid in x, y directions
phi: rotation angle for principal axes [-PI/2, PI/2]
lx, ly: stand for lambda_x and lambda_y,
        eigenvalues of the covariance matrix (lx > ly)
*/
struct gcoef{
  float sum;
  float mx;
  float my;
  float phi;
  float lx;
  float ly;
};

main(int argc, char **argv)
{
 struct gcoef *gcoef, *gcoef1, *gcoef2;
 int i_cnt;
 int OD_X, OD_Y, OD_Z;
 FILE *f_psfin, *f_psfout;
 FILE *f_fpos, *f_xyz;
 int i, j, k, p, n;
 unsigned long vox_p1, vox_p2;
 long dim_proj, num_angs;
 unsigned short ii, jj, kk;
 unsigned short x1, x2, y1, y2, z1, z2;
 unsigned long cnt;
 unsigned long MDRFpts;
 long *fpos;
 long *xyzidx;
 FILE *f_xyzidx;
 long num_vox; //total number of voxels
 long pmin, pmax;
 long idx1, idx2;
 long fpos1, fpos2;
 short flag;

 long loop, pts;
 float percent;
 time_t t0, t1, t2;
 long dt;

   setlinebuf(stdout);

   if (argc != 10){
      printf("usage: interp_gcoef2 in_psf_gcoef0 in_xyzidx in_fpos out_psf_gcoef2 XDIM YDIM ZDIM DIM_G NUM_CAM\n");
      exit(0);
   }
   OD_X = atoi(argv[5]);
   OD_Y = atoi(argv[6]);
   OD_Z = atoi(argv[7]);
   dim_proj = atoi(argv[8]);
   num_angs = atoi(argv[9]);
   printf("\n OD_X = %2d, OD_Y = %2d, OD_Z = %2d\n",OD_X,OD_Y,OD_Z);
   printf(" dim_proj = %2d, num_angs = %2d\n",dim_proj,num_angs);

   MDRFpts = dim_proj*dim_proj;

   //open xyzidx file
   xyzidx = (long *)malloc(OD_X*OD_Y*OD_Z*sizeof(long));
   f_xyzidx = fopen(argv[2], "r");
   printf("f_xyzidx is %d bytes.\n",sizeof(f_xyzidx));
   if (f_xyzidx <= 0){
      printf("couldn't open %s\n", argv[2]);
      exit(0);
   }
   else printf(" opened the input xyzidx file: %s\n",argv[2]);
   fread(xyzidx, sizeof(long), OD_X*OD_Y*OD_Z, f_xyzidx);
   for (n = 0; n < OD_X*OD_Y*OD_Z; n++) xyzidx[n] = ntohl(xyzidx[n]);
   fclose(f_xyzidx);

   //open psf_gcoef file
   f_psfin = fopen(argv[1], "r");
   if (f_psfin <= 0){
      printf("couldn't open %s\n", argv[1]);
      exit(0);
   }
   else printf(" opened the input original psf_gcoef0 file: %s\n",argv[1]);

   gcoef = (struct gcoef *)malloc(num_angs*sizeof(struct gcoef));

   time(&t0);
   num_vox = 0;
   //loop over psf voxels to get number of voxels
   fseek(f_psfin, 0, SEEK_SET);
   while (i_cnt = fread(&ii, sizeof(unsigned short), 1, f_psfin), i_cnt > 0){
     fread(&jj, sizeof(unsigned short), 1, f_psfin);
     fread(&kk, sizeof(unsigned short), 1, f_psfin);
     fread(gcoef, sizeof(struct gcoef), num_angs, f_psfin);
     ii = ntohs(ii); jj = ntohs(jj); kk = ntohs(kk);
     for (n = 0; n < num_angs; n++){
       *(long *)&gcoef[n].sum = ntohl(*(long *)&gcoef[n].sum);
       *(long *)&gcoef[n].mx = ntohl(*(long *)&gcoef[n].mx);
       *(long *)&gcoef[n].my = ntohl(*(long *)&gcoef[n].my);
       *(long *)&gcoef[n].phi = ntohl(*(long *)&gcoef[n].phi);
       *(long *)&gcoef[n].lx = ntohl(*(long *)&gcoef[n].lx);
       *(long *)&gcoef[n].ly = ntohl(*(long *)&gcoef[n].ly);
     }     
     num_vox++;
   }
   time(&t1);
   dt = (long) difftime(t1,t0);
   printf(" read over the whole psf gaussian coefficient file in %d seconds\n",dt);
   printf(" the total number of voxels = %d\n",num_vox);

   //open fpos file
   fpos = (long *)malloc(num_vox*sizeof(long));
   f_fpos = fopen(argv[3], "r");
   if (f_fpos <= 0){
      printf("couldn't open %s\n", argv[3]);
      exit(0);
   }
   else printf(" opened the input fpos file: %s\n",argv[3]);
   fread(fpos, sizeof(long), num_vox, f_fpos);
   for (n = 0; n < num_vox; n++) fpos[n] = ntohl(fpos[n]);
   fclose(f_fpos);

   //open output interpolated gaussian coefficient file
   f_psfout = fopen(argv[4], "w");
   if (f_psfout <= 0){
      printf("couldn't open %s\n", argv[4]);
      exit(0);
   }
   else printf(" opened the output psf_gcoef2 file: %s\n\n",argv[4]);

   pmin = 0;
   pmax = num_vox;
   pts = (long) ceil(pow(10.0,ceil(log10((double) (pmax - pmin)/100.0))));
   loop = pts - 1;

   gcoef1 = (struct gcoef *)malloc(num_angs*sizeof(struct gcoef));
   gcoef2 = (struct gcoef *)malloc(num_angs*sizeof(struct gcoef));

   time(&t1);
   //loop over psf voxels and assign gcoef array
   for (p = pmin; p < num_vox; p++){
     //read all data related to pth PSF voxel
     idx1 = p;
     fpos1 = fpos[idx1];
     fseek(f_psfin, fpos1, SEEK_SET);
     fread(&x1, sizeof(unsigned short), 1, f_psfin);
     fread(&y1, sizeof(unsigned short), 1, f_psfin);
     fread(&z1, sizeof(unsigned short), 1, f_psfin);
     fread(gcoef1, sizeof(struct gcoef), num_angs, f_psfin);
     x1 = ntohs(x1); y1 = ntohs(y1); z1 = ntohs(z1);
     for (n = 0; n < num_angs; n++){
       *(long *)&gcoef1[n].sum = ntohl(*(long *)&gcoef1[n].sum);
       *(long *)&gcoef1[n].mx = ntohl(*(long *)&gcoef1[n].mx);
       *(long *)&gcoef1[n].my = ntohl(*(long *)&gcoef1[n].my);
       *(long *)&gcoef1[n].phi = ntohl(*(long *)&gcoef1[n].phi);
       *(long *)&gcoef1[n].lx = ntohl(*(long *)&gcoef1[n].lx);
       *(long *)&gcoef1[n].ly = ntohl(*(long *)&gcoef1[n].ly);
     }
     //printf("x1=%d, y1=%d, z1=%d\n",x1,y1,z1);

     //6 cases: x(in,out), y(left,right), z(up,down)
     for (j = 0; j < 6; j++){
       flag = 0;

       //(x1 - 1,y1,z1)
       if (j == 0){
	 if (x1 > 0){
	   x2 = x1 - 1; y2 = y1; z2 = z1;
	   flag = 1;
	 }
       }
       
       //(x1 + 1,y1,z1)
       else if (j == 1){
	 if (x1 < (OD_X - 1)){
	   x2 = x1 + 1; y2 = y1; z2 = z1;
	   flag = 1;
	 }
       }
       
       //(x1,y1 - 1,z1)
       else if (j == 2){
	 if (y1 > 0){
	   x2 = x1; y2 = y1 - 1; z2 = z1;
	   flag = 1;
	 }
       }
       
       //(x1,y1 + 1,z1)
       else if (j == 3){
	 if (y1 < (OD_Y - 1)){
	   x2 = x1; y2 = y1 + 1; z2 = z1;
	   flag = 1;
	 }
       }
       
       //(x1,y1,z1 - 1)
       else if (j == 4){
	 if (z1 > 0){
	   x2 = x1; y2 = y1; z2 = z1 - 1;
	   flag = 1;
	 }
       }
       
       //(x1,y1,z1 + 1)
       else if (j == 5){
	 if (z1 < (OD_Z - 1)){
	   x2 = x1; y2 = y1; z2 = z1 + 1;
	   flag = 1;
	 }
       }

       if (flag){       

	 //compute vox_p2 and get all data at vox_p2
	 vox_p2=((long) x2)*OD_Y*OD_Z + ((long) y2)*OD_Z + ((long) z2);
	 //printf("x2=%d, y2=%d, z2=%d, vox_p2=%d\n",x2,y2,z2,vox_p2);
	 idx2 = xyzidx[vox_p2];
	 //printf("idx1=%d, idx2=%d\n",idx1,idx2);


	 if (idx2 > idx1){
	   fpos2 = fpos[idx2];
	   fseek(f_psfin, fpos2, SEEK_SET);
	   fread(&x2, sizeof(unsigned short), 1, f_psfin);
	   fread(&y2, sizeof(unsigned short), 1, f_psfin);
	   fread(&z2, sizeof(unsigned short), 1, f_psfin);
	   fread(gcoef2, sizeof(struct gcoef), num_angs, f_psfin);
	   x2 = ntohs(x2);y2 = ntohs(y2);z2 = ntohs(z2);
	   for (n = 0; n < num_angs; n++){
	     *(long *)&gcoef2[n].sum = ntohl(*(long *)&gcoef2[n].sum);
	     *(long *)&gcoef2[n].mx = ntohl(*(long *)&gcoef2[n].mx);
	     *(long *)&gcoef2[n].my = ntohl(*(long *)&gcoef2[n].my);
	     *(long *)&gcoef2[n].phi = ntohl(*(long *)&gcoef2[n].phi);
	     *(long *)&gcoef2[n].lx = ntohl(*(long *)&gcoef2[n].lx);
	     *(long *)&gcoef2[n].ly = ntohl(*(long *)&gcoef2[n].ly);
	   }

	   //perform interpolation
	   sub_avgcoef2(gcoef, gcoef1, gcoef2, num_angs);

	   //save voxel position and gaussian coefficients of the interpolated point
	   ii = (x1 + x2);
	   jj = (y1 + y2);
	   kk = (z1 + z2);
	   //printf("x3=%d, y3=%d, z3=%d\n",ii,jj,kk);
	   ii = htons(ii); jj = htons(jj); kk = htons(kk);
	   for (n = 0; n < num_angs; n++){
	     *(long *)&gcoef[n].sum = htonl(*(long *)&gcoef[n].sum);
	     *(long *)&gcoef[n].mx = htonl(*(long *)&gcoef[n].mx);
	     *(long *)&gcoef[n].my = htonl(*(long *)&gcoef[n].my);
	     *(long *)&gcoef[n].phi = htonl(*(long *)&gcoef[n].phi);
	     *(long *)&gcoef[n].lx = htonl(*(long *)&gcoef[n].lx);
	     *(long *)&gcoef[n].ly = htonl(*(long *)&gcoef[n].ly);
	   }
	   fwrite(&ii, sizeof(unsigned short), 1, f_psfout);
	   fwrite(&jj, sizeof(unsigned short), 1, f_psfout);
	   fwrite(&kk, sizeof(unsigned short), 1, f_psfout);
	   fwrite(gcoef, sizeof(struct gcoef), num_angs, f_psfout);
	 }
       } //flag
     } //j: 6 cases

     if ((p-pmin) == loop){
       percent = 100.0*((float) (p-pmin+1))/((float) (pmax-pmin));
       time(&t2);
       dt = (long) difftime(t2,t1);
       printf(" %4.1f percents finished,",percent);
       printf(" execution time = %d seconds for %d pts.\n",dt,pts);
       t1 = t2;
       loop += pts;
     }
   }//k: loop over voxels

   time(&t2);
   dt = (long) difftime(t2,t0);
   printf("\n read over the whole psf gaussian coefficient file\n");
   printf(" Total execution time = %d seconds for %d pts.\n\n",dt,(pmax-pmin)); 
   
   fclose(f_psfin); fclose(f_psfout);
   free(gcoef); free(gcoef1); free(gcoef2); free(xyzidx); free(fpos);
}
