/* This file scans through voxels in psfgcoef1.dat and 
interpolates for mid-point response of one cube.
Output file:
psfgcoef8.dat: new PSF gaussian coefficient file contains interpolated data.
*/
//gcc -g -static -o interp_gcoef8 interp_gcoef8.c sub_classify.c sub_rdcoef2.c sub_rdcoef4.c sub_rdcoef6.c sub_rdcoef8.c sub_avgcoef2.c sub_avgcoef4.c sub_avgcoef6.c sub_avgcoef8.c -lm

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/* The gaussian coefficients are defined as following:
sum: sum (sensitivity) of the input projection image
mx, my: centroid in x, y directions
phi: rotation angle for principal axes [-PI/2, PI/2]
lx, ly: stand for lambda_x and lambda_y,
        eigenvalues of the covariance matrix (lx > ly)
*/
struct gcoef{
  float sum;
  float mx;
  float my;
  float phi;
  float lx;
  float ly;
};

main(int argc, char **argv)
{
 struct gcoef *gcoef, *gcoef1;
 int i_cnt;
 int OD_X, OD_Y, OD_Z;
 FILE *f_psfin, *f_psfout;
 FILE *f_fpos, *f_xyz;
 int i, j, k, p, n;
 unsigned long vox_p1, vox_p2, vox_p3, vox_p4;
 unsigned long vox_p5, vox_p6, vox_p7, vox_p8;
 long dim_proj, num_angs;
 unsigned short ii, jj, kk;
 unsigned short x1, x2, y1, y2, z1, z2;
 unsigned short x3, x4, y3, y4, z3, z4;
 unsigned short x5, x6, y5, y6, z5, z6;
 unsigned short x7, x8, y7, y8, z7, z8;
 unsigned long cnt;
 unsigned long MDRFpts;
 long *fpos;
 long *xyzidx;
 FILE *f_xyzidx;
 long num_vox; //total number of voxels
 long pmin, pmax;
 long fpos1;
 short flag;
 long index[9];
 long id[9];
 long condition;
 short case_num;

 long loop, pts;
 float percent;
 time_t t0, t1, t2;
 long dt;

   setlinebuf(stdout);

   if (argc != 10){
      printf("usage: interp_gcoef8 in_psf_gcoef0 in_xyzidx in_fpos out_psf_gcoef8 XDIM YDIM ZDIM DIM_G NUM_CAM\n");
      exit(0);
   }
   OD_X = atoi(argv[5]);
   OD_Y = atoi(argv[6]);
   OD_Z = atoi(argv[7]);
   dim_proj = atoi(argv[8]);
   num_angs = atoi(argv[9]);
   printf("\n OD_X = %2d, OD_Y = %2d, OD_Z = %2d\n",OD_X,OD_Y,OD_Z);
   printf(" dim_proj = %2d, num_angs = %2d\n",dim_proj,num_angs);

   MDRFpts = dim_proj*dim_proj;

   //open xyzidx file
   xyzidx = (long *)malloc(OD_X*OD_Y*OD_Z*sizeof(long));
   f_xyzidx = fopen(argv[2], "r");
   if (f_xyzidx <= 0){
      printf("couldn't open %s\n", argv[2]);
      exit(0);
   }
   else printf(" opened the input xyzidx file: %s\n",argv[2]);
   fread(xyzidx, sizeof(long), OD_X*OD_Y*OD_Z, f_xyzidx);
   for (n = 0; n < OD_X*OD_Y*OD_Z; n++) xyzidx[n] = ntohl(xyzidx[n]);
   fclose(f_xyzidx);

   //open psf_gcoef file
   f_psfin = fopen(argv[1], "r");
   if (f_psfin <= 0){
      printf("couldn't open %s\n", argv[1]);
      exit(0);
   }
   else printf(" opened the input original psf_gcoef0 file: %s\n",argv[1]);

   gcoef = (struct gcoef *)malloc(num_angs*sizeof(struct gcoef));

   time(&t0);
   num_vox = 0;
   //loop over psf voxels to get number of voxels
   fseek(f_psfin, 0, SEEK_SET);
   while (i_cnt = fread(&ii, sizeof(unsigned short), 1, f_psfin), i_cnt > 0){
     fread(&jj, sizeof(unsigned short), 1, f_psfin);
     fread(&kk, sizeof(unsigned short), 1, f_psfin);
     fread(gcoef, sizeof(struct gcoef), num_angs, f_psfin);
     ii = ntohs(ii); jj = ntohs(jj); kk = ntohs(kk);
     for (n = 0; n < num_angs; n++){
       *(long *)&gcoef[n].sum = ntohl(*(long *)&gcoef[n].sum);
       *(long *)&gcoef[n].mx = ntohl(*(long *)&gcoef[n].mx);
       *(long *)&gcoef[n].my = ntohl(*(long *)&gcoef[n].my);
       *(long *)&gcoef[n].phi = ntohl(*(long *)&gcoef[n].phi);
       *(long *)&gcoef[n].lx = ntohl(*(long *)&gcoef[n].lx);
       *(long *)&gcoef[n].ly = ntohl(*(long *)&gcoef[n].ly);
     }
     num_vox++;
   }
   time(&t1);
   dt = (long) difftime(t1,t0);
   printf(" read over the whole psf gaussian coefficient file in %d seconds\n",dt);
   printf(" the total number of voxels = %d\n",num_vox);

   //open fpos file
   fpos = (long *)malloc(num_vox*sizeof(long));
   f_fpos = fopen(argv[3], "r");
   if (f_fpos <= 0){
      printf("couldn't open %s\n", argv[3]);
      exit(0);
   }
   else printf(" opened the input fpos file: %s\n",argv[3]);
   fread(fpos, sizeof(long), num_vox, f_fpos);
   for (n = 0; n < num_vox; n++) fpos[n] = ntohl(fpos[n]);
   fclose(f_fpos);

   //open output interpolated gaussian coefficient file
   f_psfout = fopen(argv[4], "w");
   if (f_psfout <= 0){
      printf("couldn't open %s\n", argv[4]);
      exit(0);
   }
   else printf(" opened the output psf_gcoef2 file: %s\n\n",argv[4]);

   pmin = 0;
   pmax = num_vox;
   pts = (long) ceil(pow(10.0,ceil(log10((double) (pmax - pmin)/100.0))));
   loop = pts - 1;

   gcoef1 = (struct gcoef *)malloc(num_angs*sizeof(struct gcoef));

   time(&t1);
   //loop over psf voxels and assign gcoef array
   for (p = pmin; p < num_vox; p++){
     //read all data related to pth PSF voxel
     index[1] = p;
     fpos1 = fpos[index[1]];
     fseek(f_psfin, fpos1, SEEK_SET);
     fread(&x1, sizeof(unsigned short), 1, f_psfin);
     fread(&y1, sizeof(unsigned short), 1, f_psfin);
     fread(&z1, sizeof(unsigned short), 1, f_psfin);
     fread(gcoef1, sizeof(struct gcoef), num_angs, f_psfin);
     x1 = htons(x1); y1 = htons(y1); z1 = htons(z1);
     for (n = 0; n < num_angs; n++){
       *(long *)&gcoef1[n].sum = htonl(*(long *)&gcoef1[n].sum);
       *(long *)&gcoef1[n].mx = htonl(*(long *)&gcoef1[n].mx);
       *(long *)&gcoef1[n].my = htonl(*(long *)&gcoef1[n].my);
       *(long *)&gcoef1[n].phi = htonl(*(long *)&gcoef1[n].phi);
       *(long *)&gcoef1[n].lx = htonl(*(long *)&gcoef1[n].lx);
       *(long *)&gcoef1[n].ly = htonl(*(long *)&gcoef1[n].ly);
     }
     //printf("x1=%d, y1=%d, z1=%d\n",x1,y1,z1);

     //8 cases: x(in,out), y(left,right), z(up,down), --> 2^3 = 8
     //diagonal's: (1,2), (3,8), (4,7), (5,6)
     for (j = 0; j < 8; j++){
       flag = 0;

       //(x1 - 1,y1 - 1,z1 - 1)
       if (j == 0){
	 if ((x1 > 0) && (y1 > 0) && (z1 > 0)){
	   x2 = x1 - 1; y2 = y1 - 1; z2 = z1 - 1;
	   x3 = x1;     y3 = y1 - 1; z3 = z1;
	   x4 = x1;     y4 = y1;     z4 = z1 - 1;
	   x5 = x1;     y5 = y1 - 1; z5 = z1 - 1;
	   x6 = x1 - 1; y6 = y1;     z6 = z1;
	   x7 = x1 - 1; y7 = y1 - 1; z7 = z1;
	   x8 = x1 - 1; y8 = y1;     z8 = z1 - 1;
	   flag = 1;
	 }
       }

       //(x1 + 1,y1 - 1,z1 - 1)
       else if (j == 1){
	 if ((x1 < (OD_X - 1)) && (y1 > 0) && (z1 > 0)){
	   x2 = x1 + 1; y2 = y1 - 1; z2 = z1 - 1;
	   x3 = x1;     y3 = y1 - 1; z3 = z1;
	   x4 = x1;     y4 = y1;     z4 = z1 - 1;
	   x5 = x1;     y5 = y1 - 1; z5 = z1 - 1;
	   x6 = x1 + 1; y6 = y1;     z6 = z1;
	   x7 = x1 + 1; y7 = y1 - 1; z7 = z1;
	   x8 = x1 + 1; y8 = y1;     z8 = z1 - 1;
	   flag = 1;
	 }
       }
       
       //(x1 - 1,y1 + 1,z1 - 1)
       else if (j == 2){
	 if ((x1 > 0) && (y1 < (OD_Y - 1)) && (z1 > 0)){
	   x2 = x1 - 1; y2 = y1 + 1; z2 = z1 - 1;
	   x3 = x1;     y3 = y1 + 1; z3 = z1;
	   x4 = x1;     y4 = y1;     z4 = z1 - 1;
	   x5 = x1;     y5 = y1 + 1; z5 = z1 - 1;
	   x6 = x1 - 1; y6 = y1;     z6 = z1;
	   x7 = x1 - 1; y7 = y1 + 1; z7 = z1;
	   x8 = x1 - 1; y8 = y1;     z8 = z1 - 1;
	   flag = 1;
	 }
       }

       //(x1 + 1,y1 + 1,z1 - 1)
       else if (j == 3){
	 if ((x1 < (OD_X - 1)) && (y1 < (OD_Y - 1)) && (z1 > 0)){
	   x2 = x1 + 1; y2 = y1 + 1; z2 = z1 - 1;
	   x3 = x1;     y3 = y1 + 1; z3 = z1;
	   x4 = x1;     y4 = y1;     z4 = z1 - 1;
	   x5 = x1;     y5 = y1 + 1; z5 = z1 - 1;
	   x6 = x1 + 1; y6 = y1;     z6 = z1;
	   x7 = x1 + 1; y7 = y1 + 1; z7 = z1;
	   x8 = x1 + 1; y8 = y1;     z8 = z1 - 1;
	   flag = 1;
	 }
       }

       //(x1 - 1,y1 - 1,z1 + 1)
       else if (j == 4){
	 if ((x1 > 0) && (y1 > 0) && (z1 < (OD_Z - 1))){
	   x2 = x1 - 1; y2 = y1 - 1; z2 = z1 + 1;
	   x3 = x1;     y3 = y1 - 1; z3 = z1;
	   x4 = x1;     y4 = y1;     z4 = z1 + 1;
	   x5 = x1;     y5 = y1 - 1; z5 = z1 + 1;
	   x6 = x1 - 1; y6 = y1;     z6 = z1;
	   x7 = x1 - 1; y7 = y1 - 1; z7 = z1;
	   x8 = x1 - 1; y8 = y1;     z8 = z1 + 1;
	   flag = 1;
	 }
       }

       //(x1 + 1,y1 - 1,z1 + 1)
       else if (j == 5){
	 if ((x1 < (OD_X - 1)) && (y1 > 0) && (z1 < (OD_Z - 1))){
	   x2 = x1 + 1; y2 = y1 - 1; z2 = z1 + 1;
	   x3 = x1;     y3 = y1 - 1; z3 = z1;
	   x4 = x1;     y4 = y1;     z4 = z1 + 1;
	   x5 = x1;     y5 = y1 - 1; z5 = z1 + 1;
	   x6 = x1 + 1; y6 = y1;     z6 = z1;
	   x7 = x1 + 1; y7 = y1 - 1; z7 = z1;
	   x8 = x1 + 1; y8 = y1;     z8 = z1 + 1;
	   flag = 1;
	 }
       }

       //(x1 - 1,y1 + 1,z1 + 1)
       else if (j == 6){
	 if ((x1 > 0) && (y1 < (OD_Y - 1)) && (z1 < (OD_Z - 1))){
	   x2 = x1 - 1; y2 = y1 + 1; z2 = z1 + 1;
	   x3 = x1;     y3 = y1 + 1; z3 = z1;
	   x4 = x1;     y4 = y1;     z4 = z1 + 1;
	   x5 = x1;     y5 = y1 + 1; z5 = z1 + 1;
	   x6 = x1 - 1; y6 = y1;     z6 = z1;
	   x7 = x1 - 1; y7 = y1 + 1; z7 = z1;
	   x8 = x1 - 1; y8 = y1;     z8 = z1 + 1;
	   flag = 1;
	 }
       }

       //(x1 + 1,y1 + 1,z1 + 1)
       else if (j == 7){
	 if ((x1 < (OD_X - 1)) && (y1 < (OD_Y - 1)) && (z1 < (OD_Z - 1))){
	   x2 = x1 + 1; y2 = y1 + 1; z2 = z1 + 1;
	   x3 = x1;     y3 = y1 + 1; z3 = z1;
	   x4 = x1;     y4 = y1;     z4 = z1 + 1;
	   x5 = x1;     y5 = y1 + 1; z5 = z1 + 1;
	   x6 = x1 + 1; y6 = y1;     z6 = z1;
	   x7 = x1 + 1; y7 = y1 + 1; z7 = z1;
	   x8 = x1 + 1; y8 = y1;     z8 = z1 + 1;
	   flag = 1;
	 }
       }

       if (flag){       

	 for (i = 2; i < 9; i++) id[i] = 0;

	 //compute vox_p2 and get file index of vox_p2
	 vox_p2=((long) x2)*OD_Y*OD_Z + ((long) y2)*OD_Z + ((long) z2);
	 index[2] = xyzidx[vox_p2];
	 if (index[2] > index[1]) id[2] = 1;
	 else if (index[2] == -1) id[2] = 2;

	 //compute vox_p3 and get file index of vox_p3
	 vox_p3=((long) x3)*OD_Y*OD_Z + ((long) y3)*OD_Z + ((long) z3);
	 index[3] = xyzidx[vox_p3];
	 if (index[3] > index[1]) id[3] = 1;
	 else if (index[3] == -1) id[3] = 3;
	 
	 //compute vox_p4 and get file index of vox_p4
	 vox_p4=((long) x4)*OD_Y*OD_Z + ((long) y4)*OD_Z + ((long) z4);
	 index[4] = xyzidx[vox_p4];
	 if (index[4] > index[1]) id[4] = 1;
	 else if (index[4] == -1) id[4] = 5;
	 
	 //compute vox_p5 and get file index of vox_p5
	 vox_p5=((long) x5)*OD_Y*OD_Z + ((long) y5)*OD_Z + ((long) z5);
	 index[5] = xyzidx[vox_p5];
	 if (index[5] > index[1]) id[5] = 1;
	 else if (index[5] == -1) id[5] = 7;
	 
	 //compute vox_p6 and get file index of vox_p6
	 vox_p6=((long) x6)*OD_Y*OD_Z + ((long) y6)*OD_Z + ((long) z6);
	 index[6] = xyzidx[vox_p6];
	 if (index[6] > index[1]) id[6] = 1;
	 else if (index[6] == -1) id[6] = 11;
	 
	 //compute vox_p7 and get file index of vox_p7
	 vox_p7=((long) x7)*OD_Y*OD_Z + ((long) y7)*OD_Z + ((long) z7);
	 index[7] = xyzidx[vox_p7];
	 if (index[7] > index[1]) id[7] = 1;
	 else if (index[7] == -1) id[7] = 13;
	 
	 //compute vox_p8 and get file index of vox_p8
	 vox_p8=((long) x8)*OD_Y*OD_Z + ((long) y8)*OD_Z + ((long) z8);
	 index[8] = xyzidx[vox_p8];
	 if (index[8] > index[1]) id[8] = 1;
	 else if (index[8] == -1) id[8] = 17;
	 
	 condition = 1;
	 case_num = -1;
	 
	 for (i = 2; i < 9; i++){
	   condition *= id[i];
	 }

	 //diagonal pairs: (1,2), (3,8), (4,7), (5,6)
	 //at least one pair of the diagonal corners must exist
	 if ((condition != 0) && ((index[2] != -1) 
				  || ((index[3] != -1) && (index[8] != -1))
				  || ((index[4] != -1) && (index[7] != -1))
				  || ((index[5] != -1) && (index[6] != -1)))){
	   
	   case_num = sub_classify(id, condition);
	   
	   if (case_num == 1){
	     //interpolate the center-point of one cube from all eight corners
	     sub_rdcoef8(fpos, f_psfin, index, gcoef, gcoef1, num_angs);
	   }
	   
	   else if (case_num < 6){
	     //interpolate the center-point of one cube from six corners
	     sub_rdcoef6(fpos, f_psfin, index, gcoef, gcoef1, num_angs, case_num);
	   }
	   
	   else if (case_num < 12){
	     //interpolate the center-point of one cube from four corners
	     sub_rdcoef4(fpos, f_psfin, index, gcoef, gcoef1, num_angs, case_num);
	   }
	   
	   else {
	     //interpolate the center-point of one cube from two corners
	     sub_rdcoef2(fpos, f_psfin, index, gcoef, gcoef1, num_angs, case_num);
	   }

	   //save voxel position and gaussian coefficients of the interpolated point
	   ii = (x1 + x2);
	   jj = (y1 + y2);
	   kk = (z1 + z2);
	   //printf("x3=%d, y3=%d, z3=%d\n",ii,jj,kk);
	   ii = htons(ii); jj = htons(jj); kk = htons(kk);
           for (n = 0; n < num_angs; n++){
             *(long *)&gcoef[n].sum = htonl(*(long *)&gcoef[n].sum);
             *(long *)&gcoef[n].mx = htonl(*(long *)&gcoef[n].mx);
             *(long *)&gcoef[n].my = htonl(*(long *)&gcoef[n].my);
             *(long *)&gcoef[n].phi = htonl(*(long *)&gcoef[n].phi);
             *(long *)&gcoef[n].lx = htonl(*(long *)&gcoef[n].lx);
             *(long *)&gcoef[n].ly = htonl(*(long *)&gcoef[n].ly);
           }
	   fwrite(&ii, sizeof(unsigned short), 1, f_psfout);
	   fwrite(&jj, sizeof(unsigned short), 1, f_psfout);
	   fwrite(&kk, sizeof(unsigned short), 1, f_psfout);
	   fwrite(gcoef, sizeof(struct gcoef), num_angs, f_psfout);

	 } /* if (condition != 0) */
       } /* flag */
     } /* j: 8cases */

     if ((p-pmin) == loop){
       percent = 100.0*((float) (p-pmin+1))/((float) (pmax-pmin));
       time(&t2);
       dt = (long) difftime(t2,t1);
       printf(" %4.1f percents finished,",percent);
       printf(" execution time = %d seconds for %d pts.\n",dt,pts);
       t1 = t2;
       loop += pts;
     }
   }//k: loop over voxels

   time(&t2);
   dt = (long) difftime(t2,t0);
   printf("\n read over the whole psf gaussian coefficient file\n");
   printf(" Total execution time = %d seconds for %d pts.\n\n",dt,(pmax-pmin)); 
   
   fclose(f_psfin); fclose(f_psfout);
   free(gcoef); free(gcoef1); free(xyzidx); free(fpos);
}

/* Cases for interpolation

At least one pair of diagonal pairs (1,2), (3,8), (4,7),
and (5,6) must exist to perform interpolation.

point 1 always exists.

point 2, 3, 4, 5, 6, 7, 8
id#   2, 3, 5, 7,11,13,17 if point doesn't exist.
id# = 1 if point exists.

condition# = product of all id#'s.

situation	case#	pairs missing	condition#
missing 0 pair	 1	N/A		1

missing 1 pair	 2	(1,2)		2
		 3	(3,8)		3,17,51
		 4	(4,7)		5,13,65
		 5	(5,6)		7,11,77

missing 2 pairs	 6	(1,2), (3,8)	6,34,102
		 7	(1,2), (4,7)	10,26,130
		 8	(1,2), (5,6)	14,22,154
		 9	(3,8), (4,7)	15,39,85,195,221,255,663,1105,3315
		10	(3,8), (5,6)	21,33,119,187,231,357,561,1309,3927
		11	(4,7), (5,6)	35,55,91,143,385,455,715,1001,5005

if not any of above cases:
			pair remained	conditions on id#
missing 3 pair	12	(1,2)		id[2] == 1
		13	(3,8)		id[3] == id[8] == 1
		14	(4,7)		id[4] == id[7] == 1
		15	(5,6)		id[5] == id[6] == 1

Therefore there are 15 cases, and there are at most
128 possible numbers for condition numbers (2^7 =  128)
to perform interpolation.

Another way to classify cases 12 ~ 15:

			pair missing		condition#
missing 3 pair	12	(1,2), (3,8), (4,7)	2*(15,39,85,195,221,255,663,1105,3315)
		13	(1,2), (3,8), (5,6)	2*(21,33,119,187,231,357,561,1309,3927)
		14	(1,2), (4,7), (5,6)	2*(35,55,91,143,385,455,715,1001,5005)
		15	(3,8), (4,7), (5,6)	27 numbers

These 27 numbers are:	 105, 165, 273, 429, 595, 935,1155,1365,1547,1785
			2145,2431,2805,3003,4641,6545,7293,7735,12155,15015
			17017,19635,23205,36465,51051,85085,255255

*/
