// Function for preparing reconstruction files for use with Amide (or some other
// viewing software).
// The object space is typically cylindrical in shape.  However, the viewing programs
// (like Amide) will want something on a rectangular grid.  This code just throws the
// final iteration of the reconstruction on a rectangular grid.

// jyh,11/05

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

main(int argc, char **argv)
{

  int num_iters, num_coords;
  char *coords_name,*path,*recon_name,filename[256];
  FILE *coords_file,*recon_file, *recon_out;
  float *recon_total, *recon, *recon_3D;
  short *coords;
  int i,n,N,z;

  setlinebuf(stdout);

  // Check the number of input arguments
  // The input arguments will include a path to the projection files, a base name for the
  // projection images, the mdrf dimension, number of angles, number of cameras, and the
  // cameras of choice.
  if (argc != 6){
    printf("usage:prog_name path recon_file coords_file num_iters num_coords\n"); 
    printf("e.g. prep_recon_3D ../Recon_data/ recon_test ../PSF_files/coords_00 20 41888\n");
    exit(0);
  }

  path = argv[1];
  recon_name = argv[2];
  coords_name = argv[3];
  num_iters = atoi(argv[4]);
  num_coords = atoi(argv[5]);

  coords = (short *)malloc(3*num_coords*sizeof(short));
  coords_file = fopen(coords_name,"r");
  fread(coords,sizeof(short),num_coords*3,coords_file);
  for (n = 0; n < num_coords*3; n++) coords[n] = htons(coords[n]);
  fclose(coords_file);

  N = 0; z = 0;
  for (i = 0; i < num_coords; i++){
    if (coords[i] > N) N = coords[i];
    if (coords[2*num_coords+i] > z) z = coords[2*num_coords+i];
  }

  recon_total = (float *)malloc(num_coords*(num_iters-1)*sizeof(float));
  recon = (float *)malloc(num_coords*sizeof(float));
  sprintf(filename,"%s%s",path,recon_name);
  recon_file = fopen(filename,"r");
  fread(recon_total,sizeof(float),num_coords*(num_iters-1),recon_file);
  for (n = 0; n < (num_iters-1)*num_coords; n++) *(long *)&recon_total[n] = ntohl(*(long *)&recon_total[n]);
  fread(recon,sizeof(float),num_coords,recon_file);
  for (n = 0; n < num_coords; n++) *(long *)&recon[n] = ntohl(*(long *)&recon[n]);
  fclose(recon_file);
  free(recon_total);

  recon_3D = (float *)malloc(N*N*z*sizeof(float));

  for (i = 0; i < N*N*z; i++) recon_3D[i] = 0.0;
  
  for (i = 0; i < num_coords; i++){
    recon_3D[(coords[2*num_coords+i]-1)*N*N + (coords[num_coords+i]-1)*N + coords[i]] = recon[i];
  }

  sprintf(filename,"%s%s_3D",path,recon_name);
  recon_out = fopen(filename,"w");
  for (n = 0; n < N*N*z; n++) *(long *)&recon_3D[n] = htonl(*(long*)&recon_3D[n]);
  fwrite(recon_3D,sizeof(float),N*N*z,recon_out);
  fclose(recon_out);

  free(coords);free(recon_3D);free(recon);
}
