// Function for preparing image files for the reconstruction code.
// For any combination of cameras, this code will concatenate all of the 
// projection images.

// jyh,11/05

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

main(int argc, char **argv)
{

  int mdrf, num_angs, num_cams, mdrf_sq;
  char *path,*proj_base,filename[256],basename[256],img_out_base[256];
  FILE *img_out_file,*img_temp_file;
  float *img_out;
  int *img_temp;
  int i,j,k,count,n;

  setlinebuf(stdout);

  // Check the number of input arguments
  // The input arguments will include a path to the projection files, a base name for the 
  // projection images, the mdrf dimension, number of angles, number of cameras, and the
  // cameras of choice.
  
  num_cams = atoi(argv[5]);
 
  if (argc != (6 + num_cams)){
    printf("usage:prog_name path proj_base mdrf num_angs num_cams cams\n");
    printf("e.g. ./proj_prep ../../Data/PSF_1014/img_files/ sig_1014_ 79 30 2 0 3\n");
    exit(0);
  }
    
  path = argv[1];
  proj_base = argv[2];
  mdrf = atoi(argv[3]);
  num_angs = atoi(argv[4]);
  mdrf_sq = mdrf*mdrf;

  int cams[num_cams];
  for (i = 0; i < num_cams; i++) cams[i] = atoi(argv[6+i]);

  img_out = (float *)malloc(mdrf_sq*num_cams*num_angs*sizeof(float));

  sprintf(basename,"%s%s",path,proj_base);

  // READ IN THE IMAGES AND CONCATENATE THEM AS REQUIRED FOR THE RECONSTRUCTION CODE
  for (i = 0; i < num_angs; i++){
    for (j = 0; j < num_cams; j++){
      img_temp = (int *)malloc(mdrf_sq*sizeof(int));
      //sprintf(filename,"%s_%2d_%2d_lw.img",basename,i+1,cams[j]);
      sprintf(filename,"%s_%2d_%2d.img",basename,i+1,cams[j]);
      count = 0;
      while (filename[count] != 0){
	if (filename[count] == ' '){
	  filename[count] = '0';
	}
	count++;
      }
      img_temp_file = fopen(filename,"r");
      fread(img_temp,sizeof(int),mdrf_sq,img_temp_file);
      for (n = 0; n < mdrf_sq; n++) img_temp[n] = ntohl(img_temp[n]);
      fclose(img_temp_file);
      
      for (k = 0; k < mdrf_sq; k++){
	img_temp[k] = ntohs(img_temp[k]);
	img_out[i*mdrf_sq*num_cams + j*mdrf_sq + k] = (float)img_temp[k];
      }
      free(img_temp);
    }
  }

  sprintf(img_out_base,"%sall_%s_cam_",path,proj_base);
  for (i = 0; i < num_cams; i++) sprintf(img_out_base,"%s%d",img_out_base,cams[i]);
  img_out_file = fopen(img_out_base,"w");
  for (n = 0; n < mdrf_sq*num_cams*num_angs; n++) *(long *)&img_out[n] = htonl(*(long *)&img_out[n]);
  fwrite(img_out,sizeof(float),mdrf_sq*num_cams*num_angs,img_out_file);
  fclose(img_out_file);

  free(img_out);
}
