// Function for preparing image files for the reconstruction code.
// For any combination of cameras, this code will concatenate all of the 
// projection images and will produce a sensitivity file for tha particular camera set
// if so desired.

// jyh,11/05

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

main(int argc, char **argv)
{

  int mdrf, num_angs, num_cams, mdrf_sq;
  char *path,*proj_base,filename[256],basename[256],img_out_base[256];
  FILE *img_out_file, *img_in_file;
  short int *header, *pos, *counts;
  float *img_all, *img_temp;
  int i,j,k,n;

  setlinebuf(stdout);

  // Check the number of input arguments
  // The input arguments will include a path to the projection files, a base name for the 
  // projection images, the mdrf dimension, number of angles, number of cameras, and the
  // cameras of choice.
  
  num_cams = atoi(argv[5]);
 
  if (argc != (6 + num_cams)){
    printf("usage:prog_name path proj_base mdrf num_angs num_cams cams\n");
    printf("e.g. ./proj_prep ../../Data/PSF_1014/img_files/ sig_1014_ 79 30 2 0 3\n");
    exit(0);
  }
    
  path = argv[1];
  proj_base = argv[2];
  mdrf = atoi(argv[3]);
  num_angs = atoi(argv[4]);
  mdrf_sq = mdrf*mdrf;

  int cams[num_cams];
  for (i = 0; i < num_cams; i++) cams[i] = atoi(argv[6+i]);

  img_all = (float *)malloc(mdrf_sq*num_cams*num_angs*sizeof(float));
  img_temp = (float *)malloc(mdrf_sq*sizeof(float));

  sprintf(basename,"%s%s",path,proj_base);

  // READ IN THE IMAGE FILES (PSF-STYLE) AND PREPARE THEM FOR RECONSTRUCTION
  for (i = 0; i < num_cams; i++){
    sprintf(filename,"%s_0%d.img",basename,cams[i]);
    img_in_file = fopen(filename,"r");
    for (j = 0; j < num_angs; j++){
      header = (short int*)malloc(4*sizeof(short int));
      pos = (short int*)malloc(mdrf_sq*sizeof(short int));
      counts = (short int*)malloc(mdrf_sq*sizeof(short int));
      fread(header,sizeof(short int),4,img_in_file);
      for (n = 0; n < 4; n++) header[n] = ntohs(header[n]);
      fread(pos,sizeof(short int),header[3],img_in_file);
      for (n = 0; n < header[3]; n++) pos[n] = ntohs(pos[n]);
      fread(counts,sizeof(short int),header[3],img_in_file);
      for (n = 0; n < header[3]; n++) counts[n] = ntohs(counts[n]);
      for (k = 0; k < mdrf_sq; k++) img_temp[k] = 0.0;
      for (k = 0; k < header[3]; k++){
	img_temp[pos[k]] = (float)counts[k];
      }
      for (k = 0; k < mdrf_sq; k++) img_all[j*num_cams*mdrf_sq + i*mdrf_sq + k] = img_temp[k];
      free(header);free(pos);free(counts);
    }
    fclose(img_in_file);
  }
 
  free(img_temp);
  
  sprintf(img_out_base,"%sall_%s_cam_",path,proj_base);
  for (i = 0; i < num_cams; i++) sprintf(img_out_base,"%s%d",img_out_base,cams[i]);
  img_out_file = fopen(img_out_base,"w");
  for (n = 0; n < mdrf_sq*num_cams*num_angs; n++) *(long *)&img_all[n] = htonl(*(long *)&img_all[n]);
  fwrite(img_all,sizeof(float),mdrf_sq*num_cams*num_angs,img_out_file);
  fclose(img_out_file);
  free(img_all);
}

