// Function for crunching PSF data

// jyh,11/05

// The function will read in the raw PSF data and return a crunched version with the following format
//     x,y,z coordinates --> short int (3)
//     number of non-zero values at that coordinate --> short int (1)
//     the positions of those non-zeros values --> short int (# non-zero values)
//     the values at those positions --> int (# non-zero values)
// This structure is repeated for all of the coordinate points

// The position estimation is performed using the two functions ML_zero_pad_int.c and ML_pos_est_int.c
// A mean file is required.

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

main(int argc, char **argv)
{

  int num_points, det_size;
  /*  num_events --> number of events for which to find position
      det_size --> number of MDRF steps across face (typically 78)
  */
  
  FILE *meanfile, *psf_in, *psf_out;
  char *psf_path, *mean_path;
  long int *mean_vals, *log_mean_vals, *mean_vals_all; 
  float *mean_vals_temp;
  short int *data_vals, *header;
  int *im_vals;
  short int fudge;
  int num_pmts, det_zp, size_tester, det_diff;
  int i,j,k,count,n;
  short int count_vals;
  // NOTE THESE NUMBERS.  THEY COULD POTENTIALLY CAUSE TROUBLE BUT I DON'T THINK THEY WILL
  short int jj, pos[10000];
  short int pos_vals[30000];
  int num_events;
  short int coord_vals[3];
  long int scale;
  
  setlinebuf(stdout);
  
  // This checks the number of inputs.  There should be a mean file, a raw psf file, a destination for the
  // crunched PSF file, the number of points in the PSF, and the MDRF dimension.
  if (argc != 7){
    printf("usage:prog_name meanfile psf_in psf_out num_points det_size fudge \n");
    exit(0);
  }
  
  mean_path = argv[1];
  psf_path = argv[2];
  num_points = atoi(argv[4]);
  det_size = atoi(argv[5]);
  fudge = atoi(argv[6]);
  fudge = (short)fudge;
  
  // OPEN ORIGINAL MEAN FILE
  meanfile = fopen(argv[1], "r");
  if (meanfile <= 0){
    printf("couldn't open %s\n", argv[1]);
    exit(0);
  }
  else printf(" opened the mean file: %s\n",argv[1]);
  
  // OPEN DATA FILE
  psf_in = fopen(argv[2], "r");
  if (psf_in <= 0){
    printf("couldn't open %s\n", argv[2]);
    exit(0);
  }
  else printf(" opened the data file: %s\n",argv[2]);
  
  // OPEN OUTPUT IMAGE FILE
  psf_out = fopen(argv[3], "w");
  if (psf_out <= 0){
    printf("couldn't open %s\n", argv[3]);
    exit(0);
  }
  else printf(" opened the image output file: %s\n",argv[3]);
  
  // DO SOME HOUSEKEEPING.  DECLARE NUMBER OF PMTS, SCALE, AND FIND THE NEW DET SIZE, and DIFFERENCE
  // IN DETECTOR SIZES
  scale = 65536;
  num_pmts = 9;
  size_tester = 1;
  det_zp = 1;
  count = 1;
  while (size_tester != 0){
    det_zp = 2*det_zp;
    if (det_size/det_zp == 0){
      size_tester = 0;
    }
    count++;
  }
  
  if ((det_zp - det_size)%2 == 0)
    det_diff = (det_zp - det_size)/2;
  else
    det_diff = (det_zp - det_size + 1)/2;
  
  // READ IN THE ORIGINAL MEAN FILE
  mean_vals_temp = (float *)malloc(det_size*det_size*num_pmts*sizeof(float));
  fread(mean_vals_temp,sizeof(float),det_size*det_size*num_pmts,meanfile);
  for (n = 0; n < det_size*det_size*num_pmts; n++) *(long *)&mean_vals_temp[n] = ntohl(*(long *)&mean_vals_temp[n]);
  fclose(meanfile);
  
  // ALLOCATE MEMORY FOR THE ZERO-PADDED MEAN FILE AND LOG MEAN FILE
  mean_vals = (long int *)malloc(det_zp*det_zp*sizeof(long int));
  mean_vals_all = (long int *)malloc(det_zp*det_zp*num_pmts*sizeof(long int));
  log_mean_vals = (long int *)malloc(det_zp*det_zp*num_pmts*sizeof(long int));
  
  // ZEROPAD THE ORIGINAL MEAN FILE UP TO THE NEXT POWER OF 2
  // MAYBE USE non-int VERSION IN THE LONG RUN - TEST
  ML_zero_pad_int(mean_vals_temp, det_size, det_zp, det_diff, num_pmts, scale, mean_vals, log_mean_vals,mean_vals_all);
  
  for (i = 0; i < num_points; i++){
    
    // READ IN THE HEADER - CONTAINS POSITION, NUMBER OF EVENTS
    header = (short int *)malloc(num_pmts*sizeof(short int));
    fread(header, sizeof(short int),num_pmts, psf_in);
    for (n = 0; n < num_pmts; n++) header[n] = ntohs(header[n]);
    
    coord_vals[0] = header[0]/fudge;
    coord_vals[1] = header[2]/fudge;
    coord_vals[2] = header[1]/fudge + 1;
    num_events = 1000*header[3]+header[4];
    
    for (n = 0; n < 3; n++) coord_vals[n] = htons(coord_vals[n]);
    fwrite(coord_vals,sizeof(short int),3,psf_out);
    
    // READ IN THE DATA FOR THIS POINT IN SPACE
    data_vals = (short int *)malloc(num_events*num_pmts*sizeof(short int));
    fread(data_vals,sizeof(short int),num_events*num_pmts,psf_in);
    for (n = 0; n < num_events*num_pmts; n++) data_vals[n] = ntohs(data_vals[n]);
    
    // ALLOCATE MEMORY FOR THE IMAGE OF EACH POINT
    im_vals = (int *)malloc(det_size*det_size*sizeof(int));
    
    // DO THE POSITION ESTIMATION FOR THE EVENTS AT THIS POINT
    ML_pos_est_int(data_vals, mean_vals, log_mean_vals, num_events, det_zp, det_diff, det_size, num_pmts, im_vals);
    
    // WRITE OUT ANY NON-ZERO POSITIONS AND THEIR VALUES ON THE DETECTOR FOR THIS POINT
    count_vals = 0;
    
    for (jj = 0; jj < det_size*det_size; jj++){
      if (im_vals[jj] != 0){
	pos[count_vals] = jj;
	pos_vals[count_vals] = (short int)im_vals[jj];
	count_vals++;
      }
    }
    
    count_vals = htons(count_vals);
    fwrite(&count_vals,sizeof(short int),1,psf_out);
    for (n = 0; n < count_vals; n++){
      pos[n] = htons(pos[n]);
      pos_vals[n] = htons(pos_vals[n]);
    }
    fwrite(&pos,sizeof(short int),count_vals,psf_out);
    fwrite(&pos_vals,sizeof(short int),count_vals,psf_out);
    
    // FREE MEMORY
    free(header); free(im_vals); free(data_vals);
    if (i%1000 == 0) printf("%d\n",i);
  }
  // CLOSE FILES AND FREE OTHER MEMORY
  fclose(psf_in); fclose(psf_out);
  free(mean_vals); free(log_mean_vals); free(mean_vals_temp); free(mean_vals_all);
}
  
