/* This file read the input psf and compute the gaussian coefficients 
   of the psf projection images. */
//use makefile to compile this program

/*
ssh -f gamma-32 "/home/jean/gaussian/levmar/psf_gcoef_LM /home/jean/18X/psfjun04/psfjun04.dat /home/jean/18X/psfjun04/jun04_gcoef0.dat /home/jean/18X/psfjun04/jun04_fpos.dat /home/jean/18X/psfjun04/jun04_xyz.dat 13 13 13 78 16 > /home/jean/18X/psfjun04/foo &"

ssh -f gamma-44 "/home/jean/gaussian/levmar/psf_gcoef_LM /home/jean/PSFmar04/psfmar04.dat /home/jean/PSFmar04/mar04_gcoef0.dat /home/jean/PSFmar04/mar04_fpos.dat /home/jean/PSFmar04/mar04_xyz.dat 61 77 48 78 16 > /home/jean/PSFmar04/mar04_gcoef0 &"
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "lm.h"

//#include "sub_gcoef_LM.c"

/* 
The gaussian coefficients are defined as follows:
sum:    sum (sensitivity) of the input projection image
mx, my: centroid in x, y directions
phi:    rotation angle for principal axes [-PI/2, PI/2]
lx, ly: stand for lambda_x and lambda_y,
        eigenvalues of the covariance matrix (lx > ly)
*/
struct gcoef{
  float sum;
  float mx;
  float my;
  float phi;
  float lx;
  float ly;
};

main(int argc, char **argv)
{
  struct gcoef *gcoef;
  int i_cnt, num_coords;
  unsigned long co_cnt;
  int OD_X, OD_Y, OD_Z;
  FILE *f_psfin, *f_psfout;
  FILE *f_fpos, *f_xyz;
  int i, j, k, p, n;
  unsigned long vox_pos;
  long dim_proj, num_angs;
  unsigned short ii, jj, kk;
  unsigned long cnt;
  unsigned long *pos;
  float *val;
  float *image, *g;
  unsigned long MDRFpts;
  long fpos;
  float ctmp[6];
  time_t t0, t1, t2;
  long dt;
  
  setlinebuf(stdout);
  
  if (argc != 11){
    printf("usage: psf_gcoef_LM in_psf out_psfgcoef out_fpos out_xyz XDIM YDIM ZDIM DIM_G NUM_CAM NUM_COORDS\n");
    exit(0);
  }
  
  OD_X = atoi(argv[5]);
  OD_Y = atoi(argv[6]);
  OD_Z = atoi(argv[7]);
  dim_proj = atoi(argv[8]);
  num_angs = atoi(argv[9]);
  num_coords = atoi(argv[10]);
  printf("\n OD_X = %2d, OD_Y = %2d, OD_Z = %2d\n",OD_X,OD_Y,OD_Z);
  printf(" dim_proj = %2d, num_angs = %2d\n",dim_proj,num_angs);
  
  MDRFpts = dim_proj*dim_proj;
  
  pos = (unsigned long *)malloc(MDRFpts*num_angs*sizeof(unsigned long));
  val = (float *)malloc(MDRFpts*num_angs*sizeof(float));
  image = (float *)malloc(MDRFpts*num_angs*sizeof(float));
  g = (float *)malloc(MDRFpts*sizeof(float));
  gcoef = (struct gcoef *)malloc(num_angs*sizeof(struct gcoef));
  
  //open psf file
  f_psfin = fopen(argv[1], "r");
  if (f_psfin <= 0){
    printf("couldn't open %s\n", argv[1]);
    exit(0);
  }
  else printf(" opened the input psf file: %s\n",argv[1]);

  //open output psf gaussian coefficient file
  f_psfout = fopen(argv[2], "w");
  if (f_psfout <= 0){
    printf("couldn't open %s\n", argv[2]);
    exit(0);
  }
  else printf(" opened the output psf gaussian coef file: %s\n",argv[2]);

  //open output file-reading offset file
  f_fpos = fopen(argv[3], "w");
  if (f_fpos <= 0){
    printf("couldn't open %s\n", argv[3]);
    exit(0);
  }
  else printf(" opened the output file-reading offset file: %s\n",argv[3]);
  setbuf(f_fpos, NULL);

  //open output psf voxel position file
  f_xyz = fopen(argv[4], "w");
  if (f_xyz <= 0){
    printf("couldn't open %s\n", argv[4]);
    exit(0);
  }
  else printf(" opened the output psf voxel position file: %s\n",argv[4]);
  setbuf(f_xyz, NULL);

  k = 0;
  time(&t0);  
  fseeko(f_psfin, 0, SEEK_SET);
  //loop over psf voxels
  for (co_cnt = 0; co_cnt < num_coords; co_cnt++){

		fread(&ii, sizeof(unsigned short), 1, f_psfin);
		fread(&jj, sizeof(unsigned short), 1, f_psfin);
		fread(&kk, sizeof(unsigned short), 1, f_psfin);
		//printf("I am here\n");
		fread(gcoef, sizeof(struct gcoef), num_angs, f_psfin);
		ii = ntohs(ii); jj = ntohs(jj); kk = ntohs(kk);
		//printf("I am here1\n");
    ii;jj;kk;

    vox_pos = (long)(ii*OD_Y*OD_Z + jj*OD_Z + kk);
    vox_pos = htonl(vox_pos);
    fwrite(&vox_pos, sizeof(unsigned long), 1, f_xyz);
    
    //fpos = ftello(f_psfout);
	 fpos = co_cnt*30;
	 fpos = htonl(fpos);
    fwrite(&fpos, sizeof(long), 1, f_fpos);
    
	 /*

    //initialize an empty image array
    for (i = 0; i < MDRFpts*num_angs; i++) image[i] = 0.0;
    
    //assign psf projections to image
    for (i = 0; i < cnt; i++) image[pos[i]] = val[i];
    
    //calculate gaussian coefficient for each projection      
    for (j = 0; j < num_angs; j++){
      p = j*MDRFpts;
      for (i = 0; i < MDRFpts; i++){
	g[i] = image[p+i];
      }
      sub_gcoef_LM(g, ctmp, dim_proj);

      gcoef[j].sum = ctmp[0];
      gcoef[j].mx = ctmp[1];
      gcoef[j].my = ctmp[2];
      gcoef[j].phi = ctmp[3];
      gcoef[j].lx = ctmp[4];
      gcoef[j].ly = ctmp[5]; 
      *(long *)&gcoef[j].sum = htonl(*(long *)&gcoef[j].sum);
      *(long *)&gcoef[j].mx = htonl(*(long *)&gcoef[j].mx);
      *(long *)&gcoef[j].my = htonl(*(long *)&gcoef[j].my);
      *(long *)&gcoef[j].phi = htonl(*(long *)&gcoef[j].phi);
      *(long *)&gcoef[j].lx = htonl(*(long *)&gcoef[j].lx);
      *(long *)&gcoef[j].ly = htonl(*(long *)&gcoef[j].ly);
    }
    
    ii = htons(ii); jj = htons(jj); kk = htons(kk);
    fwrite(&ii, sizeof(unsigned short), 1, f_psfout);
    fwrite(&jj, sizeof(unsigned short), 1, f_psfout);
    fwrite(&kk, sizeof(unsigned short), 1, f_psfout);
    fwrite(gcoef, sizeof(struct gcoef), num_angs, f_psfout);
    */
	 k++;
    if ((k%1000) == 0){
      time(&t1);
      dt = (long) difftime(t1,t0);
		//      printf(" processed %6d voxels in %6d seconds\n",k,dt);
    }
  }
  printf(" read over the whole psf file\n");
  time(&t1);
  dt = (long) difftime(t1,t0);
  printf(" processed %6d voxels in %6d seconds\n",k,dt);

  fclose(f_psfin); 
  fclose(f_psfout); 
  fclose(f_fpos); fclose(f_xyz);
  free(pos); free(val); free(image); free(g); free(gcoef);
}
