/* This sub-routine averages the input gaussian coefficients
   associated with four psf voxels. */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

struct gcoef{
  float sum;
  float mx;
  float my;
  float phi;
  float lx;
  float ly;
};

sub_avgcoef4(struct gcoef *gcoef, struct gcoef *gcoef1, struct gcoef *gcoef2, struct gcoef *gcoef3, struct gcoef *gcoef4, long num_angs)
{
 int i, k;
 float ctmp[6];
 float ctmp1[6];
 float ctmp2[6];
 float ctmp3[6];
 float ctmp4[6];

 //perform interpolation
 for (i = 0; i < num_angs; i++){
   
   //assign gcoef1 to ctmp1
   ctmp1[0] = gcoef1[i].sum;
   ctmp1[1] = gcoef1[i].mx; 
   ctmp1[2] = gcoef1[i].my; 
   ctmp1[3] = gcoef1[i].phi; 
   ctmp1[4] = gcoef1[i].lx; 
   ctmp1[5] = gcoef1[i].ly;
   
   //assign gcoef2 to ctmp2
   ctmp2[0] = gcoef2[i].sum;
   ctmp2[1] = gcoef2[i].mx; 
   ctmp2[2] = gcoef2[i].my; 
   ctmp2[3] = gcoef2[i].phi; 
   ctmp2[4] = gcoef2[i].lx; 
   ctmp2[5] = gcoef2[i].ly;

   //assign gcoef3 to ctmp3
   ctmp3[0] = gcoef3[i].sum;
   ctmp3[1] = gcoef3[i].mx; 
   ctmp3[2] = gcoef3[i].my; 
   ctmp3[3] = gcoef3[i].phi; 
   ctmp3[4] = gcoef3[i].lx; 
   ctmp3[5] = gcoef3[i].ly;
   
   //assign gcoef4 to ctmp4
   ctmp4[0] = gcoef4[i].sum;
   ctmp4[1] = gcoef4[i].mx; 
   ctmp4[2] = gcoef4[i].my; 
   ctmp4[3] = gcoef4[i].phi; 
   ctmp4[4] = gcoef4[i].lx; 
   ctmp4[5] = gcoef4[i].ly;
   
   //do the averaging
   //all four images have non-zero sum
   if ((ctmp1[0]*ctmp2[0]*ctmp3[0]*ctmp4[0]) > 0.0){
     for (k = 0; k < 4; k++){
       ctmp[k] = (ctmp1[k] + ctmp2[k] + ctmp3[k] + ctmp4[k])/4.0;
     }
     for (k = 4; k < 6; k++){
       ctmp[k] = sqrt(sqrt(ctmp1[k]*ctmp2[k]*ctmp3[k]*ctmp4[k]));
     }
   }
   //average by a diagonal line of the square
   else if ((ctmp1[0]*ctmp2[0]) > 0.0){
     for (k = 0; k < 4; k++){
       ctmp[k] = (ctmp1[k] + ctmp2[k])/2.0;
     }
     for (k = 4; k < 6; k++){
       ctmp[k] = sqrt(ctmp1[k]*ctmp2[k]);
     }
   }
   else if ((ctmp3[0]*ctmp4[0]) > 0.0){
     for (k = 0; k < 4; k++){
       ctmp[k] = (ctmp3[k] + ctmp4[k])/2.0;
     }
     for (k = 4; k < 6; k++){
       ctmp[k] = sqrt(ctmp3[k]*ctmp4[k]);
     }
   }
   //average by a side line of the square
   else if ((ctmp1[0]*ctmp3[0]) > 0.0){
     ctmp[0] = (ctmp1[0] + ctmp3[0])/4.0;
     for (k = 1; k < 4; k++){
       ctmp[k] = (ctmp1[k] + ctmp3[k])/2.0;
     }
     for (k = 4; k < 6; k++){
       ctmp[k] = sqrt(ctmp1[k]*ctmp3[k]);
     }
   }
   else if ((ctmp2[0]*ctmp4[0]) > 0.0){
     ctmp[0] = (ctmp2[0] + ctmp4[0])/4.0;
     for (k = 1; k < 4; k++){
       ctmp[k] = (ctmp2[k] + ctmp4[k])/2.0;
     }
     for (k = 4; k < 6; k++){
       ctmp[k] = sqrt(ctmp2[k]*ctmp4[k]);
     }
   }
   //only one image has non-zero sum
   else if (ctmp1[0] > 0.0){
     ctmp[0] = ctmp1[0]/4.0;
     for (k = 1; k < 6; k++){
       ctmp[k] = ctmp1[k];
     }
   }
   else if (ctmp2[0] > 0.0){
     ctmp[0] = ctmp2[0]/4.0;
     for (k = 1; k < 6; k++){
       ctmp[k] = ctmp2[k];
     }
   }
   else if (ctmp3[0] > 0.0){
     ctmp[0] = ctmp3[0]/4.0;
     for (k = 1; k < 6; k++){
       ctmp[k] = ctmp3[k];
     }
   }
   else if (ctmp4[0] > 0.0){
     ctmp[0] = ctmp4[0]/4.0;
     for (k = 1; k < 6; k++){
       ctmp[k] = ctmp4[k];
     }
   }
   //all 4 images are null
   else {
     for (k = 0; k < 6; k++){
       ctmp[k] = 0.0;
     }
   }
   
   //assign ctmp to gcoef
   gcoef[i].sum = ctmp[0];
   gcoef[i].mx = ctmp[1];
   gcoef[i].my = ctmp[2];
   gcoef[i].phi = ctmp[3];
   gcoef[i].lx = ctmp[4];
   gcoef[i].ly = ctmp[5];
 }
}


