/* This sub-routine averages the input gaussian coefficients
   associated with eight psf voxels. */
//pairs: (1,2) (3,8) (4,7) (5,6)

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

struct gcoef{
  float sum;
  float mx;
  float my;
  float phi;
  float lx;
  float ly;
};

sub_avgcoef8(struct gcoef *gcoef, struct gcoef *gcoef1, struct gcoef *gcoef2, struct gcoef *gcoef3, struct gcoef *gcoef4, struct gcoef *gcoef5, struct gcoef *gcoef6, struct gcoef *gcoef7, struct gcoef *gcoef8, long num_angs)
{
 int i, k;
 float ctmp[6];
 float ctmp1[6];
 float ctmp2[6];
 float ctmp3[6];
 float ctmp4[6];
 float ctmp5[6];
 float ctmp6[6];
 float ctmp7[6];
 float ctmp8[6];
 float pts;

 //perform interpolation
 for (i = 0; i < num_angs; i++){
   
   //assign gcoef1 to ctmp1
   ctmp1[0] = gcoef1[i].sum;
   ctmp1[1] = gcoef1[i].mx; 
   ctmp1[2] = gcoef1[i].my; 
   ctmp1[3] = gcoef1[i].phi; 
   ctmp1[4] = gcoef1[i].lx; 
   ctmp1[5] = gcoef1[i].ly;
   
   //assign gcoef2 to ctmp2
   ctmp2[0] = gcoef2[i].sum;
   ctmp2[1] = gcoef2[i].mx; 
   ctmp2[2] = gcoef2[i].my; 
   ctmp2[3] = gcoef2[i].phi; 
   ctmp2[4] = gcoef2[i].lx; 
   ctmp2[5] = gcoef2[i].ly;

   //assign gcoef3 to ctmp3
   ctmp3[0] = gcoef3[i].sum;
   ctmp3[1] = gcoef3[i].mx; 
   ctmp3[2] = gcoef3[i].my; 
   ctmp3[3] = gcoef3[i].phi; 
   ctmp3[4] = gcoef3[i].lx; 
   ctmp3[5] = gcoef3[i].ly;
   
   //assign gcoef4 to ctmp4
   ctmp4[0] = gcoef4[i].sum;
   ctmp4[1] = gcoef4[i].mx; 
   ctmp4[2] = gcoef4[i].my; 
   ctmp4[3] = gcoef4[i].phi; 
   ctmp4[4] = gcoef4[i].lx; 
   ctmp4[5] = gcoef4[i].ly;

   //assign gcoef5 to ctmp5
   ctmp5[0] = gcoef5[i].sum;
   ctmp5[1] = gcoef5[i].mx; 
   ctmp5[2] = gcoef5[i].my; 
   ctmp5[3] = gcoef5[i].phi; 
   ctmp5[4] = gcoef5[i].lx; 
   ctmp5[5] = gcoef5[i].ly;
   
   //assign gcoef6 to ctmp6
   ctmp6[0] = gcoef6[i].sum;
   ctmp6[1] = gcoef6[i].mx; 
   ctmp6[2] = gcoef6[i].my; 
   ctmp6[3] = gcoef6[i].phi; 
   ctmp6[4] = gcoef6[i].lx; 
   ctmp6[5] = gcoef6[i].ly;

   //assign gcoef7 to ctmp7
   ctmp7[0] = gcoef7[i].sum;
   ctmp7[1] = gcoef7[i].mx; 
   ctmp7[2] = gcoef7[i].my; 
   ctmp7[3] = gcoef7[i].phi; 
   ctmp7[4] = gcoef7[i].lx; 
   ctmp7[5] = gcoef7[i].ly;
   
   //assign gcoef8 to ctmp8
   ctmp8[0] = gcoef8[i].sum;
   ctmp8[1] = gcoef8[i].mx; 
   ctmp8[2] = gcoef8[i].my; 
   ctmp8[3] = gcoef8[i].phi; 
   ctmp8[4] = gcoef8[i].lx; 
   ctmp8[5] = gcoef8[i].ly;

   //initialize ctmp
   pts = 0.0;
   for (k = 0; k < 4; k++)
     ctmp[k] = 0.0;
   for (k = 4; k < 6; k++)
     ctmp[k] = 1.0;
   
   //check each diagonal line of the cube
   if ((ctmp1[0]*ctmp2[0]) > 0.0){
     pts += 2.0;
     for (k = 0; k < 4; k++)
       ctmp[k] += (ctmp1[k] + ctmp2[k]);
     for (k = 4; k < 6; k++)
       ctmp[k] *= (ctmp1[k]*ctmp2[k]);
   }
   if ((ctmp3[0]*ctmp8[0]) > 0.0){
     pts += 2.0;
     for (k = 0; k < 4; k++)
       ctmp[k] += (ctmp3[k] + ctmp8[k]);
     for (k = 4; k < 6; k++)
       ctmp[k] *= (ctmp3[k]*ctmp8[k]);
   }
   if ((ctmp4[0]*ctmp7[0]) > 0.0){
     pts += 2.0;
     for (k = 0; k < 4; k++)
       ctmp[k] += (ctmp4[k] + ctmp7[k]);
     for (k = 4; k < 6; k++)
       ctmp[k] *= (ctmp4[k]*ctmp7[k]);
   }
   if ((ctmp5[0]*ctmp6[0]) > 0.0){
     pts += 2.0;
     for (k = 0; k < 4; k++)
       ctmp[k] += (ctmp5[k] + ctmp6[k]);
     for (k = 4; k < 6; k++)
       ctmp[k] *= (ctmp5[k]*ctmp6[k]);
   }
   
   //do the averaging
   if (pts > 0.0){
     for (k = 0; k < 4; k++)
       ctmp[k] = ctmp[k]/pts;
     for (k = 4; k < 6; k++)
       ctmp[k] = (float) pow(ctmp[k],1.0/pts);
   }
   
   //if no diagonal lines exist at all, check each corner of the cube
   else {
     if (ctmp1[0] > 0.0){
       pts += 1.0;
       for (k = 0; k < 4; k++) ctmp[k] += ctmp1[k];
       for (k = 4; k < 6; k++) ctmp[k] *= ctmp1[k];
     }
     if (ctmp2[0] > 0.0){
       pts += 1.0;
       for (k = 0; k < 4; k++) ctmp[k] += ctmp2[k];
       for (k = 4; k < 6; k++) ctmp[k] *= ctmp2[k];
     }
     if (ctmp3[0] > 0.0){
       pts += 1.0;
       for (k = 0; k < 4; k++) ctmp[k] += ctmp3[k];
       for (k = 4; k < 6; k++) ctmp[k] *= ctmp3[k];
     }
     if (ctmp4[0] > 0.0){
       pts += 1.0;
       for (k = 0; k < 4; k++) ctmp[k] += ctmp4[k];
       for (k = 4; k < 6; k++) ctmp[k] *= ctmp4[k];
     }
     if (ctmp5[0] > 0.0){
       pts += 1.0;
       for (k = 0; k < 4; k++) ctmp[k] += ctmp5[k];
       for (k = 4; k < 6; k++) ctmp[k] *= ctmp5[k];
     }
     if (ctmp6[0] > 0.0){
       pts += 1.0;
       for (k = 0; k < 4; k++) ctmp[k] += ctmp6[k];
       for (k = 4; k < 6; k++) ctmp[k] *= ctmp6[k];
     }
     if (ctmp7[0] > 0.0){
       pts += 1.0;
       for (k = 0; k < 4; k++) ctmp[k] += ctmp7[k];
       for (k = 4; k < 6; k++) ctmp[k] *= ctmp7[k];
     }
     if (ctmp8[0] > 0.0){
       pts += 1.0;
       for (k = 0; k < 4; k++) ctmp[k] += ctmp8[k];
       for (k = 4; k < 6; k++) ctmp[k] *= ctmp8[k];
     }
     //do the average
     if (pts > 0.0){
       ctmp[0] = ctmp[0]/pts/2.0;
       for (k = 1; k < 4; k++)
	 ctmp[k] = ctmp[k]/pts;
       for (k = 4; k < 6; k++)
	 ctmp[k] = (float) pow(ctmp[k],1.0/pts);
     }
     //all images are null
     else {
       for (k = 0; k < 6; k++)
	 ctmp[k] = 0.0;
     }
   }
   
   //assign ctmp to gcoef
   gcoef[i].sum = ctmp[0];
   gcoef[i].mx = ctmp[1];
   gcoef[i].my = ctmp[2];
   gcoef[i].phi = ctmp[3];
   gcoef[i].lx = ctmp[4];
   gcoef[i].ly = ctmp[5];
 }
}


