/* This subroutine classifies the interpolation type
for center-point response of one cube.
Input: id[9], condition
Output: case number
*/

#include <stdio.h>
#include <stdlib.h>

int compare(const void *a, const void *b);

short sub_classify(long id[9], long condition)
{
long *match;
short elem_idx;
short case_num = -1;

long cond[47] = {
  1,2,3,5,6,7,10,11,13,14,15,17,21,22,26,33,34,35,39,51,
  55,65,77,85,91,102,119,130,143,154,187,195,221,231,255,
  357,385,455,561,663,715,1001,1105,1309,3315,3927,5005
};
short cases[47] = {
  1,2,3,4,6,5, 7, 5, 4, 8, 9, 3,10, 8, 7,10, 6,11, 9, 3,
  11, 4, 5, 9,11,  6, 10,  7, 11,  8, 10,  9,  9, 10,  9,
   10, 11, 11, 10,  9, 11,  11,   9,  10,   9,  10,  11
};

 match = (long *) bsearch(&condition, cond, 47, sizeof(long), compare);

 if (match != NULL){
   elem_idx = match - &cond[0];
   case_num = cases[elem_idx];
 }
 else {
   if (id[2] == 1) case_num = 12;
   else if ((id[3] == 1) && (id[8] == 1)) case_num = 13;
   else if ((id[4] == 1) && (id[7] == 1)) case_num = 14;
   else if ((id[5] == 1) && (id[6] == 1)) case_num = 15;
 }

 return case_num;
}


int compare(const void *a, const void *b)
{
  return *(int *)a - *(int *)b;
}


/* Cases for interpolation

At least one pair of diagonal pairs (1,2), (3,8), (4,7),
and (5,6) must exist to perform interpolation.

point 1 always exists.

point 2, 3, 4, 5, 6, 7, 8
id#   2, 3, 5, 7,11,13,17 if point doesn't exist.
id# = 1 if point exists.

condition# = product of all id#'s.

situation	case#	pairs missing	condition#
missing 0 pair	 1	N/A		1

missing 1 pair	 2	(1,2)		2
		 3	(3,8)		3,17,51
		 4	(4,7)		5,13,65
		 5	(5,6)		7,11,77

missing 2 pairs	 6	(1,2), (3,8)	6,34,102
		 7	(1,2), (4,7)	10,26,130
		 8	(1,2), (5,6)	14,22,154
		 9	(3,8), (4,7)	15,39,85,195,221,255,663,1105,3315
		10	(3,8), (5,6)	21,33,119,187,231,357,561,1309,3927
		11	(4,7), (5,6)	35,55,91,143,385,455,715,1001,5005

if not any of above cases:
			pair remained	conditions on id#
missing 3 pair	12	(1,2)		id[2] == 1
		13	(3,8)		id[3] == id[8] == 1
		14	(4,7)		id[4] == id[7] == 1
		15	(5,6)		id[5] == id[6] == 1

Therefore there are 15 cases, and there are at most
128 possible numbers for condition numbers (2^7 =  128)
to perform interpolation.

Another way to classify cases 12 ~ 15:

			pair missing		condition#
missing 3 pair	12	(1,2), (3,8), (4,7)	2*(15,39,85,195,221,255,663,1105,3315)
		13	(1,2), (3,8), (5,6)	2*(21,33,119,187,231,357,561,1309,3927)
		14	(1,2), (4,7), (5,6)	2*(35,55,91,143,385,455,715,1001,5005)
		15	(3,8), (4,7), (5,6)	27 numbers

These 27 numbers are:	 105, 165, 273, 429, 595, 935,1155,1365,1547,1785
			2145,2431,2805,3003,4641,6545,7293,7735,12155,15015
			17017,19635,23205,36465,51051,85085,255255

*/
