/*This program tranforms the input image into a gaussian blob
  based on input gaussian coefficients.

  The gaussian coefficients are defined as following:
  sum:    sum (sensitivity) of the input projection image
  mx, my: centroid in x, y directions
  phi:    rotation angle for principal axes [-PI/2, PI/2]
  lx, ly: stand for lambda_x and lambda_y,
          eigenvalues of the covariance matrix (lx > ly)
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

sub_gblob(float *g, float *gcoef, int L)
{
  int i, j;
  float sum_g, mx, my, phi, lx, ly;
  float k11, k22, k12;
  float amp, argtmp, argxy, limit;
  float pi;
  float cos2phi, sin2phi;
  float sens = 0.0;
  float factor;

  sum_g = gcoef[0];
  
  if (sum_g > 0.0){
    
    mx = gcoef[1];
    my = gcoef[2];
    phi = gcoef[3];
    lx = gcoef[4];
    ly = gcoef[5];
    
    pi = (float) acos(-1.0);
    cos2phi = (float) cos((double) (2.0*phi));
    sin2phi = (float) sin((double) (2.0*phi));
    
    k11 = 0.5*(lx + ly) - 0.5*(ly - lx)*cos2phi;
    k22 = 0.5*(lx + ly) + 0.5*(ly - lx)*cos2phi;
    k12 = 0.5*(ly - lx)*sin2phi;
    
    amp = sum_g/2.0/pi/sqrt(lx*ly);
    argtmp = -0.5/(lx*ly);
    limit = 9.0*(lx*ly); //ignore everything outside 3 sigma
    
    for (i = 0; i < L; i++)
      for (j = 0; j < L; j++){
	argxy = (k22*(((float) i) - mx)*(((float) i) - mx)
		 + k11*(((float) j) - my)*(((float) j) - my)
		 - 2.0*k12*(((float) i) - mx)*(((float) j) - my));
	
	if (argxy > limit) g[i*L+j] = 0.0;
	else g[i*L+j] = amp*exp(argtmp*argxy);
	sens += g[i*L+j];
      }
    //sensitivity correction
    factor = sum_g/sens;
    for (i = 0; i < L*L; i++) g[i] *= factor;
    //printf("sensitivity correction factor = %6.4f\n",factor);
  }
  else {
    for (i = 0; i < L*L; i++) g[i] = 0.0;
  }
}
