/*This program calculates the gaussian coefficients
  of the input image.

  The gaussian coefficients are defined as following:
  sum: sum (sensitivity) of the input projection image
  mx, my: centroid in x, y directions
  phi: rotation angle for principal axes [-PI/2, PI/2]
  lx, ly: stand for lambda_x and lambda_y,
          eigenvalues of the covariance matrix (lx > ly)
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

sub_gcoef(float *g, float *gcoef, int L)
{
  int i, j;

  float mx = 0.0;
  float my = 0.0;
  float phi = 0.0;
  float lx = 0.0;
  float ly = 0.0;

  float k11 = 0.0;
  float k12 = 0.0;
  float k22 = 0.0;
  double theta = 0.0;
  float pi;
  float thres;

  float sum_g = 0.0;
  float sum_x = 0.0;
  float sum_y = 0.0;
  float sum_xx = 0.0;
  float sum_yy = 0.0;
  float sum_xy = 0.0;

  for (i = 0; i < L*L; i++) sum_g += g[i];

  //calculate gaussian coefficients
  if (sum_g > 0.0){
    for (i = 0; i < L; i++){
      for (j = 0; j < L; j++){
	sum_x += g[i*L+j]*((float) i);
	sum_y += g[i*L+j]*((float) j);
	sum_xx += g[i*L+j]*((float) (i*i));
	sum_yy += g[i*L+j]*((float) (j*j));
	sum_xy += g[i*L+j]*((float) (i*j));
      }
    }
    mx = sum_x/sum_g;
    my = sum_y/sum_g;
    k11 = sum_xx/sum_g - mx*mx;
    k22 = sum_yy/sum_g - my*my;
    k12 = sum_xy/sum_g - mx*my;

    pi = (float) acos(-1.0);
    thres = 1.0/((float) (L*L));
    //assume the standard deviation should be smaller than L

    if (k12 == 0.0){
      if (k11 >= k22){
	lx = k11;
	ly = k22;
	phi = 0.0;
      }
      else {
	lx = k22;
	ly = k11;
	phi = pi/2.0;
      }
    }
    else {
      if (k11 == k22){
	if (k12 < 0.0){
	  lx = k11 - k12;
	  ly = k11 + k12;
	  phi = pi/4.0;
	}
	else {
	  lx = k11 + k12;
	  ly = k11 - k12;
	  phi = -pi/4.0;
	}
      }
      else {
	theta = (0.5*atan2((double) (-2.0*k12),(double) (k11-k22)));
	lx = 0.5*(k11 + k22 - 2.0*k12/((float) sin(2.0*theta)));
	ly = 0.5*(k11 + k22 + 2.0*k12/((float) sin(2.0*theta)));
	phi = (float) theta; 
      }
    }
    //remove unreasonable lambda's
    if (lx <= thres) lx = 1.0;
    if (ly <= thres) ly = 1.0;
  }

  gcoef[0] = sum_g;
  gcoef[1] = mx;
  gcoef[2] = my;
  gcoef[3] = phi;
  gcoef[4] = lx;
  gcoef[5] = ly;
}
