#!/usr/bin/perl -w

use strict;

if (@ARGV != 6) {
    print <<'USAGE';
Usage:
  ProcessImage <ImageFileBase> <Cameras> <Dimension> <Number Angles> <MDRF Directory> <Output Directory>

Position estimation with likelihood windowing is performed on image data and the resulting
projection images are prepared for reconstruction.
For a more detailed description, please see the documentation.

USAGE
exit;
}

my $ImgFileTotal = shift;
my @Cams = split(//,shift);
my $Dim = shift;
my $NumAngs = shift;
my $MDRFDir = shift;
$MDRFDir =~ s|/$||g;
my $OutDir = shift;
$OutDir =~ s|/$||g;

# Get all of the image file folders and names in order.
(my $ImgFilePath, my $ImgFile, my $ImgExt) = $ImgFileTotal =~ m|^(.*/)?(.*?)(\.[^\.]*)?$|;
my $ComboImgFile = $ImgFile."_cam_".join("",@Cams);
my $ImgFileIn = $ImgFileTotal."_0";
my $ImgFileOut = $ImgFile."_0";

`mkdir $OutDir` unless -d $OutDir;

foreach my $elem (@Cams) {
    if (-f "$OutDir/$ImgFileOut$elem.img") {
	print "Projection image file : $ImgFileOut$elem.img exists\n";
	print "Position estimation is being skipped\n";
    } else {
	print "Generating projection images for camera $elem...\n";
    `psf_LW_cruncher $MDRFDir/MeanFile_0$elem $MDRFDir/Log_Fact $MDRFDir/FiltThresholds_0$elem 8 $ImgFileIn$elem.dat $OutDir/$ImgFileOut$elem.img $NumAngs $Dim 1 >/dev/null`;
    }
}

my $cam_length = $#Cams + 1;
print "Preparing projection images for reconstruction...\n";
my $cmd = "proj_prep_psf_style_data $OutDir/ $ImgFile $Dim $NumAngs $cam_length ".join(" ",@Cams);
my $tmpFile = "all_".$ComboImgFile;
`$cmd`;




