#!/usr/bin/perl -w

use strict;

if (@ARGV != 7) {
    print <<'USAGE';
Usage:
  ProcessInterpPSF <PSFInFile> <Cameras> <Dimension> <Number Coordinates> <Radial Steps> <Z Steps> <Output Directory>

A routine to interpolate a single-pinhole PSF using Jean Chen's Gaussian Interpolation Code.
For a more detailed description, please see the documentation.

USAGE
exit;
}

my $PSFInFile = shift;
my @Cams = split(//,shift);
my $Dim = shift;
my $Coords = shift;
my $N = shift;
my $z = shift;
my $OutDir = shift;

# Make OutDir a proper directory
$OutDir =~ s|/$||g;

`mkdir $OutDir` unless -d $OutDir;

foreach my $elem (@Cams) {

my $tmpfile = "$OutDir/psf_0$elem"."_gcoef";
if (-f "$tmpfile") {
print "Coefficient file : $tmpfile exists\n";
print "Computation of coefficients is being skipped\n";
} else {
  print "Generating Gaussian coefficients...\n";
  my $cmd = "psf_gcoef_LM_2 $PSFInFile $OutDir/psf_0$elem"."_gcoef"." $OutDir/fpos_0$elem $OutDir/xyz_0$elem $N $N $z $Dim 1 $Coords";
  `$cmd`;
}

print "Generating indices...\n";
`gen_xyzidx $OutDir/xyz_0$elem $OutDir/xyzidx_0$elem $N $N $z` unless -f "$OutDir/xyzidx_0"."$elem";

print "Interpolating Gaussian coefficients...\n";
my $cmd = "interp_gcoef1 $OutDir/psf_0$elem"."_gcoef"." $OutDir/psf_0$elem"."_gcoef1 $N $N $z $Dim 1";
$tmpfile = "$OutDir/psf_0$elem"."_gcoef1";
`$cmd` unless -f $tmpfile;

$cmd = "interp_gcoef2 $OutDir/psf_0$elem"."_gcoef $OutDir/xyzidx_0$elem $OutDir/fpos_0$elem $OutDir/psf_0$elem"."_gcoef2 $N $N $z $Dim 1";
$tmpfile = "$OutDir/psf_0$elem"."_gcoef2";
`$cmd` unless -f $tmpfile;

$cmd = "interp_gcoef4 $OutDir/psf_0$elem"."_gcoef $OutDir/xyzidx_0$elem $OutDir/fpos_0$elem $OutDir/psf_0$elem"."_gcoef4 $N $N $z $Dim 1";
$tmpfile = "$OutDir/psf_0$elem"."_gcoef4";
`$cmd` unless -f $tmpfile;

$cmd = "interp_gcoef8 $OutDir/psf_0$elem"."_gcoef $OutDir/xyzidx_0$elem $OutDir/fpos_0$elem $OutDir/psf_0$elem"."_gcoef8 $N $N $z $Dim 1";
$tmpfile = "$OutDir/psf_0$elem"."_gcoef8";
`$cmd` unless -f $tmpfile;

print "Combining Gaussian coefficients...\n";
$cmd = "comb_gcoef $OutDir/psf_0"."$elem"."_gcoef $OutDir/psf_0$elem"."_gcoef_comb 1";
`$cmd`;

my $N_interp = $N*2-1;
my $z_interp = $z*2-1;

print "Sorting interpolated coefficients...\n";
$cmd = "gcoef_vox_sort $OutDir/psf_0$elem"."_gcoef_comb $OutDir/psf_0$elem"."_gcoef_sorted $N_interp $N_interp $z_interp 1";
my $output = `$cmd`;

(my $num_voxels) = ($output =~ m/number of voxel = (.*)$/g);

print "There will be $num_voxels voxels in the interpolated PSF\n";

print "Generating interpolated PSF...\n";
$cmd = "gen_gauspsf $OutDir/psf_0$elem"."_gcoef_sorted $OutDir/psf_0$elem"."_interp.dat $N_interp $N_interp $z_interp $Dim 1 0 $num_voxels";
`$cmd`;

print "Sorting interpolated PSF files...\n";
$cmd = "PSF_sort_float $OutDir/ psf_0$elem"."_interp.dat $elem $num_voxels $Dim";
`$cmd`;

}




