#!/usr/bin/perl -w

use strict;

if (@ARGV != 8) {
    print <<'USAGE';
Usage:
  ProcessInterp <PSFInFile> <CoordsFile> <PinholeFile> <Dimension> <Number Coordinates> <Radial Steps> <Z Steps> <Output Directory Base>

A routine to interpolate a multiple-pinhole PSF using Jacob's multiple-pinhole code in combination with
Jean Chen's Gaussian Interpolation Code.
For a more detailed description, please see the documentation.

USAGE
exit;
}

my $PSFInFile = shift;
my $CoordsFile = shift;
my $PHFile = shift;
my $Dim = shift;
my $Coords = shift;
my $N = shift;
my $z = shift;
my $OutDir = shift;

# Make OutDir a proper directory
$OutDir =~ s|/$||g;

`mkdir $OutDir` unless -d $OutDir;

# Read in all of the data from the pinhole information file.
open(PH_DATA, $PHFile) || die("Could not open pinhole information file!");
my $raw_data = <PH_DATA>;
chop($raw_data);
(my $text,my $Cam) = split(/:/,$raw_data);

$raw_data = <PH_DATA>;
chop($raw_data);
($text,my $PSF_step) = split(/:/,$raw_data);

$raw_data = <PH_DATA>;
chop($raw_data);
($text,my $MDRF_step) = split(/:/,$raw_data);

$raw_data = <PH_DATA>;
chop($raw_data);
($text,my $ph_num) = split(/:/,$raw_data);

$raw_data = <PH_DATA>;
chop($raw_data);
($text,my $x_p_array) = split(/:/,$raw_data);
my @x_p = split(/,/,$x_p_array);
$x_p[0] =~ s/\[//g;
$x_p[$ph_num-1] =~ s/\]//g;

$raw_data = <PH_DATA>;
chop($raw_data);
($text,my $y_p_array) = split(/:/,$raw_data);
my @y_p = split(/,/,$y_p_array);
$y_p[0] =~ s/\[//g;
$y_p[$ph_num-1] =~ s/\]//g;

$raw_data = <PH_DATA>;
chop($raw_data);
($text,my $O_P_dist) = split(/:/,$raw_data);

$raw_data = <PH_DATA>;
chop($raw_data);
($text,my $P_D_dist) = split(/:/,$raw_data);

$raw_data = <PH_DATA>;
chop($raw_data);
($text,my $x_corr) = split(/:/,$raw_data);

$raw_data = <PH_DATA>;
chop($raw_data);
($text,my $y_corr) = split(/:/,$raw_data);

$raw_data = <PH_DATA>;
chop($raw_data);
($text,my $z_corr) = split(/:/,$raw_data);

$raw_data = <PH_DATA>;
chop($raw_data);
($text,my $Range) = split(/:/,$raw_data);

$raw_data = <PH_DATA>;
chop($raw_data);
($text,my $Sum_Chk) = split(/:/,$raw_data);


close(PH_DATA);

my $OutDir_Interp = $OutDir . "_Interp";
`mkdir $OutDir_Interp` unless -d $OutDir_Interp;

my @PSF_File_Array = split(/\//,$PSFInFile);
my $PSF_Array_Length = @PSF_File_Array;
my $PSFSplitBase = $PSF_File_Array[$PSF_Array_Length-1];

for (my $count = 1; $count < $ph_num+1; $count++){
  my $CurrDir = "$OutDir/interp_cam_"."$Cam"."_ph_"."$count";
  `mkdir $CurrDir` unless -d $CurrDir;

  print "Generating predicted centers...\n";
  my $cmd = "Predict_PH_Centers $CoordsFile $CurrDir/pred_cens $Coords $O_P_dist $P_D_dist $x_p[$count-1] $y_p[$count-1] $Dim $MDRF_step $PSF_step $Cam $x_corr $y_corr $z_corr";
  `$cmd`;
  
  my $PSFSplitFile = $CurrDir . "/" . $PSFSplitBase;
  $PSFSplitFile =~ s/.dat/_ph_$count.dat/;
  print "Splitting PSF...\n";
  $cmd = "Gen_Split_PSF $PSFInFile $PSFSplitFile $CoordsFile $CurrDir/pred_cens $Cam $count $Dim $Coords $Range $Sum_Chk";
  `$cmd`;
  print "$cmd\n";

  print "Interpolating Split PSF...\n";
  $cmd = "~/M3R/Code/Code_Library/utils/ProcessInterp.pl $PSFSplitFile $Cam $Dim $Coords $N $z $CurrDir";
  `$cmd`;

}

#my $PSFBaseTotal = $PSFSplitBase;
#(my $PSFBase, $text) = split(/\./,$PSFBaseTotal);
print "Recombining interpolated PSF files...\n";
my $cmd = "Recombine_PH $OutDir $Cam $ph_num $Coords $Dim";
print "$cmd\n";
`$cmd`;






