#!/usr/bin/perl -w

use strict;

if (@ARGV != 5) {
    print <<'USAGE';
Usage:
  ProcessMDRF <MDRFFile> <Camera> <Dimension> <Cutoff> <OutputDirectory>

This routine takes a raw MDRF file and produces a mean file and a likelihood thresholds file.
For a more detailed description, please see the documentation.

USAGE
exit;
}

my $MDRFFile = shift;
my $Cam = shift;
my $Dim = shift;
my $Cutoff = shift;
my $OutDir = shift;

# Make OutDir a proper directory
$OutDir =~ s|/$||g;

`mkdir $OutDir` unless -d $OutDir;

print "Generating pre-smoothed mean file...\n";
`GenMean $MDRFFile $OutDir/MeanFilePreSmooth_0$Cam $Dim >/dev/null`;

print "Smoothing mean file...\n";
`pfit2ad_interp $OutDir/MeanFilePreSmooth_0$Cam 0 $Dim $Dim 9 3 5 15 $Dim $Dim $OutDir/MeanFile_0$Cam`;

print "Generating supplemental files...\n";
`GenLogFact $OutDir/Log_Fact`;

print "Generating likelihoods...\n";
`Gen_LW_int $OutDir/MeanFile_0$Cam $OutDir/Log_Fact $MDRFFile $OutDir/MDRFImage_0$Cam $OutDir/Like_0$Cam $OutDir/Pos_0$Cam $Dim 8.0`;

print "Sorting likelihoods...\n";
`sort_like_thresh $OutDir/Pos_0$Cam $OutDir/Like_0$Cam $OutDir/Thresholds_0$Cam $Dim $Cutoff`;

print "Median filtering the likelihood thresholds...\n";
`medfilt_2d_float $OutDir/Thresholds_0$Cam $OutDir/FiltThresholds_0$Cam $Dim 3`;
