#!/usr/bin/perl -w

use strict;

if (@ARGV != 10) {
    print <<'USAGE';
Usage:
  ProcessRecon <ImageFileBase> <Cameras> <Dimension> <Number Coordinates> <Number Angles> <Number Subsets> <Number Iterations> <MDRF Directory> <PSF Directory> <Output Directory>

Position estimation with likelihood windowing is performed on image data and the resulting
projection images are used with the PSF files to produce a reconstruction of the original object.
For a more detailed description, please see the documentation.

USAGE
exit;
}

my $ImgFileTotal = shift;
my @Cams = split(//,shift);
my $Dim = shift;
my $Coords = shift;
my $NumAngs = shift;
my $NumSets = shift;
my $NumIters = shift;
my $MDRFDir = shift;
$MDRFDir =~ s|/$||g;
my $PSFDir = shift;
$PSFDir =~ s|/$||g;
my $OutDir = shift;

# Make OutDir a proper directory
# $OutDir =~ s|([^/])$|$1/|g;
$OutDir =~ s|/$||g;

# Get all of the image file folders and names in order.
(my $ImgFilePath, my $ImgFile, my $ImgExt) = $ImgFileTotal =~ m|^(.*/)?(.*?)(\.[^\.]*)?$|;
my $ComboImgFile = $ImgFile."_cam_".join("",@Cams);
my $ImgFileIn = $ImgFileTotal."_0";
my $ImgFileOut = $ImgFile."_0";

`mkdir $OutDir` unless -d $OutDir;

foreach my $elem (@Cams) {
    if (-f "$OutDir/$ImgFileOut$elem.img") {
        print "Projection image file : $ImgFileOut$elem.img exists\n";
        print "Position estimation is being skipped\n";
    } else {
        print "Generating projection images for camera $elem...\n";
    `psf_LW_cruncher $MDRFDir/MeanFile_0$elem $MDRFDir/Log_Fact $MDRFDir/FiltThresholds_0$elem 8 $ImgFileIn$elem.dat $OutDir/$ImgFileOut$elem.img $NumAngs $Dim 1 >/dev/null`;
    }
}


print "Preparing projection images for reconstruction...\n";
my $cam_length = $#Cams + 1;
my $cmd = "proj_prep_psf_style_data $OutDir/ $ImgFile $Dim $NumAngs $cam_length ".join(" ",@Cams);
my $tmpFile = "all_".$ComboImgFile;
`$cmd`;


if (-f "$PSFDir/coordsbank_$NumAngs") {
    print "Coordinates file : $PSFDir/coordsbank_$NumAngs exists\n";
    print "Coordinate bank generation is being skipped\n";
} else {
    print "Preparing coordinates file...\n";
    `gen_coords_bank $PSFDir/ coords_0$Cams[0] $Coords $NumAngs`;
}

print "Generating sensitivity file...\n";
$cmd = "gen_sn_files $PSFDir/ $Coords $cam_length ".join(" ",@Cams);
`$cmd`;

if (-f "$OutDir/recon_$ComboImgFile") {
    print "Image file : $OutDir/recon_$ComboImgFile exists\n";
    print "Reconstruction is being skipped\n";
} else {
    printf "Reconstructing...\n";
    $cmd = "OSEM_".scalar(@Cams)."cam $PSFDir $OutDir/all_$ComboImgFile $OutDir/recon_$ComboImgFile $Coords $Dim $NumAngs $NumSets $NumIters ".join(" ",@Cams)." >/dev/null";
    `$cmd`;

    printf "Preparing reconstruction for Amide...\n";
    `prep_recon_3D $OutDir/ recon_$ComboImgFile $PSFDir/coords_0$Cams[0] $NumIters $Coords`;
}
