#!/usr/bin/perl -w

use strict;

if (@ARGV != 7) {
    print <<'USAGE';
Usage:
  ProcessImage <ImageFileBase> <Cameras> <Dimension> <Number Angles> <Percent Events>  <MDRF Directory> <Output Directory>

Position estimation with likelihood windowing is performed on image data and the resulting
projection images are prepared for reconstruction.
For a more detailed description, please see the documentation.

USAGE
exit;
}

my $ImgFileTotal = shift;
my @Cams = split(//,shift);
my $Dim = shift;
my $NumAngs = shift;
my $PctEvts = shift;
my $MDRFDir = shift;
$MDRFDir =~ s|/$||g;
my $OutDir = shift;
$OutDir =~ s|/$||g;

# Get all of the image file folders and names in order.
(my $ImgFilePath, my $ImgFile, my $ImgExt) = $ImgFileTotal =~ m|^(.*/)?(.*?)(\.[^\.]*)?$|;
my $ComboImgFile = $ImgFile."_cam_".join("",@Cams);
my $ImgFileIn = $ImgFileTotal."_0";
my $ImgFileOut = $ImgFile."_".$PctEvts."_0";
$ImgFileOut =~ s/\./p/g;
my $ImgFilePrep = $ImgFile."_".$PctEvts;
$ImgFilePrep =~ s/\./p/g;

`mkdir $OutDir` unless -d $OutDir;

foreach my $elem (@Cams) {
    print "Generating projection images for camera $elem...\n";
    `ML_sig_psf_style $MDRFDir/MeanFile_0$elem $MDRFDir/Log_Fact $MDRFDir/FiltThresholds_0$elem 8 $ImgFileIn$elem.dat $OutDir/$ImgFileOut$elem.img $NumAngs $Dim $PctEvts >/dev/null` unless -f "$OutDir/$ImgFileOut$elem.img";
   # print "ML_sig_psf_style $MDRFDir/MeanFile_0$elem $MDRFDir/Log_Fact $MDRFDir/FiltThresholds_0$elem 8 $ImgFileIn$elem.dat $OutDir/$ImgFileOut$elem.img $NumAngs $Dim $PctEvts \n";
}

my $cam_length = $#Cams + 1;
print "Preparing projection images for reconstruction...\n";
my $cmd = "proj_prep_psf_style_data $OutDir/ $ImgFilePrep $Dim $NumAngs $cam_length ".join(" ",@Cams);
#my $cmd = "proj_prep_psf_style_data $OutDir/ $ImgFile"."_"."$PctEvts $Dim $NumAngs $cam_length ".join(" ",@Cams);
#print "$cmd\n";
my $tmpFile = "all_".$ComboImgFile;
`$cmd`  unless -f $tmpFile;




